@@ -18,8 +18,9 @@
 
 #include "keyboard_applet.h"
 
+#include <kdebug.h>
 #include <kglobalsettings.h>
-#include <KIconLoader>
+#include <kiconloader.h>
 #include <plasma/theme.h>
 #include <plasma/tooltipmanager.h>
 
@@ -27,14 +28,14 @@
 #include <QtGui/QPixmap>
 #include <QtGui/QGraphicsSceneMouseEvent>
 #include <QtGui/QAction>
-#include <QtGui/QActionGroup>
-#include <QtGui/QCheckBox>
 #include <QtDBus/QtDBus>
 
 #include "x11_helper.h"
 #include "xkb_rules.h"
 #include "keyboard_config.h"
 #include "keyboard_dbus.h"
+#include "layouts_menu.h"
+//#include "utils.h"
 
 
 K_EXPORT_PLASMA_APPLET(keyboard, KeyboardApplet)
@@ -42,26 +43,24 @@ K_EXPORT_PLASMA_APPLET(keyboard, KeyboardApplet)
 KeyboardApplet::KeyboardApplet(QObject *parent, const QVariantList &args):
 	Plasma::Applet(parent, args),
 	xEventNotifier(),
-	actionGroup(NULL),
-	rules(NULL),
-	keyboardConfig(new KeyboardConfig())
+	rules(Rules::readRules()),
+	keyboardConfig(new KeyboardConfig()),
+	layoutsMenu(new LayoutsMenu(*keyboardConfig, *rules, flags))
 {
 	if( ! X11Helper::xkbSupported(NULL) ) {
 		setFailedToLaunch(true, i18n("XKB extension failed to initialize"));
 		return;
 	}
 
-	setHasConfigurationInterface(false);
-
 	resize(48,48);
 
+	setHasConfigurationInterface(false);
+
 	setAspectRatioMode(Plasma::KeepAspectRatio);
 	//setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
 	setBackgroundHints(DefaultBackground);
 	connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
 
-	rules = Rules::readRules();
-
     QDBusConnection dbus = QDBusConnection::sessionBus();
     dbus.connect(QString(), KEYBOARD_DBUS_OBJECT_PATH, KEYBOARD_DBUS_SERVICE_NAME, KEYBOARD_DBUS_CONFIG_RELOAD_MESSAGE, this, SLOT( configChanged() ));
 }
@@ -71,7 +70,7 @@ KeyboardApplet::~KeyboardApplet()
     QDBusConnection dbus = QDBusConnection::sessionBus();
     dbus.disconnect(QString(), KEYBOARD_DBUS_OBJECT_PATH, KEYBOARD_DBUS_SERVICE_NAME, KEYBOARD_DBUS_CONFIG_RELOAD_MESSAGE, this, SLOT( configChanged() ));
 
-    delete actionGroup;
+    delete layoutsMenu;
 	delete rules;
 }
 
@@ -140,6 +139,7 @@ void KeyboardApplet::paintInterface(QPainter *p, const QStyleOptionGraphicsItem
 {
 	p->setRenderHint(QPainter::SmoothPixmapTransform);
 	p->setRenderHint(QPainter::Antialiasing);
+	//p->setBrush(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
 
 	LayoutUnit layoutUnit = X11Helper::getCurrentLayout();
 	if( layoutUnit.isEmpty() )
@@ -168,7 +168,11 @@ void KeyboardApplet::paintInterface(QPainter *p, const QStyleOptionGraphicsItem
 		}
 		font.setPixelSize(fontSize);
 		p->setFont(font);
-		p->drawText(contentsRect, Qt::AlignCenter | Qt::AlignHCenter, shortText);
+		p->drawText(contentsRect, Qt::AlignCenter, shortText);
+
+//		QPixmap pixmap = Utils::shadowText(shortText, font, Qt::black, Qt::white, QPoint(), 4);
+//		p->drawPixmap(contentsRect, pixmap);
+
 		p->restore();
 	}
 }
@@ -181,31 +185,6 @@ void KeyboardApplet::mousePressEvent ( QGraphicsSceneMouseEvent * event )
 	event->ignore();
 }
 
-void KeyboardApplet::actionTriggered(QAction* action)
-{
-//	kDebug() << "actionTriggerd" << action->data().toString();
-	X11Helper::setLayout(LayoutUnit(action->data().toString()));
-}
-
-QList<QAction*> KeyboardApplet::contextualActions()
-{
-	if( actionGroup ) {
-		disconnect(actionGroup, SIGNAL(triggered(QAction*)), this, SLOT(actionTriggered(QAction*)));
-		delete actionGroup;
-	}
-	actionGroup = new QActionGroup(this);
-	QList<LayoutUnit> layouts = X11Helper::getLayoutsList();
-	foreach(const LayoutUnit& layout, layouts) {
-		QAction* action;
-		QString menuText = Flags::getLongText(layout, rules);
-		action = new QAction(getFlag(layout.layout), menuText, actionGroup);
-		action->setData(layout.toString());
-		actionGroup->addAction(action);
-	}
-	connect(actionGroup, SIGNAL(triggered(QAction*)), this, SLOT(actionTriggered(QAction*)));
-	return actionGroup->actions();
-}
-
 void KeyboardApplet::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::FormFactorConstraint) {
@@ -220,13 +199,7 @@ void KeyboardApplet::constraintsEvent(Plasma::Constraints constraints)
     }
 }
 
-//void KeyboardApplet::createConfigurationInterface(KConfigDialog *parent)
-//{
-////	Applet::createConfigurationInterface(parent);
-//	QWidget* widget = new QWidget();
-////	QCheckBox* showFlagCheckbox =
-//			new QCheckBox(i18n("Show Flag"), widget);
-//	parent->addPage(widget, i18n("Indicator Options"));
-//	//TODO: apply config
-//}
-
+QList<QAction*> KeyboardApplet::contextualActions()
+{
+	return layoutsMenu->contextualActions();
+}
