@@ -69,11 +69,16 @@ bool X11Helper::xkbSupported(int* xkbOpcode)
 void X11Helper::switchToNextLayout()
 {
 	int size = getLayoutsList().size();	//TODO: could optimize a bit as we don't need the layouts - just count
-	int group = X11Helper::getGroup() + 1;
+	int group = (X11Helper::getGroup() + 1) % size;
+	X11Helper::setGroup(group);
+}
+
+void X11Helper::scrollLayouts(int delta)
+{
+	int size = getLayoutsList().size();	//TODO: could optimize a bit as we don't need the layouts - just count
+	int group = X11Helper::getGroup() + delta;
+	group = group < 0 ? size - ((-group) % size) : group % size;
 
-	if( group >= size ) {
-		group = 0;
-	}
 	X11Helper::setGroup(group);
 }
 
@@ -119,6 +124,26 @@ LayoutUnit X11Helper::getCurrentLayout()
 	return LayoutUnit();
 }
 
+LayoutSet X11Helper::getCurrentLayouts()
+{
+	LayoutSet layoutSet;
+
+	QList<LayoutUnit> currentLayouts = getLayoutsList();
+	layoutSet.layouts = currentLayouts;
+
+	unsigned int group = X11Helper::getGroup();
+	if( group < (unsigned int)currentLayouts.size() ) {
+		layoutSet.currentLayout = currentLayouts[group];
+	}
+	else {
+		qWarning() << "Current group number" << group << "is outside of current layout list" << getLayoutsListAsString(currentLayouts);
+		layoutSet.currentLayout = LayoutUnit();
+	}
+
+	return layoutSet;
+}
+
+
 //static QString addNum(const QString& str, int n)
 //{
 //    QString format("%1%2");
@@ -221,27 +246,30 @@ bool X11Helper::getGroupNames(Display* display, XkbConfig* xkbConfig, FetchType
 		return false;
 	}
 
-	QStringList layouts = names[2].split(OPTIONS_SEPARATOR);
-	QStringList variants = names[3].split(OPTIONS_SEPARATOR);
+	if( fetchType == ALL || fetchType == LAYOUTS_ONLY ) {
+		QStringList layouts = names[2].split(OPTIONS_SEPARATOR);
+		QStringList variants = names[3].split(OPTIONS_SEPARATOR);
 
-	for(int ii=0; ii<layouts.count(); ii++) {
-		xkbConfig->layouts << (layouts[ii] != NULL ? layouts[ii] : "");
-		xkbConfig->variants << (ii < variants.count() && variants[ii] != NULL ? variants[ii] : "");
+		for(int ii=0; ii<layouts.count(); ii++) {
+			xkbConfig->layouts << (layouts[ii] != NULL ? layouts[ii] : "");
+			xkbConfig->variants << (ii < variants.count() && variants[ii] != NULL ? variants[ii] : "");
+		}
+		qDebug() << "Fetched layout groups from X server:"
+				<< "\tlayouts:" << xkbConfig->layouts
+				<< "\tvariants:" << xkbConfig->variants;
 	}
-	qDebug() << "Fetched layout groups from X server:"
-			<< "\tlayouts:" << xkbConfig->layouts
-			<< "\tvariants:" << xkbConfig->variants;
 
-	if( fetchType == ALL ) {
+	if( fetchType == ALL || fetchType == MODEL_ONLY ) {
 		xkbConfig->keyboardModel = (names[1] != NULL ? names[1] : "");
 		qDebug() << "Fetched keyboard model from X server:" << xkbConfig->keyboardModel;
+	}
 
+	if( fetchType == ALL ) {
 		if( names.count() >= 5 ) {
 			QString options = (names[4] != NULL ? names[4] : "");
 			xkbConfig->options = options.split(OPTIONS_SEPARATOR);
 			qDebug() << "Fetched xkbOptions from X server:" << options;
 		}
-
 	}
 
 	XFree(prop_data);
