@@ -45,6 +45,8 @@ OutputConfig::OutputConfig(QWidget *parent, RandROutput *output, OutputConfigLis
 	connect(m_output, SIGNAL(outputChanged(RROutput, int)),
 	        this,     SLOT(outputChanged(RROutput, int)));
 		  
+	load();
+
 	connect(sizeCombo,    SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
 	connect(refreshCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
 	connect(orientationCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
@@ -63,7 +65,6 @@ OutputConfig::OutputConfig(QWidget *parent, RandROutput *output, OutputConfigLis
 	updatePositionListTimer.setSingleShot( true );
 	connect( &updatePositionListTimer, SIGNAL( timeout()), SLOT( updatePositionListDelayed()));
 
-	load();
 }
 
 OutputConfig::~OutputConfig()
@@ -294,6 +295,9 @@ void OutputConfig::updatePositionList(void)
 
 void OutputConfig::updatePositionListDelayed()
 {
+	disconnect(positionCombo,    SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
+	disconnect(positionOutputCombo,    SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
+
 	bool enable = !resolution().isEmpty();
 	positionCombo->setEnabled( enable );
 	positionLabel->setEnabled( enable );
@@ -338,6 +342,9 @@ void OutputConfig::updatePositionListDelayed()
 		if(index != -1)
 			positionOutputCombo->setCurrentIndex(index);
 	}*/
+
+	connect(positionCombo,    SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
+	connect(positionOutputCombo,    SIGNAL(currentIndexChanged(int)), this, SLOT(setConfigDirty()));
 }
 
 void OutputConfig::updateRotationList(void)
@@ -376,9 +383,19 @@ void OutputConfig::updateSizeList(void)
 		sizeCombo->addItem( sizeDesc, s );
 	}
 	
-	int index = sizeCombo->findData( m_output->rect().size() );
+	int index = -1;
+
+    // if output is rotated 90 or 270 degrees, swap width and height before searching in combobox data
+    // otherwise 90 or 270 degrees rotated outputs will be set as "Disabled" in GUI
+	if (m_output->rotation() == RandR::Rotate90 || m_output->rotation() == RandR::Rotate270)
+		index = sizeCombo->findData( QSize(m_output->rect().height(), m_output->rect().width()) );
+	else
+		index = sizeCombo->findData( m_output->rect().size() );
+
 	if (index != -1)
 		sizeCombo->setCurrentIndex( index );
+    else
+        kDebug() << "Output size cannot be matched!";
 
 	index = refreshCombo->findData(m_output->refreshRate());
 	if (index != -1)
