@@ -31,6 +31,8 @@
 #include <kglobalsettings.h>
 #include <kmessagebox.h>
 #include <kprocess.h>
+#include <kshell.h>
+#include <qmenu.h>
 
 RandRConfig::RandRConfig(QWidget *parent, RandRDisplay *display)
 	: QWidget(parent), Ui::RandRConfigBase()
@@ -54,6 +56,12 @@ RandRConfig::RandRConfig(QWidget *parent, RandRDisplay *display)
 	identifyTimer.setSingleShot( true );
 	compressUpdateViewTimer.setSingleShot( true );
 
+	connect( saveAsDefaultButton, SIGNAL( clicked()), SLOT( saveStartup()));
+	QMenu* saveMenu = new QMenu(saveAsDefaultButton);
+	saveMenu->addAction(i18n("Save as Default"),this, SLOT(saveStartup()));
+	saveMenu->addAction(i18n("Reset"),this, SLOT(disableStartup()));
+	saveAsDefaultButton->setMenu(saveMenu);
+
 	// create the container for the settings widget
 	QHBoxLayout *layout = new QHBoxLayout(outputList);
 	layout->setSpacing(0);
@@ -63,6 +71,17 @@ RandRConfig::RandRConfig(QWidget *parent, RandRDisplay *display)
 						  QSizePolicy::Minimum);
 	layout->addWidget(m_container);
 
+#ifdef HAS_RANDR_1_3
+        if (RandR::has_1_3)
+        {
+            primaryDisplaySelector->setVisible(true);
+        }
+        else
+#endif //HAS_RANDR_1_3
+        {
+            primaryDisplaySelector->setVisible(false);
+        }
+
 	// create the scene
 	m_scene = new QGraphicsScene(m_display->currentScreen()->rect());	
 	screenView->setScene(m_scene);
@@ -90,6 +109,16 @@ void RandRConfig::load(void)
 	m_configs.clear(); // objects deleted above
 	
 	OutputMap outputs = m_display->currentScreen()->outputs();
+#ifdef HAS_RANDR_1_3
+	RandROutput *primary = m_display->currentScreen()->primaryOutput();
+	if (RandR::has_1_3)
+	{
+		// disconnect while we repopulate the combo box
+		disconnect(primaryDisplayBox, SIGNAL(currentIndexChanged(int)), this, SLOT(slotChanged()));
+		primaryDisplayBox->clear();
+		primaryDisplayBox->addItem(i18nc("No display selected", "None"));
+	}
+#endif //HAS_RANDR_1_3
 
 	// FIXME: adjust it to run on a multi screen system
 	CollapsibleWidget *w;
@@ -119,7 +148,24 @@ void RandRConfig::load(void)
 
 		connect(config, SIGNAL(updateView()), this, SLOT(slotUpdateView()));
 		connect(config, SIGNAL(optionChanged()), this, SLOT(slotChanged()));
-	}		    
+
+#ifdef HAS_RANDR_1_3
+		if (RandR::has_1_3 && output->isConnected())
+		{
+			primaryDisplayBox->addItem(output->name(), QVariant::fromValue(output->id()));
+			if (primary == output)
+			{
+				primaryDisplayBox->setCurrentIndex(primaryDisplayBox->count()-1);
+			}
+		}
+#endif //HAS_RANDR_1_3
+	}
+#ifdef HAS_RANDR_1_3
+	if (RandR::has_1_3)
+	{
+		connect(primaryDisplayBox, SIGNAL(currentIndexChanged(int)), this, SLOT(slotChanged()));
+	}
+#endif //HAS_RANDR_1_3
 	slotUpdateView();
 }
 
@@ -196,6 +242,22 @@ void RandRConfig::apply()
 			output->slotDisable();
 		}
 	}
+#ifdef HAS_RANDR_1_3
+	if (RandR::has_1_3)
+	{
+		int primaryOutputIndex = primaryDisplayBox->currentIndex();
+		RandRScreen *screen = m_display->currentScreen();
+		if (primaryOutputIndex > 0)
+		{
+			QVariant output = primaryDisplayBox->itemData(primaryOutputIndex);
+			screen->proposePrimaryOutput(screen->output(output.value<RROutput>()));
+		}
+		else
+		{
+			screen->proposePrimaryOutput(0);
+		}
+	}
+#endif //HAS_RANDR_1_3
 	m_display->applyProposed();
 	update();
 }
@@ -214,6 +276,43 @@ void RandRConfig::update()
 	emit changed(false);
 }
 
+void RandRConfig::saveStartup()
+{
+	if (!m_display->isValid())
+		return;
+	KConfig config("krandrrc");
+	m_display->saveStartup(config);
+#ifdef HAS_RANDR_1_3
+	if (RandR::has_1_3)
+	{
+		// Add setting the primary screen to the list of commands
+		KConfigGroup group = config.group("Display");
+		QStringList commands = group.readEntry("StartupCommands").split("\n");
+		int primaryOutputIndex = primaryDisplayBox->currentIndex();
+		if (primaryOutputIndex > 0)
+		{
+			QString primaryOutput = primaryDisplayBox->itemText(primaryOutputIndex);
+			commands += QString("xrandr --output %1 --primary")
+			    .arg( KShell::quoteArg( primaryOutput ));
+		}
+		else
+			commands += "xrandr --noprimary";
+		group.writeEntry("StartupCommands",commands.join("\n"));
+	}
+#endif //HAS_RANDR_1_3
+	KMessageBox::information( window(), i18n( "Configuration has been set as the desktop default." ));
+}
+
+void RandRConfig::disableStartup()
+{
+	if (!m_display->isValid())
+		return;
+	KConfig config("krandrrc");
+	m_display->disableStartup(config);
+	KMessageBox::information( window(), i18n( "Default desktop setup has been reset." ));
+}
+
+
 bool RandRConfig::eventFilter(QObject *obj, QEvent *event)
 {
 	if ( obj == screenView && event->type() == QEvent::Resize ) {
@@ -337,3 +436,4 @@ void RandRConfig::insufficientVirtualSize()
 
 #include "randrconfig.moc"
 
+// vim:noet:sts=8:sw=8:
