@@ -729,7 +729,7 @@ static const char def_session1[] =
 "    xsess_tmp=";
 static const char def_session2[] =
 "\n"
-"    $SHELL -c \"if (-f /etc/csh.login) source /etc/csh.login; if (-f ~/.login) source ~/.login; /bin/sh -c export -p >! $xsess_tmp\"\n"
+"    $SHELL -c \"if (-f /etc/csh.login) source /etc/csh.login; if (-f ~/.login) source ~/.login; /bin/sh -c 'export -p' >! $xsess_tmp\"\n"
 "    . $xsess_tmp\n"
 "    rm -f $xsess_tmp\n"
 "    ;;\n"
@@ -742,6 +742,11 @@ static const char def_session2[] =
 "[ -f /etc/xprofile ] && . /etc/xprofile\n"
 "[ -f $HOME/.xprofile ] && . $HOME/.xprofile\n"
 "\n"
+"for i in /etc/X11/Xresources/*; do\n"
+"  [ -f $i ] && xrdb -merge $i\n"
+"done\n"
+"[ -f $HOME/.Xresources ] && xrdb -merge $HOME/.Xresources\n"
+"\n"
 "case $session in\n"
 "  \"\")\n"
 "    exec xmessage -center -buttons OK:0 -default OK \"Sorry, $DESKTOP_SESSION is no valid session.\"\n"
