@@ -31,16 +31,21 @@
 
 #include "oxygenwindowmanager.h"
 #include "oxygenwindowmanager.moc"
+#include "oxygenpropertynames.h"
 #include "oxygenstyleconfigdata.h"
 
-
 #include <QtGui/QApplication>
+#include <QtGui/QComboBox>
+#include <QtGui/QDialog>
 #include <QtGui/QDockWidget>
 #include <QtGui/QGroupBox>
 #include <QtGui/QLabel>
 #include <QtGui/QListView>
+#include <QtGui/QMainWindow>
 #include <QtGui/QMenuBar>
 #include <QtGui/QMouseEvent>
+#include <QtGui/QProgressBar>
+#include <QtGui/QStatusBar>
 #include <QtGui/QStyle>
 #include <QtGui/QStyleOptionGroupBox>
 #include <QtGui/QTabBar>
@@ -68,7 +73,7 @@ namespace Oxygen
         QObject( parent ),
         enabled_( true ),
         useWMMoveResize_( true ),
-        dragMode_( OxygenStyleConfigData::WD_FULL ),
+        dragMode_( StyleConfigData::WD_FULL ),
         dragDistance_( KGlobalSettings::dndEventDelay() ),
         dragDelay_( QApplication::startDragTime() ),
         dragAboutToStart_( false ),
@@ -87,9 +92,9 @@ namespace Oxygen
     void WindowManager::initialize( void )
     {
 
-        setEnabled( OxygenStyleConfigData::windowDragEnabled() );
-        setDragMode( OxygenStyleConfigData::windowDragMode() );
-        setUseWMMoveResize( OxygenStyleConfigData::useWMMoveResize() );
+        setEnabled( StyleConfigData::windowDragEnabled() );
+        setDragMode( StyleConfigData::windowDragMode() );
+        setUseWMMoveResize( StyleConfigData::useWMMoveResize() );
 
         setDragDistance( KGlobalSettings::dndEventDelay() );
         setDragDelay( QApplication::startDragTime() );
@@ -141,7 +146,7 @@ namespace Oxygen
         whiteList_.insert( ExceptionId( "ViewSliders@kmix" ) );
         whiteList_.insert( ExceptionId( "Sidebar_Widget@konqueror" ) );
 
-        foreach( const QString& exception, OxygenStyleConfigData::windowDragWhiteList() )
+        foreach( const QString& exception, StyleConfigData::windowDragWhiteList() )
         {
             ExceptionId id( exception );
             if( !id.className().isEmpty() )
@@ -155,7 +160,8 @@ namespace Oxygen
 
         blackList_.clear();
         blackList_.insert( ExceptionId( "CustomTrackView@kdenlive" ) );
-        foreach( const QString& exception, OxygenStyleConfigData::windowDragBlackList() )
+        blackList_.insert( ExceptionId( "MuseScore" ) );
+        foreach( const QString& exception, StyleConfigData::windowDragBlackList() )
         {
             ExceptionId id( exception );
             if( !id.className().isEmpty() )
@@ -312,13 +318,13 @@ namespace Oxygen
 
         // all accepted default types
         if(
-            ( widget->inherits( "QDialog" ) && widget->isWindow() ) ||
-            ( widget->inherits( "QMainWindow" ) && widget->isWindow() ) ||
-            widget->inherits( "QGroupBox" ) ||
-            widget->inherits( "QMenuBar" ) ||
-            widget->inherits( "QTabBar" ) ||
-            widget->inherits( "QStatusBar" ) ||
-            widget->inherits( "QToolBar" ) )
+            ( qobject_cast<QDialog*>( widget ) && widget->isWindow() ) ||
+            ( qobject_cast<QMainWindow*>( widget ) && widget->isWindow() ) ||
+            qobject_cast<QGroupBox*>( widget ) ||
+            qobject_cast<QMenuBar*>( widget ) ||
+            qobject_cast<QTabBar*>( widget ) ||
+            qobject_cast<QStatusBar*>( widget ) ||
+            qobject_cast<QToolBar*>( widget ) )
         { return true; }
 
         if( widget->inherits( "KScreenSaver" ) && widget->inherits( "KCModule" ) )
@@ -359,7 +365,7 @@ namespace Oxygen
             QWidget* parent = label->parentWidget();
             while( parent )
             {
-                if( parent->inherits( "QStatusBar" ) ) return true;
+                if( qobject_cast<QStatusBar*>( parent ) ) return true;
                 parent = parent->parentWidget();
             }
         }
@@ -372,6 +378,10 @@ namespace Oxygen
     bool WindowManager::isBlackListed( QWidget* widget )
     {
 
+        // check against noAnimations propery
+        QVariant propertyValue( widget->property( PropertyNames::noWindowGrab ) );
+        if( propertyValue.isValid() && propertyValue.toBool() ) return true;
+
         // list-based blacklisted widgets
         QString appName( qApp->applicationName() );
         foreach( const ExceptionId& id, blackList_ )
@@ -394,7 +404,6 @@ namespace Oxygen
     bool WindowManager::isWhiteListed( QWidget* widget ) const
     {
 
-        //foreach( const QString& className, blackList_ )
         QString appName( qApp->applicationName() );
         foreach( const ExceptionId& id, whiteList_ )
         {
@@ -442,14 +451,14 @@ namespace Oxygen
         even if mousePress/Move has been passed to the parent
         */
         if( child && (
-          child->inherits( "QComboBox" ) ||
-          child->inherits( "QProgressBar" ) ) )
+          qobject_cast<QComboBox*>(child ) ||
+          qobject_cast<QProgressBar*>( child ) ) )
         { return false; }
 
         // tool buttons
         if( QToolButton* toolButton = qobject_cast<QToolButton*>( widget ) )
         {
-            if( dragMode() == OxygenStyleConfigData::WD_MINIMAL && !qobject_cast<QToolBar*>(widget->parentWidget() ) ) return false;
+            if( dragMode() == StyleConfigData::WD_MINIMAL && !qobject_cast<QToolBar*>(widget->parentWidget() ) ) return false;
             return toolButton->autoRaise() && !toolButton->isEnabled();
         }
 
@@ -476,9 +485,9 @@ namespace Oxygen
         in MINIMAL mode, anything that has not been already accepted
         and does not come from a toolbar is rejected
         */
-        if( dragMode() == OxygenStyleConfigData::WD_MINIMAL )
+        if( dragMode() == StyleConfigData::WD_MINIMAL )
         {
-            if( widget->inherits( "QToolBar" ) ) return true;
+            if( qobject_cast<QToolBar*>( widget ) ) return true;
             else return false;
         }
 
