@@ -41,7 +41,7 @@ KWIN_EFFECT( boxswitch, BoxSwitchEffect )
 BoxSwitchEffect::BoxSwitchEffect()
     : mActivated( 0 )
     , mMode( 0 )
-    , thumbnailFrame( EffectFrame::Styled )
+    , thumbnailFrame( effects->effectFrame( EffectFrameStyled ) )
     , selected_window( 0 )
     , painting_desktop( 0 )
     , animation( false )
@@ -56,8 +56,8 @@ BoxSwitchEffect::BoxSwitchEffect()
     {
     text_font.setBold( true );
     text_font.setPointSize( 12 );
-    thumbnailFrame.setFont( text_font );
-    thumbnailFrame.setAlignment( Qt::AlignBottom | Qt::AlignHCenter );
+    thumbnailFrame->setFont( text_font );
+    thumbnailFrame->setAlignment( Qt::AlignBottom | Qt::AlignHCenter );
 
     highlight_margin = 10;
     reconfigure( ReconfigureAll );
@@ -65,6 +65,7 @@ BoxSwitchEffect::BoxSwitchEffect()
 
 BoxSwitchEffect::~BoxSwitchEffect()
     {
+    delete thumbnailFrame;
     }
 
 void BoxSwitchEffect::reconfigure( ReconfigureFlags )
@@ -150,15 +151,13 @@ void BoxSwitchEffect::paintScreen( int mask, QRegion region, ScreenPaintData& da
             {
             if( !painting_desktop )
                 {
-                thumbnailFrame.render( region );
+                thumbnailFrame->setSelection( desktops[ selected_desktop ]->area );
+                thumbnailFrame->render( region );
 
                 QHash< int, ItemInfo* >::const_iterator i;
                 for( i = desktops.constBegin(); i != desktops.constEnd(); ++i )
                     {
                     painting_desktop = i.key();
-                    if( painting_desktop == selected_desktop )
-                        paintHighlight( desktops[ painting_desktop ]->area ); //effects->desktopName( painting_desktop )
-
                     paintDesktopThumbnail( painting_desktop );
                     }
                 painting_desktop = 0;
@@ -169,13 +168,16 @@ void BoxSwitchEffect::paintScreen( int mask, QRegion region, ScreenPaintData& da
 
 void BoxSwitchEffect::paintWindowsBox(const QRegion& region)
     {
-    thumbnailFrame.render( region );
+    if( (mAnimateSwitch && !mProxyActivated) || (mProxyActivated && mProxyAnimateSwitch) )
+        thumbnailFrame->setSelection( highlight_area );
+    else
+        thumbnailFrame->setSelection( windows[ selected_window ]->area );
+    thumbnailFrame->render( region );
 
     if( (mAnimateSwitch && !mProxyActivated) || (mProxyActivated && mProxyAnimateSwitch) )
         {
         // HACK: PaintClipper is used because window split is somehow wrong if the height is greater than width
         PaintClipper::push( frame_area );
-        paintHighlight( highlight_area );
         QHash< EffectWindow*, ItemInfo* >::const_iterator i;
         for( i = windows.constBegin(); i != windows.constEnd(); ++i )
             {
@@ -189,8 +191,6 @@ void BoxSwitchEffect::paintWindowsBox(const QRegion& region)
         QHash< EffectWindow*, ItemInfo* >::const_iterator i;
         for( i = windows.constBegin(); i != windows.constEnd(); ++i )
             {
-            if( i.key() == selected_window )
-                paintHighlight( i.value()->area );
             paintWindowThumbnail( i.key() );
             paintWindowIcon( i.key() );
             }
@@ -438,7 +438,7 @@ void BoxSwitchEffect::tabBoxUpdated()
                 effects->addRepaint( desktops.value( selected_desktop )->area );
             selected_desktop = effects->currentTabBoxDesktop();
             if( !mProxyActivated || mProxyShowText )
-                thumbnailFrame.setText( effects->desktopName( selected_desktop ));
+                thumbnailFrame->setText( effects->desktopName( selected_desktop ));
             if( desktops.contains( selected_desktop ))
                 effects->addRepaint( desktops.value( selected_desktop )->area );
             effects->addRepaint( text_area );
@@ -474,7 +474,7 @@ void BoxSwitchEffect::setActive()
         original_desktops = effects->currentTabBoxDesktopList();
         selected_desktop = effects->currentTabBoxDesktop();
         if( !mProxyActivated || mProxyShowText )
-            thumbnailFrame.setText( effects->desktopName( selected_desktop ));
+            thumbnailFrame->setText( effects->desktopName( selected_desktop ));
         }
     calculateFrameSize();
     calculateItemSizes();
@@ -530,7 +530,7 @@ void BoxSwitchEffect::setInactive()
         qDeleteAll( windows );
         desktops.clear();
         }
-    thumbnailFrame.free();
+    thumbnailFrame->free();
     effects->addRepaint( frame_area );
     frame_area = QRect();
     }
@@ -543,7 +543,7 @@ void BoxSwitchEffect::setSelectedWindow( EffectWindow* w )
         }
     selected_window = w;
     if( selected_window && ( !mProxyActivated || mProxyShowText ) )
-        thumbnailFrame.setText( selected_window->caption() );
+        thumbnailFrame->setText( selected_window->caption() );
     if( elevate_window && w )
         {
         effects->setElevatedWindow( selected_window, true );
@@ -612,7 +612,7 @@ void BoxSwitchEffect::calculateFrameSize()
     text_area.moveTo( frame_area.x(),
                       frame_area.y() + item_max_size.height() + separator_height);
 
-    thumbnailFrame.setGeometry( frame_area );
+    thumbnailFrame->setGeometry( frame_area );
     }
 
 void BoxSwitchEffect::calculateItemSizes()
@@ -698,7 +698,7 @@ void BoxSwitchEffect::calculateItemSizes()
                 EffectWindow* w = ordered_windows.at( i );
                 windows[ w ] = new ItemInfo();
 
-                windows[ w ]->iconFrame = new EffectFrame( EffectFrame::Unstyled, false );
+                windows[ w ]->iconFrame = effects->effectFrame( EffectFrameUnstyled, false );
                 windows[ w ]->iconFrame->setAlignment( Qt::AlignTop | Qt::AlignLeft );
                 windows[ w ]->iconFrame->setIcon( w->icon() );
 
@@ -736,7 +736,7 @@ void BoxSwitchEffect::calculateItemSizes()
                 EffectWindow* w = original_windows.at( i );
                 windows[ w ] = new ItemInfo();
 
-                windows[ w ]->iconFrame = new EffectFrame( EffectFrame::Unstyled, false );
+                windows[ w ]->iconFrame = effects->effectFrame( EffectFrameUnstyled, false );
                 windows[ w ]->iconFrame->setAlignment( Qt::AlignTop | Qt::AlignLeft );
                 windows[ w ]->iconFrame->setIcon( w->icon() );
 
@@ -763,38 +763,6 @@ void BoxSwitchEffect::calculateItemSizes()
         }
     }
 
-void BoxSwitchEffect::paintHighlight( QRect area )
-    {
-#ifdef KWIN_HAVE_OPENGL_COMPOSITING
-    if( effects->compositingType() == OpenGLCompositing )
-        {
-        glPushAttrib( GL_CURRENT_BIT );
-        glColor4f( color_highlight.redF(), color_highlight.greenF(), color_highlight.blueF(), color_highlight.alphaF());
-        renderRoundBox( area, 6 );
-        glPopAttrib();
-        }
-#endif
-#ifdef KWIN_HAVE_XRENDER_COMPOSITING
-    if( effects->compositingType() == XRenderCompositing )
-        {
-        Pixmap pixmap = XCreatePixmap( display(), rootWindow(),
-            area.width(), area.height(), 32 );
-        XRenderPicture pic( pixmap, 32 );
-        XFreePixmap( display(), pixmap );
-        XRenderColor col;
-        col.alpha = int( color_highlight.alphaF() * 0xffff );
-        col.red = int( color_highlight.redF() * color_highlight.alphaF() * 0xffff );
-        col.green = int( color_highlight.greenF() * color_highlight.alphaF() * 0xffff );
-        col.blue = int( color_highlight.blueF() * color_highlight.alphaF() * 0xffff );
-        XRenderFillRectangle( display(), PictOpSrc, pic, &col, 0, 0,
-            area.width(), area.height());
-        XRenderComposite( display(), color_highlight.alphaF() != 1.0 ? PictOpOver : PictOpSrc,
-            pic, None, effects->xrenderBufferPicture(),
-            0, 0, 0, 0, area.x(), area.y(), area.width(), area.height());
-        }
-#endif
-    }
-
 void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
     {
     if( !windows.contains( w ))
@@ -847,7 +815,7 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
 
         // paint one part of the thumbnail
         effects->paintWindow( w,
-            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
+            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS,
             windows[ w ]->thumbnail, data );
 
         QRect secondThumbnail;
@@ -896,7 +864,7 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
             secondThumbnail.adjusted( highlight_margin, highlight_margin, -highlight_margin, -highlight_margin ),
             Qt::KeepAspectRatio );
         effects->paintWindow( w,
-            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
+            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS,
             windows[ w ]->thumbnail, data );
         }
     else if( ( windows.size() % 2 == 0 ) && ( w == right_window ) )
@@ -948,7 +916,7 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
         // left quads are painted on right side of frame
         data.quads = leftQuads;
         effects->drawWindow( w,
-            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
+            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS,
             windows[ w ]->thumbnail, data );
 
         // right quads are painted on left side of frame
@@ -962,16 +930,13 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
             secondThumbnail.adjusted( highlight_margin, highlight_margin, -highlight_margin, -highlight_margin ),
             Qt::KeepAspectRatio );
         effects->drawWindow( w,
-            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
+            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS,
             windows[ w ]->thumbnail, data );        
         }
     else
         {
-        int mask = PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED;
-        if (!mAnimateSwitch)
-            mask |= PAINT_WINDOW_LANCZOS;
         effects->drawWindow( w,
-                 mask,
+                 PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS,
                 windows[ w ]->thumbnail, data );
         }
     }
@@ -1086,7 +1051,7 @@ void BoxSwitchEffect::activateFromProxy( int mode, bool animate, bool showText,
         mProxyAnimateSwitch = animate;
         mProxyShowText = showText;
         mPositioningFactor = positioningFactor;
-        thumbnailFrame.setText(" ");
+        thumbnailFrame->setText(" ");
         if( ( mode == TabBoxWindowsMode ) || ( mode == TabBoxWindowsAlternativeMode ) )
             {
             if( effects->currentTabBoxWindowList().count() > 0 )
