@@ -21,7 +21,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "kwinglutils.h"
 
 #ifdef KWIN_HAVE_OPENGL
-
 #include "kwinglobals.h"
 #include "kwineffects.h"
 
@@ -38,7 +37,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #define MAKE_GL_VERSION(major, minor, release)  ( ((major) << 16) | ((minor) << 8) | (release) )
 
-
 namespace KWin
 {
 // Variables
@@ -83,6 +81,7 @@ void initGL()
     GLTexture::initStatic();
     GLShader::initStatic();
     GLRenderTarget::initStatic();
+    GLVertexBuffer::initStatic();
     }
 
 bool hasGLVersion(int major, int minor, int release)
@@ -175,10 +174,20 @@ void renderGLGeometry( const QRegion& region, int count,
         }
 
     // Clip using scissoring
-    PaintClipper pc( region );
-    for( PaintClipper::Iterator iterator;
-         !iterator.isDone();
-         iterator.next())
+    if( !effects->isRenderTargetBound() )
+        {
+        PaintClipper pc( region );
+        for( PaintClipper::Iterator iterator;
+            !iterator.isDone();
+            iterator.next())
+            {
+            if( use_arrays )
+                glDrawArrays( GL_QUADS, 0, count );
+            else
+                renderGLGeometryImmediate( count, vertices, texture, color, dim, stride );
+            }
+        }
+    else
         {
         if( use_arrays )
             glDrawArrays( GL_QUADS, 0, count );
@@ -258,82 +267,6 @@ void addQuadVertices(QVector<float>& verts, float x1, float y1, float x2, float
     verts << x2 << y1;
 }
 
-void renderRoundBox( const QRect& area, float roundness, GLTexture* texture )
-{
-    static GLTexture* circleTexture = 0;
-    if( !texture && !circleTexture )
-        {
-        QString texturefile =  KGlobal::dirs()->findResource("data", "kwin/circle.png");
-        circleTexture = new GLTexture(texturefile);
-        }
-    if( !texture )
-        {
-        texture = circleTexture;
-        }
-
-    glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
-    glEnable( GL_BLEND );
-    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
-
-    glPushMatrix();
-
-    QVector<float> verts, texcoords;
-    verts.reserve(80);
-    texcoords.reserve(80);
-    // center
-    addQuadVertices(verts, area.left() + roundness, area.top() + roundness, area.right() - roundness, area.bottom() - roundness);
-    addQuadVertices(texcoords, 0.5, 0.5, 0.5, 0.5);
-    // sides
-    // left
-    addQuadVertices(verts, area.left(), area.top() + roundness, area.left() + roundness, area.bottom() - roundness);
-    addQuadVertices(texcoords, 0.0, 0.5, 0.5, 0.5);
-    // top
-    addQuadVertices(verts, area.left() + roundness, area.top(), area.right() - roundness, area.top() + roundness);
-    addQuadVertices(texcoords, 0.5, 0.0, 0.5, 0.5);
-    // right
-    addQuadVertices(verts, area.right() - roundness, area.top() + roundness, area.right(), area.bottom() - roundness);
-    addQuadVertices(texcoords, 0.5, 0.5, 1.0, 0.5);
-    // bottom
-    addQuadVertices(verts, area.left() + roundness, area.bottom() - roundness, area.right() - roundness, area.bottom());
-    addQuadVertices(texcoords, 0.5, 0.5, 0.5, 1.0);
-    // corners
-    // top-left
-    addQuadVertices(verts, area.left(), area.top(), area.left() + roundness, area.top() + roundness);
-    addQuadVertices(texcoords, 0.0, 0.0, 0.5, 0.5);
-    // top-right
-    addQuadVertices(verts, area.right() - roundness, area.top(), area.right(), area.top() + roundness);
-    addQuadVertices(texcoords, 0.5, 0.0, 1.0, 0.5);
-    // bottom-left
-    addQuadVertices(verts, area.left(), area.bottom() - roundness, area.left() + roundness, area.bottom());
-    addQuadVertices(texcoords, 0.0, 0.5, 0.5, 1.0);
-    // bottom-right
-    addQuadVertices(verts, area.right() - roundness, area.bottom() - roundness, area.right(), area.bottom());
-    addQuadVertices(texcoords, 0.5, 0.5, 1.0, 1.0);
-
-    texture->bind();
-    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-    // We have two elements per vertex in the verts array
-    int verticesCount = verts.count() / 2;
-    texture->enableNormalizedTexCoords();
-    renderGLGeometry( verticesCount, verts.data(), texcoords.data() );
-    texture->disableNormalizedTexCoords();
-    texture->unbind();
-
-    glPopMatrix();
-    glPopAttrib();
-}
-
-void renderRoundBoxWithEdge( const QRect& area, float roundness )
-{
-    static GLTexture* texture = 0;
-    if( !texture )
-    {
-        QString texturefile =  KGlobal::dirs()->findResource("data", "kwin/circle-edgy.png");
-        texture = new GLTexture(texturefile);
-    }
-    renderRoundBox( area, roundness, texture );
-}
-
 //****************************************
 // GLTexture
 //****************************************
@@ -386,6 +319,7 @@ GLTexture::GLTexture( int width, int height )
 
 GLTexture::~GLTexture()
     {
+    delete m_vbo;
     discard();
     assert( mUnnormalizeActive == 0 );
     assert( mNormalizeActive == 0 );
@@ -401,6 +335,7 @@ void GLTexture::init()
     has_valid_mipmaps = false;
     mUnnormalizeActive = 0;
     mNormalizeActive = 0;
+    m_vbo = 0;
     }
 
 void GLTexture::initStatic()
@@ -499,23 +434,34 @@ void GLTexture::unbind()
 
 void GLTexture::render( QRegion region, const QRect& rect )
     {
-    const float verts[ 4 * 2 ] =
+    if( rect.size() != m_cachedSize )
         {
-        rect.x(), rect.y(),
-        rect.x(), rect.y() + rect.height(),
-        rect.x() + rect.width(), rect.y() + rect.height(),
-        rect.x() + rect.width(), rect.y()
-        };
-    const float texcoords[ 4 * 2 ] =
-        {
-        0, 1, // y needs to be swapped (normalized coords)
-        0, 0,
-        1, 0,
-        1, 1
-        };
-    enableNormalizedTexCoords();
-    renderGLGeometry( region, 4, verts, texcoords );
-    disableNormalizedTexCoords();
+        m_cachedSize = rect.size();
+        QRect r(rect);
+        r.moveTo(0,0);
+        if( !m_vbo )
+            {
+            m_vbo = new GLVertexBuffer( KWin::GLVertexBuffer::Static );
+            }
+        const float verts[ 4 * 2 ] =
+            {   // NOTICE: r.x/y could be replaced by "0", but that would make it unreadable...
+            r.x(), r.y(),
+            r.x(), r.y() + rect.height(),
+            r.x() + rect.width(), r.y(),
+            r.x() + rect.width(), r.y() + rect.height()
+            };
+        const float texcoords[ 4 * 2 ] =
+            {
+            0.0f, y_inverted ? 0.0f : 1.0f, // y needs to be swapped (normalized coords)
+            0.0f, y_inverted ? 1.0f : 0.0f,
+            1.0f, y_inverted ? 0.0f : 1.0f,
+            1.0f, y_inverted ? 1.0f : 0.0f
+            };
+        m_vbo->setData( 4, 2, verts, texcoords );
+        }
+    glTranslatef( rect.x(), rect.y(), 0.0f );
+    m_vbo->render( region, GL_TRIANGLE_STRIP );
+    glTranslatef( -rect.x(), -rect.y(), 0.0f );
     }
 
 void GLTexture::enableUnnormalizedTexCoords()
@@ -971,26 +917,6 @@ bool GLRenderTarget::mSupported = false;
 void GLRenderTarget::initStatic()
     {
     mSupported = hasGLExtension("GL_EXT_framebuffer_object") && glFramebufferTexture2D;
-    if( mSupported )
-        {
-        // some drivers claim to support the extension, but fail to return a complete FBO (see Bug 240956)
-        // generate a FBO and test if it is valid. If it isn't valid there's no need to claim that FBO's are supported
-        int w = displayWidth();
-        int h = displayHeight();
-        if ( !GLTexture::NPOTTextureSupported() )
-            {
-            w = nearestPowerOfTwo( w );
-            h = nearestPowerOfTwo( h );
-            }
-        GLTexture* tex = new GLTexture( w, h );
-        tex->setFilter( GL_LINEAR );
-        tex->setWrapMode( GL_CLAMP_TO_EDGE );
-        GLRenderTarget* target = new GLRenderTarget( tex );
-        if( !target->valid() )
-            mSupported = false;
-        delete target;
-        delete tex;
-        }
     }
 
 GLRenderTarget::GLRenderTarget(GLTexture* color)
@@ -1074,7 +1000,7 @@ static QString formatFramebufferStatus( GLenum status )
             // The read buffer is missing
             return "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER";
         default:
-            return "Unknown (0x" + QString::number(status, 16) + ")";
+            return "Unknown (0x" + QString::number(status, 16) + ')';
         }
     }
 
@@ -1137,6 +1063,175 @@ void GLRenderTarget::initFBO()
     mValid = true;
     }
 
+
+//*********************************
+// GLVertexBufferPrivate
+//*********************************
+class GLVertexBufferPrivate
+    {
+    public:
+        GLVertexBufferPrivate( GLVertexBuffer::UsageHint usageHint )
+            : hint( usageHint )
+            , numberVertices( 0 )
+            , dimension( 2 )
+            {
+            if( GLVertexBufferPrivate::supported )
+                {
+                glGenBuffers( 2, buffers );
+                }
+            }
+        ~GLVertexBufferPrivate()
+            {
+            if( GLVertexBufferPrivate::supported )
+                {
+                glDeleteBuffers( 2, buffers );
+                }
+            }
+        GLVertexBuffer::UsageHint hint;
+        GLuint buffers[2];
+        int numberVertices;
+        int dimension;
+        static bool supported;
+        QVector<float> legacyVertices;
+        QVector<float> legacyTexCoords;
+
+        void legacyPainting( QRegion region, GLenum primitiveMode );
+    };
+bool GLVertexBufferPrivate::supported = false;
+
+void GLVertexBufferPrivate::legacyPainting( QRegion region, GLenum primitiveMode )
+    {
+    // Enable arrays
+    glEnableClientState( GL_VERTEX_ARRAY );
+    glVertexPointer( dimension, GL_FLOAT, 0, legacyVertices.constData() );
+    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+    glTexCoordPointer( 2, GL_FLOAT, 0, legacyTexCoords.constData() );
+
+    // Clip using scissoring
+    PaintClipper pc( region );
+    for( PaintClipper::Iterator iterator;
+        !iterator.isDone();
+        iterator.next())
+        {
+        glDrawArrays( primitiveMode, 0, numberVertices );
+        }
+
+    glDisableClientState( GL_VERTEX_ARRAY );
+    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
+    }
+
+//*********************************
+// GLVertexBuffer
+//*********************************
+GLVertexBuffer::GLVertexBuffer( UsageHint hint )
+    : d( new GLVertexBufferPrivate( hint ) )
+    {
+    }
+
+GLVertexBuffer::~GLVertexBuffer()
+    {
+    delete d;
+    }
+
+void GLVertexBuffer::setData( int numberVertices, int dim, const float* vertices, const float* texcoords )
+    {
+    d->numberVertices = numberVertices;
+    d->dimension = dim;
+    if( !GLVertexBufferPrivate::supported )
+        {
+        // legacy data
+        d->legacyVertices.clear();
+        d->legacyVertices.reserve( numberVertices * dim );
+        for( int i=0; i<numberVertices*dim; ++i)
+            {
+            d->legacyVertices << vertices[i];
+            }
+        d->legacyTexCoords.clear();
+        d->legacyTexCoords.reserve( numberVertices * 2 );
+        for( int i=0; i<numberVertices*2; ++i)
+            {
+            d->legacyTexCoords << texcoords[i];
+            }
+        return;
+        }
+    GLenum hint;
+    switch( d->hint )
+        {
+        case Dynamic:
+            hint = GL_DYNAMIC_DRAW;
+            break;
+        case Static:
+            hint = GL_STATIC_DRAW;
+            break;
+        case Stream:
+            hint = GL_STREAM_DRAW;
+            break;
+        default:
+            // just to make the compiler happy
+            hint = GL_STREAM_DRAW;
+            break;
+        }
+    glEnableClientState( GL_VERTEX_ARRAY );
+    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+    glBindBuffer( GL_ARRAY_BUFFER, d->buffers[ 0 ] );
+    glBufferData( GL_ARRAY_BUFFER, sizeof(GLfloat)*numberVertices*d->dimension, vertices, hint );
+
+    glBindBuffer( GL_ARRAY_BUFFER, d->buffers[ 1 ] );
+    glBufferData( GL_ARRAY_BUFFER, sizeof(GLfloat)*numberVertices*2, texcoords, hint );
+
+    glBindBuffer( GL_ARRAY_BUFFER, 0 );
+
+    glDisableClientState( GL_VERTEX_ARRAY );
+    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
+    }
+
+void GLVertexBuffer::render( GLenum primitiveMode )
+    {
+    render( infiniteRegion(), primitiveMode );
+    }
+
+void GLVertexBuffer::render( const QRegion& region, GLenum primitiveMode )
+    {
+    if( !GLVertexBufferPrivate::supported )
+        {
+        d->legacyPainting( region, primitiveMode );
+        return;
+        }
+    glEnableClientState( GL_VERTEX_ARRAY );
+    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+    glBindBuffer( GL_ARRAY_BUFFER, d->buffers[ 0 ] );
+    glVertexPointer( d->dimension, GL_FLOAT, 0, 0 );
+
+    glBindBuffer( GL_ARRAY_BUFFER, d->buffers[ 1 ] );
+    glTexCoordPointer( 2, GL_FLOAT, 0, 0 );
+
+    // Clip using scissoring
+    PaintClipper pc( region );
+    for( PaintClipper::Iterator iterator;
+        !iterator.isDone();
+        iterator.next())
+        {
+        glDrawArrays( primitiveMode, 0, d->numberVertices );
+        }
+
+    glBindBuffer( GL_ARRAY_BUFFER, 0 );
+
+    glDisableClientState( GL_VERTEX_ARRAY );
+    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
+    }
+
+
+bool GLVertexBuffer::isSupported()
+    {
+    return GLVertexBufferPrivate::supported;
+    }
+
+void GLVertexBuffer::initStatic()
+    {
+    GLVertexBufferPrivate::supported = hasGLExtension( "GL_ARB_vertex_buffer_object" );
+    }
+
+
 } // namespace
 
 #endif
