@@ -32,6 +32,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "rules.h"
 #include "group.h"
 
+#include "scripting/workspaceproxy.h"
+
 namespace KWin
 {
 
@@ -44,6 +46,14 @@ bool Client::manage( Window w, bool isMapped )
     {
     StackingUpdatesBlocker stacking_blocker( workspace() );
 
+    //Scripting call. Does not use a signal/slot mechanism
+    //as ensuring connections was a bit difficult between
+    //so many clients and the workspace
+    SWrapper::WorkspaceProxy* ws_wrap = SWrapper::WorkspaceProxy::instance();
+    if(ws_wrap != 0) {
+	ws_wrap->sl_clientManaging(this);
+    }
+
     grabXServer();
 
     XWindowAttributes attr;
@@ -149,12 +159,15 @@ bool Client::manage( Window w, bool isMapped )
     init_minimize = rules()->checkMinimize( init_minimize, !isMapped );
     noborder = rules()->checkNoBorder( noborder, !isMapped );
 
+    checkActivities();
+
     // Initial desktop placement
     if( session )
         {
         desk = session->desktop;
         if( session->onAllDesktops )
             desk = NET::OnAllDesktops;
+        setOnActivities(session->activities);
         }
     else
         {
@@ -165,6 +178,7 @@ bool Client::manage( Window w, bool isMapped )
             {
             ClientList mainclients = mainClients();
             bool on_current = false;
+            bool on_all = false;
             Client* maincl = NULL;
             // This is slightly duplicated from Placement::placeOnMainWindow()
             for( ClientList::ConstIterator it = mainclients.constBegin();
@@ -176,16 +190,33 @@ bool Client::manage( Window w, bool isMapped )
                 maincl = *it;
                 if( (*it)->isOnCurrentDesktop() )
                     on_current = true;
+                if( (*it)->isOnAllDesktops() )
+                    on_all = true;
                 }
-            if( on_current )
+            if( on_all )
+                desk = NET::OnAllDesktops;
+            else if( on_current )
                 desk = workspace()->currentDesktop();
             else if( maincl != NULL )
                 desk = maincl->desktop();
+
+            if ( maincl )
+                setOnActivities(maincl->activities());
             }
         if( info->desktop() )
             desk = info->desktop(); // Window had the initial desktop property, force it
         if( desktop() == 0 && asn_valid && asn_data.desktop() != 0 )
             desk = asn_data.desktop();
+        if (!isMapped && !noborder && isNormalWindow() && !activitiesDefined) {
+            //a new, regular window, when we're not recovering from a crash,
+            //and it hasn't got an activity. let's try giving it the current one.
+            //TODO: decide whether to keep this before the 4.6 release
+            //TODO: if we are keeping it (at least as an option), replace noborder checking
+            //with a public API for setting windows to be on all activities.
+            //something like KWindowSystem::setOnAllActivities or
+            //KActivityConsumer::setOnAllActivities
+            setOnActivity(Workspace::self()->currentActivity(), true);
+        }
         }
     if( desk == 0 ) // Assume window wants to be visible on the current desktop
         desk = workspace()->currentDesktop();
@@ -311,6 +342,7 @@ bool Client::manage( Window w, bool isMapped )
             foreach( ClientGroup* group, workspace()->clientGroups )
                 if( geom == QRect( group->visible()->pos(), group->visible()->clientSize() ) &&
                     desk == group->visible()->desktop() &&
+                    activities() == group->visible()->activities() &&
                     group->visible()->maximizeMode() != MaximizeFull )
                     {
                     group->add( this, -1, true );
@@ -519,10 +551,32 @@ bool Client::manage( Window w, bool isMapped )
         else
             allow = workspace()->allowClientActivation( this, userTime(), false );
 
-        // If session saving, force showing new windows (i.e. "save file?" dialogs etc.)
-        // also force if activation is allowed
-        if( !isOnCurrentDesktop() && !isMapped && !session && ( allow || workspace()->sessionSaving() ))
-            workspace()->setCurrentDesktop( desktop() );
+        if (!(isMapped || session)) {
+            if (workspace()->sessionSaving()) {
+                /*
+                 * If we get a new window during session saving, we assume it's some 'save file?' dialog
+                 * which the user really needs to see (to know why logout's stalled).
+                 *
+                 * Given the current session management protocol, I can't see a nicer way of doing this.
+                 * Someday I'd like to see a protocol that tells the windowmanager who's doing SessionInteract.
+                 */
+                needsSessionInteract = true;
+                //show the parent too
+                ClientList mainclients = mainClients();
+                for( ClientList::ConstIterator it = mainclients.constBegin();
+                    it != mainclients.constEnd(); ++it ) {
+                    (*it)->setSessionInteract(true);
+                }
+            } else if (allow) {
+                // also force if activation is allowed
+                if( !isOnCurrentDesktop() ) {
+                    workspace()->setCurrentDesktop( desktop() );
+                }
+                /*if (!isOnCurrentActivity()) {
+                    workspace()->setCurrentActivity( activities().first() );
+                } FIXME no such method*/
+            }
+        }
 
         bool belongs_to_desktop = false;
         for( ClientList::ConstIterator it = group()->members().constBegin();
