@@ -52,6 +52,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <kxerrorhandler.h>
 
+#include <QtGui/QPainter>
+
 namespace KWin
 {
 
@@ -66,7 +68,7 @@ struct RegionDebug
    XserverRegion rr;   
    };   
 
-kdbgstream& operator<<( kdbgstream& stream, RegionDebug r )
+QDebug& operator<<( QDebug& stream, RegionDebug r )
     {       
     if( r.rr == None )
         return stream << "EMPTY";
@@ -269,6 +271,7 @@ bool SceneXrender::selfCheckFinish()
 // the entry point for painting
 void SceneXrender::paint( QRegion damage, ToplevelList toplevels )
     {
+    QTime t = QTime::currentTime();
     foreach( Toplevel* c, toplevels )
         {
         assert( windows.contains( c ));
@@ -283,6 +286,7 @@ void SceneXrender::paint( QRegion damage, ToplevelList toplevels )
         selfCheckSetup();
         damage |= selfCheckRegion();
         }
+    lastRenderTime = t.elapsed();
     flushBuffer( mask, damage );
     if( !selfCheckDone )
         {
@@ -730,7 +734,7 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
             {
             QRect& r = rects[ i ];
             r = QRect( int( r.x() * xscale ), int( r.y() * yscale ),
-                int( r.width() * xscale ), int( r.height() * xscale ));
+                int( r.width() * xscale ), int( r.height() * yscale ));
             }
         transformed_shape.setRects( rects.constData(), rects.count());
         }
@@ -843,5 +847,177 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
         }
     }
 
+//****************************************
+// SceneXrender::EffectFrame
+//****************************************
+
+SceneXrender::EffectFrame::EffectFrame( EffectFrameImpl* frame )
+    : Scene::EffectFrame( frame )
+    {
+    m_picture = NULL;
+    m_textPicture = NULL;
+    m_iconPicture = NULL;
+    m_selectionPicture = NULL;
+    }
+
+SceneXrender::EffectFrame::~EffectFrame()
+    {
+    delete m_picture;
+    delete m_textPicture;
+    delete m_iconPicture;
+    delete m_selectionPicture;
+    }
+
+void SceneXrender::EffectFrame::free()
+    {
+    delete m_picture;
+    m_picture = NULL;
+    delete m_textPicture;
+    m_textPicture = NULL;
+    delete m_iconPicture;
+    m_iconPicture = NULL;
+    delete m_selectionPicture;
+    m_selectionPicture = NULL;
+    }
+
+void SceneXrender::EffectFrame::freeIconFrame()
+    {
+    delete m_iconPicture;
+    m_iconPicture = NULL;
+    }
+
+void SceneXrender::EffectFrame::freeTextFrame()
+    {
+    delete m_textPicture;
+    m_textPicture = NULL;
+    }
+
+void SceneXrender::EffectFrame::freeSelection()
+    {
+    delete m_selectionPicture;
+    m_selectionPicture = NULL;
+    }
+
+void SceneXrender::EffectFrame::crossFadeIcon()
+    {
+    // TODO: implement me
+    }
+
+void SceneXrender::EffectFrame::crossFadeText()
+    {
+    // TODO: implement me
+    }
+
+void SceneXrender::EffectFrame::render( QRegion region, double opacity, double frameOpacity )
+    {
+    if( m_effectFrame->geometry().isEmpty() )
+        {
+        return; // Nothing to display
+        }
+
+    // Render the actual frame
+    if( m_effectFrame->style() == EffectFrameUnstyled )
+        xRenderRoundBox( effects->xrenderBufferPicture(), m_effectFrame->geometry().adjusted( -5, -5, 5, 5 ),
+            5, QColor( 0, 0, 0, int( opacity * frameOpacity * 255 )));
+    else if( m_effectFrame->style() == EffectFrameStyled )
+        {
+        if( !m_picture ) // Lazy creation
+            {
+            updatePicture();
+            }
+        qreal left, top, right, bottom;
+        m_effectFrame->frame().getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
+        QRect geom = m_effectFrame->geometry().adjusted( -left, -top, right, bottom );
+        XRenderComposite( display(), PictOpOver, *m_picture, None, effects->xrenderBufferPicture(),
+            0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
+
+        if( !m_effectFrame->selection().isNull() )
+            {
+            if( !m_selectionPicture ) // Lazy creation
+                {
+                m_selectionPicture = new XRenderPicture( m_effectFrame->selectionFrame().framePixmap() );
+                }
+            geom = m_effectFrame->selection();
+            XRenderComposite( display(), PictOpOver, *m_selectionPicture, None, effects->xrenderBufferPicture(),
+                          0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
+            }
+        }
+
+    XRenderPicture fill = xRenderBlendPicture(opacity);
+
+    // Render icon
+    if( !m_effectFrame->icon().isNull() && !m_effectFrame->iconSize().isEmpty() )
+        {
+        QPoint topLeft( m_effectFrame->geometry().x(), m_effectFrame->geometry().center().y() - m_effectFrame->iconSize().height() / 2 );
+
+        if( !m_iconPicture ) // lazy creation
+            m_iconPicture = new XRenderPicture( m_effectFrame->icon() );
+        QRect geom = QRect( topLeft, m_effectFrame->iconSize() );
+        XRenderComposite( display(), PictOpOver, *m_iconPicture, fill, effects->xrenderBufferPicture(),
+            0, 0, 0, 0, geom.x(), geom.y(), geom.width(), geom.height() );
+        }
+
+    // Render text
+    if( !m_effectFrame->text().isEmpty() )
+        {
+        if( !m_textPicture ) // Lazy creation
+            {
+            updateTextPicture();
+            }
+        XRenderComposite( display(), PictOpOver, *m_textPicture, fill, effects->xrenderBufferPicture(),
+            0, 0, 0, 0, m_effectFrame->geometry().x(), m_effectFrame->geometry().y(),
+                          m_effectFrame->geometry().width(), m_effectFrame->geometry().height() );
+        }
+    }
+
+void SceneXrender::EffectFrame::updatePicture()
+    {
+    delete m_picture;
+    if( m_effectFrame->style() == EffectFrameStyled )
+        m_picture = new XRenderPicture( m_effectFrame->frame().framePixmap() );
+    }
+
+void SceneXrender::EffectFrame::updateTextPicture()
+    { // Mostly copied from SceneOpenGL::EffectFrame::updateTextTexture() above
+    delete m_textPicture;
+
+    if( m_effectFrame->text().isEmpty() )
+        {
+        return;
+        }
+
+    // Determine position on texture to paint text
+    QRect rect( QPoint( 0, 0 ), m_effectFrame->geometry().size() );
+    if( !m_effectFrame->icon().isNull() && !m_effectFrame->iconSize().isEmpty() )
+        {
+        rect.setLeft( m_effectFrame->iconSize().width() );
+        }
+
+    // If static size elide text as required
+    QString text = m_effectFrame->text();
+    if( m_effectFrame->isStatic() )
+        {
+        QFontMetrics metrics( m_effectFrame->text() );
+        text = metrics.elidedText( text, Qt::ElideRight, rect.width() );
+        }
+
+    QPixmap pixmap( m_effectFrame->geometry().size() );
+    pixmap.fill( Qt::transparent );
+    QPainter p( &pixmap );
+    p.setFont( m_effectFrame->font() );
+    if( m_effectFrame->style() == EffectFrameStyled )
+        {
+        p.setPen( m_effectFrame->styledTextColor() );
+        }
+    else
+        {
+        // TODO: What about no frame? Custom color setting required
+        p.setPen( Qt::white );
+        }
+    p.drawText( rect, m_effectFrame->alignment(), text );
+    p.end();
+    m_textPicture = new XRenderPicture( pixmap );
+    }
+
 } // namespace
 #endif
