@@ -19,6 +19,7 @@
  */
 
 #include "calendartable.h"
+#include "config-calendartable.h"
 
 //Qt
 #include <QtCore/QDate>
@@ -40,7 +41,11 @@
 #include <Plasma/Theme>
 #include <Plasma/DataEngine>
 
+#ifdef HAVE_KDEPIMLIBS
+#include "ui_calendarHolidaysConfig.h"
+#else
 #include "ui_calendarConfig.h"
+#endif
 
 #include <cmath>
 
@@ -284,10 +289,48 @@ class CalendarTablePrivate
 
         QString defaultHolidaysRegion()
         {
+            if (!dataEngine) {
+                return QString();
+            }
+
             return dataEngine->query("holidaysDefaultRegion").value("holidaysDefaultRegion").toString();
         }
 
-        void insertPimOccurence(const QString &type, const QDate &date, Plasma::DataEngine::Data occurrence) {
+        bool isValidHolidaysRegion(const QString &holidayRegion)
+        {
+            if (holidaysRegions.contains(holidayRegion)) {
+                return true;
+            } else {
+                QString queryString = "holidaysIsValidRegion" + QString(':') + holidayRegion;
+                return (dataEngine && dataEngine->query(queryString).value(queryString).toBool());
+            }
+        }
+
+        void addHolidaysRegion(const QString &holidayRegion, bool daysOff)
+        {
+            if (!holidaysRegions.contains(holidayRegion) && dataEngine) {
+                Plasma::DataEngine::Data regions;
+                QString queryString = "holidaysRegion" + QString(':') + holidayRegion;
+                regions = dataEngine->query(queryString);
+                Plasma::DataEngine::DataIterator i(regions);
+                while (i.hasNext()) {
+                    i.next();
+                    Plasma::DataEngine::Data region = i.value().toHash();
+                    region.insert("UseForDaysOff", daysOff);
+                    holidaysRegions.insert(i.key(), region);
+                }
+            }
+        }
+
+        bool holidayIsDayOff(Plasma::DataEngine::Data holidayData)
+        {
+            return ( holidayData.value("ObservanceType").toString() == "PublicHoliday" &&
+                     holidaysRegions.value(holidayData.value("RegionCode").toString()).value("UseForDaysOff").toBool() );
+        }
+
+
+        void insertPimOccurence(const QString &type, const QDate &date, Plasma::DataEngine::Data occurrence)
+        {
             if (date >= viewStartDate && date <= viewEndDate) {
                 int julian = date.toJulianDay();
                 if (type == "Event" && !events.contains(julian, occurrence)) {
@@ -317,17 +360,23 @@ class CalendarTablePrivate
 
         bool displayEvents;
         bool displayHolidays;
-        QString holidaysRegion;
         Plasma::DataEngine *dataEngine;
-        // Hash key: int = Julian Day number of holiday, Data = details of holiday
-        QMultiHash<int, Plasma::DataEngine::Data> holidays;
+        // List of Holiday Regions to display
+        // Hash key: QString = Holiday Region Code, Data = details of Holiday Region
+        QHash<QString, Plasma::DataEngine::Data> holidaysRegions;
+        // Index to Holidays that occur on a given date.
+        // Hash key: int = Julian Day number of holiday observance, int = key of holiday event
+        QMultiHash<int, int> holidays;
+        // Holiday details.  A holiday may be observed on multiple dates.
+        // Hash key: int = key of holiday event, Data = details of holiday
+        QHash<int, Plasma::DataEngine::Data> holidayEvents;
         // Index to Events/Todos/Journals that occur on a given date.
         // Hash key: int = Julian Day number of event/todo occurrence, Data = occurence details including start date, end date and Akonadi incidence UID
         QMultiHash<int, Plasma::DataEngine::Data> events;
         QMultiHash<int, Plasma::DataEngine::Data> todos;
         QMultiHash<int, Plasma::DataEngine::Data> journals;
         // Event/Todo/Journal details.  An event may recur on multiple dates.
-        // Hask key: QString = Akonadi UID of event/todo/journal, Data = details of event/todo/journal
+        // Hash key: QString = Akonadi UID of event/todo/journal, Data = details of event/todo/journal
         QHash<QString, Plasma::DataEngine::Data> pimEvents;
         QString eventsQuery;
 
@@ -335,7 +384,11 @@ class CalendarTablePrivate
 
         QPointF lastSeenMousePos;
 
+#ifdef HAVE_KDEPIMLIBS
+        Ui::calendarHolidaysConfig calendarConfigUi;
+#else
         Ui::calendarConfig calendarConfigUi;
+#endif
 
         Plasma::Svg *svg;
         float opacity; //transparency for the inactive text
@@ -427,7 +480,7 @@ void CalendarTable::setDate(const QDate &newDate)
 
     d->updateHoveredPainting(d->lastSeenMousePos);
     update(); //FIXME: we shouldn't need this update here, but something in Qt is broken (but with plasmoidviewer everything work)
-    
+
     if (oldYear != d->selectedYear || oldMonth != d->selectedMonth) {
         populateHolidays();
         populateEvents();
@@ -455,7 +508,6 @@ const QDate& CalendarTable::date() const
 
 void CalendarTable::setDataEngine(Plasma::DataEngine *dataEngine)
 {
-    // JPL What happens to the old data engine, who cleans it up, do we need to delete first?
     if (d->dataEngine != dataEngine) {
         d->dataEngine = dataEngine;
         populateHolidays();
@@ -471,18 +523,19 @@ const Plasma::DataEngine *CalendarTable::dataEngine() const
 void CalendarTable::setDisplayHolidays(bool showHolidays)
 {
     if (showHolidays) {
-        if (!dataEngine()) {
+        if (!d->dataEngine) {
             clearHolidays();
             return;
         }
 
-        if (holidaysRegion().isEmpty()) {
-            setHolidaysRegion(d->defaultHolidaysRegion());
+        if (d->holidaysRegions.isEmpty()) {
+            addHolidaysRegion(d->defaultHolidaysRegion(), true);
         }
 
-        QString queryString = "holidaysIsValidRegion:" + holidaysRegion();
-        if (!dataEngine()->query(queryString).value(queryString).toBool()) {
-            return;
+        foreach ( const QString &holidayRegion, holidaysRegions() ) {
+            if (!d->isValidHolidaysRegion(holidayRegion)) {
+                return;
+            }
         }
     }
 
@@ -494,7 +547,7 @@ void CalendarTable::setDisplayHolidays(bool showHolidays)
 
 bool CalendarTable::displayHolidays()
 {
-    return d->displayHolidays && !holidaysRegion().isEmpty();
+    return d->displayHolidays && !holidaysRegions().isEmpty();
 }
 
 bool CalendarTable::displayEvents()
@@ -522,32 +575,54 @@ void CalendarTable::setDisplayEvents(bool display)
     }
 }
 
-void CalendarTable::setHolidaysRegion(const QString &region)
+void CalendarTable::clearHolidaysRegions()
 {
-    QString queryString = "holidaysIsValidRegion:" + region;
-    if (!dataEngine()->query(queryString).value(queryString).toBool()) {
-        return;
-    }
+    clearHolidays();
+    d->holidaysRegions.clear();
+    populateHolidays();
+}
 
-    if (d->holidaysRegion != region) {
-        d->holidaysRegion = region;
+void CalendarTable::addHolidaysRegion(const QString &region, bool daysOff)
+{
+    if (d->isValidHolidaysRegion(region)) {
+        d->addHolidaysRegion(region, daysOff);
         populateHolidays();
     }
 }
 
-QString CalendarTable::holidaysRegion() const
+QStringList CalendarTable::holidaysRegions() const
 {
-    return d->holidaysRegion;
+    return QStringList(d->holidaysRegions.keys());
+}
+
+QStringList CalendarTable::holidaysRegionsDaysOff() const
+{
+    QStringList regions;
+    QHashIterator<QString, Plasma::DataEngine::Data> it(d->holidaysRegions);
+    while (it.hasNext()) {
+        it.next();
+        if (it.value().value("UseForDaysOff").toBool()) {
+            regions.append(it.key());
+        }
+    }
+    return regions;
 }
 
 void CalendarTable::clearHolidays()
 {
+    d->holidayEvents.clear();
     d->holidays.clear();
 }
 
-void CalendarTable::addHoliday(const QDate &date,  Plasma::DataEngine::Data holidayData)
+void CalendarTable::addHoliday(Plasma::DataEngine::Data holidayData)
 {
-    d->holidays.insertMulti(date.toJulianDay(), holidayData);
+    QDate holidayStartDate = QDate::fromString(holidayData.value("ObservanceStartDate").toString(), Qt::ISODate);
+    int holidayDuration = holidayData.value("ObservanceDuration").toInt();
+    int uid = d->holidayEvents.count();
+    d->holidayEvents.insert(uid, holidayData);
+    for (int i = 0; i < holidayDuration; ++i) {
+        d->holidays.insertMulti(holidayStartDate.addDays(i).toJulianDay(), uid);
+    }
 }
 
 bool CalendarTable::dateHasDetails(const QDate &date) const
@@ -567,17 +642,26 @@ QString CalendarTable::dateDetails(const QDate &date) const
     if (d->holidays.contains(julian)) {
         details += "<br>";
 
-        foreach (Plasma::DataEngine::Data holidayData, d->holidays.values(julian)) {
-            if (holidayData.value("observanceType").toString() == "PublicHoliday") {
-                details += i18n("<i>Holiday</i>: %1", holidayData.value("name").toString());
+        foreach (int holidayUid, d->holidays.values(julian)) {
+            Plasma::DataEngine::Data holidayData = d->holidayEvents.value(holidayUid);
+            if (d->holidayIsDayOff(holidayData)) {
+                QString region = holidayData.value("RegionCode").toString();
+                details += i18nc("Day off: Holiday name (holiday region)",
+                                 "<i>Holiday</i>: %1 (%2)",
+                                 holidayData.value("Name").toString(),
+                                 d->holidaysRegions.value(region).value("Name").toString());
                 details += "<br>";
             }
         }
 
-        foreach (Plasma::DataEngine::Data holidayData, d->holidays.values(julian)) {
-            if (holidayData.value("observanceType").toString() == "Other") {
-                //TODO add a type when strings not frozen
-                details += holidayData.value("name").toString();
+        foreach (int holidayUid, d->holidays.values(julian)) {
+            Plasma::DataEngine::Data holidayData = d->holidayEvents.value(holidayUid);
+            if (!d->holidayIsDayOff(holidayData)) {
+                QString region = holidayData.value("RegionCode").toString();
+                details += i18nc("Not day off: Holiday name (holiday region)",
+                                 "<i>Other</i>: %1 (%2)",
+                                 holidayData.value("Name").toString(),
+                                 d->holidaysRegions.value(region).value("Name").toString());
                 details += "<br>";
             }
         }
@@ -646,7 +730,7 @@ bool CalendarTable::isAutomaticUpdateEnabled() const
 {
     return d->automaticUpdates;
 }
-    
+
 void CalendarTable::setCurrentDate(const QDate &date)
 {
     d->currentDate = date; 
@@ -661,20 +745,19 @@ void CalendarTable::populateHolidays()
 {
     clearHolidays();
 
-    if (!displayHolidays() || !dataEngine() || holidaysRegion().isEmpty()) {
+    if (!displayHolidays() || !d->dataEngine || holidaysRegions().isEmpty()) {
         return;
     }
 
     // Just fetch the days displayed in the grid
-    QString queryString = "holidays:" + holidaysRegion() + ':' + d->viewStartDate.toString(Qt::ISODate)
-                          + ':' + d->viewEndDate.toString(Qt::ISODate);
+    QString queryString = "holidays" + QString(':') + holidaysRegions().join(QString(','))
+                                     + QString(':') + d->viewStartDate.toString(Qt::ISODate)
+                                     + QString(':') + d->viewEndDate.toString(Qt::ISODate);
     QList<QVariant> holidays = d->dataEngine->query(queryString).value(queryString).toList();
 
     QMutableListIterator<QVariant> i(holidays);
     while (i.hasNext()) {
-        Plasma::DataEngine::Data holidayData = i.next().toHash();
-        QDate holidayDate = QDate::fromString(holidayData.take("date").toString(), Qt::ISODate);
-        addHoliday(holidayDate, holidayData);
+        addHoliday(i.next().toHash());
     }
 }
 
@@ -691,7 +774,8 @@ void CalendarTable::populateEvents()
 
     // Just fetch the days displayed in the grid
     d->dataEngine->disconnectSource(d->eventsQuery, this);
-    d->eventsQuery = "events:" + d->viewStartDate.toString(Qt::ISODate) + ':' + d->viewEndDate.toString(Qt::ISODate);
+    d->eventsQuery = "events" + QString(':') + d->viewStartDate.toString(Qt::ISODate)
+                              + QString(':') + d->viewEndDate.toString(Qt::ISODate);
     d->dataEngine->connectSource(d->eventsQuery, this);
 }
 
@@ -731,14 +815,21 @@ void CalendarTable::dataUpdated(const QString &source, const Plasma::DataEngine:
 void CalendarTable::applyConfiguration(KConfigGroup cg)
 {
     setCalendar(cg.readEntry("calendarType", "locale"));
-    setHolidaysRegion(cg.readEntry("holidaysRegion", d->defaultHolidaysRegion()));
+    QStringList regions = cg.readEntry("holidaysRegions", QStringList(d->defaultHolidaysRegion()));
+    QStringList daysOff = cg.readEntry("holidaysRegionsDaysOff", QStringList(d->defaultHolidaysRegion()));
+    clearHolidaysRegions();
+    foreach(const QString & region, regions) {
+        addHolidaysRegion(region, daysOff.contains(region));
+    }
     setDisplayHolidays(cg.readEntry("displayHolidays", true));
+    setDisplayEvents(cg.readEntry("displayEvents", true));
 }
 
 void CalendarTable::writeConfiguration(KConfigGroup cg)
 {
     cg.writeEntry("calendarType", d->calendarType);
-    cg.writeEntry("holidaysRegion", d->holidaysRegion);
+    cg.writeEntry("holidaysRegions", holidaysRegions());
+    cg.writeEntry("holidaysRegionsDaysOff", holidaysRegionsDaysOff());
     cg.writeEntry("displayHolidays", d->displayHolidays);
 }
 
@@ -755,39 +846,41 @@ void CalendarTable::createConfigurationInterface(KConfigDialog *parent)
     }
     d->calendarConfigUi.calendarComboBox->setCurrentIndex( d->calendarConfigUi.calendarComboBox->findData( QVariant( d->calendarType ) ) );
 
-    Plasma::DataEngine::Data regions = dataEngine()->query("holidaysRegions");
-    QMap<QString, QString> regionsMap;
-    Plasma::DataEngine::DataIterator i(regions);
-    while (i.hasNext()) {
-        i.next();
-        Plasma::DataEngine::Data regionData = i.value().toHash();
-        QString name = regionData.value("name").toString();
-        QString languageName = KGlobal::locale()->languageCodeToName(regionData.value("languageCode").toString());
-        QString label;
-        if (languageName.isEmpty()) {
-            label = name;
+#ifdef HAVE_KDEPIMLIBS
+    QHashIterator<QString, Plasma::DataEngine::Data> it(d->holidaysRegions);
+    while (it.hasNext()) {
+        it.next();
+        if (it.value().value("UseForDaysOff").toBool()) {
+            d->calendarConfigUi.holidayRegionWidget->setRegionUseFlags(it.key(), KHolidays::HolidayRegionSelector::UseDaysOff);
         } else {
-            // Need to get permission to break string freeze, in the meantime don't translate!
-            //label = i18nc("Holday region, region language", "%1 (%2)", name, languageName);
-            label = QString("%1 (%2)").arg(name).arg(languageName);
+            d->calendarConfigUi.holidayRegionWidget->setRegionUseFlags(it.key(), KHolidays::HolidayRegionSelector::UseInformationOnly);
         }
-        regionsMap.insert(label, i.key());
     }
-
-    d->calendarConfigUi.regionComboBox->addItem(i18n("Do not show holidays"), QString());
-    QMapIterator<QString, QString> j(regionsMap);
-    while ( j.hasNext() ) {
-        j.next();
-        d->calendarConfigUi.regionComboBox->addItem(j.key(), QVariant(j.value()));
-    }
-    d->calendarConfigUi.regionComboBox->setCurrentIndex( d->calendarConfigUi.regionComboBox->findData( QVariant( d->holidaysRegion ) ) );
+    d->calendarConfigUi.holidayRegionWidget->setDescriptionHidden(true);
+#endif
 }
 
 void CalendarTable::applyConfigurationInterface()
 {
     setCalendar(d->calendarConfigUi.calendarComboBox->itemData(d->calendarConfigUi.calendarComboBox->currentIndex()).toString());
-    setHolidaysRegion(d->calendarConfigUi.regionComboBox->itemData(d->calendarConfigUi.regionComboBox->currentIndex()).toString());
-    setDisplayHolidays(!d->calendarConfigUi.regionComboBox->itemData(d->calendarConfigUi.regionComboBox->currentIndex()).toString().isEmpty());
+
+#ifdef HAVE_KDEPIMLIBS
+    clearHolidaysRegions();
+    QHash<QString, KHolidays::HolidayRegionSelector::RegionUseFlags> regions = d->calendarConfigUi.holidayRegionWidget->regionUseFlags();
+    QHashIterator<QString, KHolidays::HolidayRegionSelector::RegionUseFlags> it(regions);
+    bool displayHolidays = false;
+    while (it.hasNext()) {
+        it.next();
+        if (it.value() == KHolidays::HolidayRegionSelector::UseDaysOff) {
+            addHolidaysRegion(it.key(), true);
+            displayHolidays = true;
+        } else if (it.value() == KHolidays::HolidayRegionSelector::UseInformationOnly) {
+            addHolidaysRegion(it.key(), false);
+            displayHolidays = true;
+        }
+    }
+    setDisplayHolidays(displayHolidays);
+#endif
 }
 
 void CalendarTable::configAccepted(KConfigGroup cg)
@@ -980,8 +1073,8 @@ void CalendarTable::paint(QPainter *p, const QStyleOptionGraphicsItem *option, Q
                 type |= CalendarTable::Selected;
             }
 
-            foreach (Plasma::DataEngine::Data holidayData, d->holidays.values(julian)) {
-                if (holidayData.value("observanceType").toString() == "PublicHoliday") {
+            foreach (int holidayUid, d->holidays.values(julian)) {
+                if (d->holidayIsDayOff(d->holidayEvents.value(holidayUid))) {
                     type |= CalendarTable::PublicHoliday;
                 } else {
                     type |= CalendarTable::Holiday;
