@@ -95,25 +95,25 @@ public:
     Plasma::Label *label;
     Plasma::Calendar *calendarWidget;
     int announceInterval;
-    int prevHour;
-    int prevMinute;
+    QTime lastTimeSeen;
     bool forceTzDisplay : 1;
 
     void addTzToTipText(QString &subText, const QString& tz)
     {
         Plasma::DataEngine::Data data = q->dataEngine("time")->query(tz);
 
+        subText.append("<tr><td align=\"right\">");
         if (tz == "UTC")  {
-            subText.append("<br><b>UTC</b>&nbsp;");
+            subText.append("UTC");
         } else {
-            subText.append("<br><b>")
-                   .append(data["Timezone City"].toString().replace('_', "&nbsp;"))
-                   .append("</b> ");
+            subText.append(data["Timezone City"].toString().replace('_', "&nbsp;"));
         }
+        subText.append(":</td><td>");
 
         subText.append(KGlobal::locale()->formatTime(data["Time"].toTime(), false).replace(' ', "&nbsp;"))
                .append(",&nbsp;")
-               .append(q->calendar()->formatDate(data["Date"].toDate()).replace(' ', "&nbsp;"));
+               .append(q->calendar()->formatDate(data["Date"].toDate()).replace(' ', "&nbsp;"))
+               .append("</td></tr>");
     }
 
     void createCalendarExtender()
@@ -127,7 +127,8 @@ public:
 
     void createToday()
     {
-        QString tmpStr = "isHoliday:" + calendarWidget->holidaysRegion() + ':' + QDate::currentDate().toString(Qt::ISODate);
+        QString tmpStr = "isHoliday" + QString(':') + calendarWidget->holidaysRegions().join(QString(','))
+                                     + QString(':') + QDate::currentDate().toString(Qt::ISODate);
         bool isHoliday = q->dataEngine("calendar")->query(tmpStr).value(tmpStr).toBool();
 
         Plasma::ExtenderItem *todayExtender = q->extender()->item("today");
@@ -183,15 +184,12 @@ void ClockApplet::speakTime(const QTime &time)
         return;
     }
 
-    if (time.minute() != d->prevMinute && (time.minute() % d->announceInterval) == 0) {
-        d->prevHour = time.hour();
-        d->prevMinute = time.minute();
-
+    if ((time.minute() % d->announceInterval) == 0) {
         // If KTTSD not running, start it.
         if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kttsd")) {
             QString error;
             if (KToolInvocation::startServiceByDesktopName("kttsd", QStringList(), &error)) {
-                KPassivePopup::message(i18n("Starting KTTSD Failed"), error, static_cast<QWidget *>(0));
+                KPassivePopup::message(i18n("Starting Jovie Text-to-Speech Service Failed"), error, static_cast<QWidget *>(0));
                 return;
             }
         }
@@ -267,7 +265,7 @@ void ClockApplet::updateTipContent()
     mainText += calendar()->formatDate(tipDate);
     tipData.setMainText(mainText);
 
-    QString subText;
+    QString subText("<table>");
     if (!isLocalTimezone()) {
         d->addTzToTipText(subText, localTimezone());
     }
@@ -280,14 +278,13 @@ void ClockApplet::updateTipContent()
         d->addTzToTipText(subText, tz);
     }
 
+    if (!subText.isEmpty()) {
+        subText.prepend("<table>");
+        subText.append("</table>");
+    }
+
     if (d->calendarWidget->dateHasDetails(tipDate)) {
-        QString property = d->calendarWidget->dateDetails(tipDate);
-        QString countryString = KGlobal::locale()->countryCodeToName(d->calendarWidget->holidaysRegion());
-        if (countryString.isEmpty()) {
-            subText.append("<br>").append(property);
-        } else {
-            subText.append("<br><b>").append(countryString).append("</b> ").append(property);
-        }
+        subText.append("<br>").append(d->calendarWidget->dateDetails(tipDate));
     }
 
     tipData.setSubText(subText);
@@ -316,14 +313,34 @@ void ClockApplet::updateTipContent()
 
 void ClockApplet::updateClockApplet()
 {
-    bool updateSelectedDate = (d->calendarWidget->currentDate() == d->calendarWidget->date());
-
     Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
+    updateClockApplet(data);
+}
+
+void ClockApplet::updateClockApplet(const Plasma::DataEngine::Data &data)
+{
+    bool updateSelectedDate = (d->calendarWidget->currentDate() == d->calendarWidget->date());
     d->calendarWidget->setCurrentDate(data["Date"].toDate());
-    
+
     if (updateSelectedDate){
         d->calendarWidget->setDate(d->calendarWidget->currentDate());
     }
+
+    const QTime t = d->lastTimeSeen;
+    d->lastTimeSeen = data["Time"].toTime();
+    if (d->lastTimeSeen.minute() != t.minute() || d->lastTimeSeen.hour() != t.hour()) {
+        speakTime(d->lastTimeSeen);
+    }
+}
+
+QTime ClockApplet::lastTimeSeen() const
+{
+    return d->lastTimeSeen;
+}
+
+void ClockApplet::resetLastTimeSeen()
+{
+    d->lastTimeSeen = QTime();
 }
 
 Plasma::ToolTipContent ClockApplet::toolTipContent()
@@ -588,9 +605,7 @@ void ClockApplet::popupEvent(bool show)
 
 void ClockApplet::constraintsEvent(Plasma::Constraints constraints)
 {
-    if (constraints & Plasma::FormFactorConstraint) {
-        setPassivePopup(formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical);
-    }
+    Q_UNUSED(constraints)
 }
 
 void ClockApplet::setCurrentTimezone(const QString &tz)
@@ -688,14 +703,15 @@ void ClockApplet::updateClipboardMenu()
     sep2->setSeparator(true);
     d->clipboardMenu->addAction(sep2);
 
+    QMenu *calendarMenu = d->clipboardMenu->addMenu( i18nc( "@item:inmenu Submenu for alternative calendar dates", "Other Calendars" ) );
     QStringList calendars = KCalendarSystem::calendarSystems();
     foreach ( const QString &cal, calendars ) {
         if (cal != calendar()->calendarType()) {
             KCalendarSystem *tempCal = KCalendarSystem::create(cal);
             QString text = tempCal->formatDate(dateTime.date(), KLocale::LongDate) + " (" + KCalendarSystem::calendarLabel(cal) + ')';
-            d->clipboardMenu->addAction(text);
+            calendarMenu->addAction(text);
             text = tempCal->formatDate(dateTime.date(), KLocale::ShortDate) + " (" + KCalendarSystem::calendarLabel(cal) + ')';
-            d->clipboardMenu->addAction(text);
+            calendarMenu->addAction(text);
             delete tempCal;
         }
     }
