@@ -52,10 +52,10 @@
 #include <KRun>
 #include <KServiceTypeTrader>
 #include <KToolInvocation>
+#include <Solid/PowerManagement>
 
 // KDE Base
 #include <kworkspace/kworkspace.h>
-#include <solid/control/powermanager.h>
 
 // Plasma
 #include <Plasma/IconWidget>
@@ -125,6 +125,8 @@ public:
     QAction* switcher;
     Kickoff::ContextMenuFactory *contextMenuFactory;
 
+    bool delayedConfigLoad;
+
     explicit Private(MenuLauncherApplet *q)
             : q(q),
               icon(0),
@@ -135,11 +137,11 @@ public:
               iconButton(0),
               formatComboBox(0),
               showMenuTitlesCheckBox(0),
-              switcher(0), 
-              contextMenuFactory(0) 
+              switcher(0),
+              contextMenuFactory(0)
     {}
 
-    ~Private() 
+    ~Private()
     {
         delete bookmarkmenu;
         delete bookmarkowner;
@@ -204,8 +206,8 @@ public:
             case SuspendDisk:              return i18n("Hibernate");
             case SuspendRAM:               return i18n("Sleep");
             case Restart:                  return i18nc("Restart Computer", "Restart");
-            case Shutdown:                 return i18n("Shutdown");
-            case Logout:                   return i18n("Logout");
+            case Shutdown:                 return i18n("Shut down");
+            case Logout:                   return i18n("Log out");
             case Leave:                    return i18n("Leave");
         }
         return QString();
@@ -281,20 +283,30 @@ MenuLauncherApplet::MenuLauncherApplet(QObject *parent, const QVariantList &args
 
     d->icon = new Plasma::IconWidget(QString(), this);
     d->icon->setFlag(ItemIsMovable, false);
-    connect(d->icon, SIGNAL(pressed(bool)), this, SLOT(toggleMenu(bool)));
+    connect(d->icon, SIGNAL(pressed(bool)), this, SLOT(showMenu(bool)));
     connect(this, SIGNAL(activate()), this, SLOT(toggleMenu()));
 
-    if (args.count() < 2) { // assuming args is only used for passing in submenu paths
-        d->viewtypes << "RecentlyUsedApplications" << "Applications" << "Favorites";
-        if (KAuthorized::authorize("run_command")) {
-            d->viewtypes << "RunCommand";
+    d->delayedConfigLoad = false;
+    switch(args.count()) {
+        case 2: { //Use submenu paths
+            d->viewtypes << "Applications";
+            d->relativePath = args.value(0).toString();
+            d->iconname = args.value(1).toString();
+            break;
+        }
+        case 1: { //Check for delayed config (switch from Kickoff)
+            d->delayedConfigLoad = true;
+            //Do not "break;", as we may need the default views configuration
+            //(if SimpleLauncher was never used before)
+        }
+        default: { //Default configuration
+            d->viewtypes << "RecentlyUsedApplications" << "Applications" << "Favorites";
+            if (KAuthorized::authorize("run_command")) {
+                d->viewtypes << "RunCommand";
+            }
+            d->viewtypes << "Leave";
+            d->iconname = "start-here-kde";
         }
-        d->viewtypes << "Leave";
-        d->iconname = "start-here-kde";
-    } else {
-        d->viewtypes << "Applications";
-        d->relativePath = args.value(0).toString();
-        d->iconname = args.value(1).toString();
     }
     d->formattype = NameDescription;
 
@@ -319,58 +331,31 @@ void MenuLauncherApplet::init()
         receivedArgs = true;
     }
 
-    KConfigGroup cg = config();
+    Q_ASSERT(! d->switcher);
+    d->switcher = new QAction(i18n("Switch to Application Launcher Style"), this);
+    d->actions.append(d->switcher);
+    connect(d->switcher, SIGNAL(triggered(bool)), this, SLOT(switchMenuStyle()));
 
-    const QStringList viewtypes = cg.readEntry("views", QStringList());
-    if(viewtypes.isEmpty()) { // backward-compatibility to <KDE4.3
-        QByteArray oldview = cg.readEntry("view", QByteArray());
-        if (!oldview.isEmpty() && oldview != "Combined") {
-            d->viewtypes = QStringList() << oldview;
-            d->iconname = d->viewIcon(d->viewType(oldview));
-        } // else we use the default d->viewtypes
-    } else {
-        d->viewtypes = viewtypes;
-    }
-    
-    QMetaEnum fte = metaObject()->enumerator(metaObject()->indexOfEnumerator("FormatType"));
-    QByteArray ftb = cg.readEntry("format", QByteArray(fte.valueToKey(d->formattype)));
-    d->formattype = (MenuLauncherApplet::FormatType) fte.keyToValue(ftb);
-
-    d->setMaxRecentApps(cg.readEntry("maxRecentApps", qMin(5, Kickoff::RecentApplications::self()->maximum())));
-    d->showMenuTitles = cg.readEntry("showMenuTitles", false);
-
-    d->icon->setIcon(KIcon(cg.readEntry("icon", d->iconname)));
-
-    d->relativePath = cg.readEntry("relativePath", d->relativePath);
-    if (!d->relativePath.isEmpty()) {
-        d->viewtypes.clear();
-        d->viewtypes << "Applications";
-    }
+    configChanged();
 
     Kickoff::UrlItemLauncher::addGlobalHandler(Kickoff::UrlItemLauncher::ExtensionHandler, "desktop", new Kickoff::ServiceItemHandler);
     Kickoff::UrlItemLauncher::addGlobalHandler(Kickoff::UrlItemLauncher::ProtocolHandler, "leave", new Kickoff::LeaveItemHandler);
 
-    if (KService::serviceByStorageId("kde4-kmenuedit.desktop")) {
-        QAction* menueditor = new QAction(i18n("Menu Editor"), this);
+    if (KService::serviceByStorageId("kde4-kmenuedit.desktop") && KAuthorized::authorize("action/menuedit")) {
+        QAction* menueditor = new QAction(i18n("Edit Applications..."), this);
         d->actions.append(menueditor);
         connect(menueditor, SIGNAL(triggered(bool)), this, SLOT(startMenuEditor()));
     }
 
-    Q_ASSERT(! d->switcher);
-    d->switcher = new QAction(i18n("Switch to Kickoff Menu Style"), this);
-    d->actions.append(d->switcher);
-    connect(d->switcher, SIGNAL(triggered(bool)), this, SLOT(switchMenuStyle()));
-
-    Plasma::ToolTipManager::self()->registerWidget(this);
-    d->updateTooltip();
-
     if (receivedArgs) {
+        KConfigGroup cg = config();
         cg.writeEntry("relativePath", d->relativePath);
         cg.writeEntry("icon", d->iconname);
         emit configNeedsSaving();
     }
 
-    constraintsEvent(Plasma::ImmutableConstraint);
+    connect(KGlobalSettings::self(), SIGNAL(iconChanged(int)),
+        this, SLOT(iconSizeChanged(int)));
 }
 
 void MenuLauncherApplet::constraintsEvent(Plasma::Constraints constraints)
@@ -393,8 +378,12 @@ void MenuLauncherApplet::constraintsEvent(Plasma::Constraints constraints)
 void MenuLauncherApplet::switchMenuStyle()
 {
     if (containment()) {
-        Plasma::Applet * launcher = 
-                                containment()->addApplet("launcher", QVariantList(), geometry());
+        Plasma::Applet *launcher = containment()->addApplet("launcher", QVariantList(), geometry());
+        //Copy all the config items to the simple launcher
+        QMetaObject::invokeMethod(launcher, "saveConfigurationFromSimpleLauncher",
+                                  Qt::DirectConnection, Q_ARG(KConfigGroup, config()),
+                                  Q_ARG(KConfigGroup, globalConfig()));
+
         //Switch shortcuts with the new launcher to avoid losing it
         KShortcut currentShortcut = globalShortcut();
         setGlobalShortcut(KShortcut());
@@ -516,7 +505,7 @@ void MenuLauncherApplet::configAccepted()
             if(item->checkState() == Qt::Checked)
                 viewtypes << vtname;
             if( !needssaving && ((item->checkState() == Qt::Checked && ! d->viewtypes.contains(vtname)) || (item->checkState() == Qt::Unchecked && d->viewtypes.contains(vtname))) )
-                needssaving = true;    
+                needssaving = true;
         }
         if(needssaving) {
             d->viewtypes = viewtypes;
@@ -582,11 +571,17 @@ KService::List sortServices(KService::List list)
     return list;
 }
 
-void MenuLauncherApplet::toggleMenu(bool pressed)
+void MenuLauncherApplet::toggleMenu()
+{
+    showMenu(!d->menuview || !d->menuview.data()->isVisible());
+}
+
+void MenuLauncherApplet::showMenu(bool pressed)
 {
     if (!pressed || d->viewtypes.count()<=0) {
         if (d->menuview) {
             d->menuview.data()->deleteLater();
+            d->menuview.clear();
         }
 
         return;
@@ -610,6 +605,9 @@ void MenuLauncherApplet::toggleMenu(bool pressed)
         foreach(const QString &vtname, d->viewtypes) {
             if(vtname == "Applications") {
                 Kickoff::ApplicationModel *appModel = new Kickoff::ApplicationModel(menuview, true /*allow separators*/);
+
+                appModel->setNameDisplayOrder(Kickoff::NameBeforeDescription);
+
                 appModel->setDuplicatePolicy(Kickoff::ApplicationModel::ShowLatestOnlyPolicy);
                 if (d->formattype == Name || d->formattype == NameDescription || d->formattype == NameDashDescription)
                     appModel->setPrimaryNamePolicy(Kickoff::ApplicationModel::AppNamePrimary);
@@ -730,18 +728,18 @@ void MenuLauncherApplet::toggleMenu(bool pressed)
             } else if(vtname == "Leave") {
                 Kickoff::LeaveModel *leavemodel = new Kickoff::LeaveModel(menuview);
                 leavemodel->updateModel();
-                d->addModel(leavemodel, Leave, Kickoff::MenuView::MergeFirstLevel, Kickoff::MenuView::Description);
+                d->addModel(leavemodel, Leave, Kickoff::MenuView::MergeFirstLevel, Kickoff::MenuView::Name);
             } else {
 #ifndef Q_WS_WIN
-                Solid::Control::PowerManager::SuspendMethods spdMethods = Solid::Control::PowerManager::supportedSuspendMethods();
+                QSet< Solid::PowerManagement::SleepState > spdMethods = Solid::PowerManagement::supportedSleepStates();
                 if (vtname == "Standby") {
-                    if (spdMethods & Solid::Control::PowerManager::Standby)
+                    if (spdMethods.contains(Solid::PowerManagement::StandbyState))
                         menuview->addAction(KIcon(d->viewIcon(Standby)), d->viewText(Standby))->setData(KUrl("leave:/standby"));
                 } else if(vtname == "SuspendDisk") {
-                    if (spdMethods & Solid::Control::PowerManager::ToDisk)
+                    if (spdMethods.contains(Solid::PowerManagement::HibernateState))
                         menuview->addAction(KIcon(d->viewIcon(SuspendDisk)), d->viewText(SuspendDisk))->setData(KUrl("leave:/suspenddisk"));
                 } else if(vtname == "SuspendRAM") {
-                    if (spdMethods & Solid::Control::PowerManager::ToRam)
+                    if (spdMethods.contains(Solid::PowerManagement::SuspendState))
                         menuview->addAction(KIcon(d->viewIcon(SuspendRAM)), d->viewText(SuspendRAM))->setData(KUrl("leave:/suspendram"));
                 } else if(vtname == "Restart") {
                     if (KWorkSpace::canShutDown(KWorkSpace::ShutdownConfirmDefault, KWorkSpace::ShutdownTypeReboot))
@@ -756,6 +754,7 @@ void MenuLauncherApplet::toggleMenu(bool pressed)
     }
 
     Plasma::ToolTipManager::self()->hide(this);
+    setStatus(Plasma::NeedsAttentionStatus);
     d->menuview.data()->popup(popupPosition(d->menuview.data()->sizeHint()));
 }
 
@@ -787,4 +786,83 @@ QList<QAction*> MenuLauncherApplet::contextualActions()
     return d->actions;
 }
 
+void MenuLauncherApplet::iconSizeChanged(int group)
+{
+    if (group == KIconLoader::Desktop || group == KIconLoader::Panel) {
+        updateGeometry();
+    }
+}
+
+QSizeF MenuLauncherApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
+{
+    if (which == Qt::PreferredSize) {
+        int iconSize;
+
+        switch (formFactor()) {
+            case Plasma::Planar:
+            case Plasma::MediaCenter:
+                iconSize = IconSize(KIconLoader::Desktop);
+                break;
+
+            case Plasma::Horizontal:
+            case Plasma::Vertical:
+                iconSize = IconSize(KIconLoader::Panel);
+                break;
+        }
+
+        return QSizeF(iconSize, iconSize);
+    }
+
+    return Plasma::Applet::sizeHint(which, constraint);
+}
+
+
+void MenuLauncherApplet::saveConfigurationFromKickoff(const KConfigGroup & configGroup, const KConfigGroup & globalConfigGroup)
+{
+    //Copy configuration values
+    KConfigGroup cg = config();
+    configGroup.copyTo(&cg);
+
+    KConfigGroup gcg = globalConfig();
+    globalConfigGroup.copyTo(&gcg);
+
+    configChanged();
+    emit configNeedsSaving();
+}
+
+void MenuLauncherApplet::configChanged()
+{
+    KConfigGroup cg = config();
+
+    const QStringList viewtypes = cg.readEntry("views", QStringList());
+    if(viewtypes.isEmpty()) { // backward-compatibility to <KDE4.3
+        QByteArray oldview = cg.readEntry("view", QByteArray());
+        if (!oldview.isEmpty() && oldview != "Combined") {
+            d->viewtypes = QStringList() << oldview;
+            d->iconname = d->viewIcon(d->viewType(oldview));
+        } // else we use the default d->viewtypes
+    } else {
+        d->viewtypes = viewtypes;
+    }
+
+    QMetaEnum fte = metaObject()->enumerator(metaObject()->indexOfEnumerator("FormatType"));
+    QByteArray ftb = cg.readEntry("format", QByteArray(fte.valueToKey(d->formattype)));
+    d->formattype = (MenuLauncherApplet::FormatType) fte.keyToValue(ftb);
+
+    d->setMaxRecentApps(cg.readEntry("maxRecentApps", qMin(5, Kickoff::RecentApplications::self()->maximum())));
+    d->showMenuTitles = cg.readEntry("showMenuTitles", false);
+
+    d->icon->setIcon(KIcon(cg.readEntry("icon", d->iconname)));
+
+    d->relativePath = cg.readEntry("relativePath", d->relativePath);
+    if (!d->relativePath.isEmpty()) {
+        d->viewtypes.clear();
+        d->viewtypes << "Applications";
+    }
+
+    d->updateTooltip();
+
+    constraintsEvent(Plasma::ImmutableConstraint);
+}
+
 #include "simpleapplet.moc"
