@@ -49,6 +49,7 @@
 #include <Plasma/Animator>
 
 #include <kephal/screens.h>
+#include <kactivityconsumer.h>
 
 #include <taskmanager/task.h>
 
@@ -122,20 +123,18 @@ Pager::Pager(QObject *parent, const QVariantList &args)
     resize(m_size);
 }
 
+Pager::~Pager()
+{
+    delete m_colorScheme;
+}
+
 void Pager::init()
 {
     createMenu();
 
-    KConfigGroup cg = config();
-    m_displayedText = (DisplayedText)cg.readEntry("displayedText", (int)m_displayedText);
-    m_showWindowIcons = cg.readEntry("showWindowIcons", m_showWindowIcons);
-    m_rows = globalConfig().readEntry("rows", m_rows);
     m_verticalFormFactor = (formFactor() == Plasma::Vertical);
-    m_currentDesktopSelected = (CurrentDesktopSelected)cg.readEntry("currentDesktopSelected", (int)m_currentDesktopSelected);
 
-    if (m_rows < 1) {
-        m_rows = 1;
-    }
+    configChanged();
 
     m_timer = new QTimer(this);
     m_timer->setSingleShot(true);
@@ -152,7 +151,7 @@ void Pager::init()
     connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)), this, SLOT(numberOfDesktopsChanged(int)));
     connect(KWindowSystem::self(), SIGNAL(desktopNamesChanged()), this, SLOT(desktopNamesChanged()));
     connect(KWindowSystem::self(), SIGNAL(stackingOrderChanged()), this, SLOT(stackingOrderChanged()));
-    connect(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)), this, SLOT(windowChanged(WId,unsigned int)));
+    connect(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned long*)), this, SLOT(windowChanged(WId,unsigned long*)));
     connect(KWindowSystem::self(), SIGNAL(showingDesktopChanged(bool)), this, SLOT(showingDesktopChanged(bool)));
     connect(Kephal::Screens::self(), SIGNAL(screenAdded(Kephal::Screen *)), SLOT(desktopsSizeChanged()));
     connect(Kephal::Screens::self(), SIGNAL(screenRemoved(int)), SLOT(desktopsSizeChanged()));
@@ -170,11 +169,44 @@ void Pager::init()
     recalculateGridSizes(m_rows);
 
     m_currentDesktop = KWindowSystem::currentDesktop();
+
+    KActivityConsumer *act = new KActivityConsumer(this);
+    connect(act, SIGNAL(currentActivityChanged(QString)), this, SLOT(currentActivityChanged(QString)));
+    m_currentActivity = act->currentActivity();
 }
 
-Pager::~Pager()
+void Pager::configChanged()
 {
-    delete m_colorScheme;
+    KConfigGroup cg = config();
+    bool changed = false;
+
+    DisplayedText displayedText = (DisplayedText) cg.readEntry("displayedText", (int) m_displayedText);
+    if (displayedText != m_displayedText) {
+        m_displayedText = displayedText;
+        changed = true;
+    }
+
+    bool showWindowIcons = cg.readEntry("showWindowIcons", m_showWindowIcons);
+    if (showWindowIcons != m_showWindowIcons) {
+        m_showWindowIcons = showWindowIcons;
+        changed = true;
+    }
+
+    CurrentDesktopSelected currentDesktopSelected =
+        (CurrentDesktopSelected) cg.readEntry("currentDesktopSelected",
+                                              (int) m_currentDesktopSelected);
+    if (currentDesktopSelected != m_currentDesktopSelected) {
+        m_currentDesktopSelected = currentDesktopSelected;
+        changed = true;
+    }
+
+    int rows = globalConfig().readEntry("rows", m_rows);
+
+    if (changed || rows != m_rows) {
+        recalculateGridSizes(rows);
+        recalculateWindowRects();
+        update();
+    }
 }
 
 void Pager::constraintsEvent(Plasma::Constraints constraints)
@@ -284,6 +316,7 @@ void Pager::createConfigurationInterface(KConfigDialog *parent)
                     m_configureDesktopsWidget->moduleInfo().icon());
 
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
 
     switch (m_displayedText){
         case Number:
@@ -546,6 +579,17 @@ void Pager::recalculateWindowRects()
             continue;
         }
 
+        //check activity
+        unsigned long properties[] = { 0, NET::WM2Activities };
+        NETWinInfo netInfo(QX11Info::display(), window, QX11Info::appRootWindow(), properties, 2);
+        QString result(netInfo.activities());
+        if (!result.isEmpty()) {
+            QStringList activities = result.split(',');
+            if (!activities.contains(m_currentActivity)) {
+                continue;
+            }
+        }
+
         for (int i = 0; i < m_desktopCount; i++) {
             if (!info.isOnDesktop(i+1)) {
                 continue;
@@ -575,34 +619,23 @@ void Pager::recalculateWindowRects()
 
 void Pager::configAccepted()
 {
+    // only write the config here, it will be loaded in by configChanged(),
+    // which is called after this when the config dialog is accepted
     KConfigGroup cg = config();
-    bool changed = false;
 
     DisplayedText displayedText;
-
     if (ui.desktopNumberRadioButton->isChecked()) {
         displayedText = Number;
-
     } else if (ui.desktopNameRadioButton->isChecked()) {
         displayedText = Name;
     } else {
         displayedText = None;
     }
+    cg.writeEntry("displayedText", (int) displayedText);
 
-    if ((int)m_displayedText != (int)displayedText) {
-        m_displayedText = displayedText;
-        cg.writeEntry("displayedText", (int)m_displayedText);
-        changed = true;
-    }
-
-    if (m_showWindowIcons != ui.showWindowIconsCheckBox->isChecked()) {
-        m_showWindowIcons = ui.showWindowIconsCheckBox->isChecked();
-        cg.writeEntry("showWindowIcons", m_showWindowIcons);
-        changed = true;
-    }
+    cg.writeEntry("showWindowIcons", ui.showWindowIconsCheckBox->isChecked());
 
     CurrentDesktopSelected currentDesktopSelected;
-
     if (ui.doNothingRadioButton->isChecked()) {
         currentDesktopSelected = DoNothing;
     } else if (ui.showDesktopRadioButton->isChecked()) {
@@ -610,17 +643,9 @@ void Pager::configAccepted()
     } else {
         currentDesktopSelected = ShowDashboard;
     }
+    cg.writeEntry("currentDesktopSelected", (int) currentDesktopSelected);
 
-    if ((int)m_currentDesktopSelected != (int)currentDesktopSelected) {
-        m_currentDesktopSelected = currentDesktopSelected;
-        cg.writeEntry("currentDesktopSelected", (int)m_currentDesktopSelected);
-        changed = true;
-    }
-
-    if (m_configureDesktopsWidget->changed()) {
-        m_configureDesktopsWidget->save();
-        changed = true;
-    }
+    m_configureDesktopsWidget->save();
 
     // we need to keep all pager applets consistent since this affects
     // the layout of the desktops as used by the window manager,
@@ -634,26 +659,10 @@ void Pager::configAccepted()
     } else {
         rows = ui.spinRows->value();
     }
+    rows = qBound(1, rows, m_desktopCount);
+    globalConfig().writeEntry("rows", rows);
 
-    if (m_rows != rows) {
-        KConfigGroup globalcg = globalConfig();
-        m_rows = rows;
-        if (m_rows > m_desktopCount) {
-            m_rows = m_desktopCount;
-        }
-        globalcg.writeEntry("rows", m_rows);
-        changed = true;
-    }
-
-    if (changed) {
-        emit configNeedsSaving();
-        // force an update
-        m_columns = 0;
-        m_size = QSizeF(-1, -1);
-        recalculateGridSizes(m_rows);
-        recalculateWindowRects();
-        update();
-    }
+    emit configNeedsSaving();
 }
 
 void Pager::currentDesktopChanged(int desktop)
@@ -670,8 +679,19 @@ void Pager::currentDesktopChanged(int desktop)
     }
 }
 
+void Pager::currentActivityChanged(const QString &activity)
+{
+    m_currentActivity = activity;
+
+    if (!m_timer->isActive()) {
+        m_timer->start(FAST_UPDATE_DELAY);
+    }
+}
+
 void Pager::windowAdded(WId id)
 {
+    Q_UNUSED(id)
+
     if (!m_timer->isActive()) {
         m_timer->start(FAST_UPDATE_DELAY);
     }
@@ -730,12 +750,12 @@ void Pager::stackingOrderChanged()
     }
 }
 
-void Pager::windowChanged(WId id, unsigned int properties)
+void Pager::windowChanged(WId id, unsigned long* dirty)
 {
     Q_UNUSED(id)
 
-    if (properties & NET::WMGeometry ||
-        properties & NET::WMDesktop) {
+    if (dirty[NETWinInfo::PROTOCOLS] & (NET::WMGeometry | NET::WMDesktop) ||
+        dirty[NETWinInfo::PROTOCOLS2] & NET::WM2Activities) {
         if (!m_timer->isActive()) {
             m_timer->start(UPDATE_DELAY);
         }
@@ -848,16 +868,16 @@ void Pager::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
             // don't move windows to negative positions
             dest = QPointF(qMax(dest.x(), qreal(0.0)), qMax(dest.y(), qreal(0.0)));
             if (!KWindowSystem::mapViewport()) {
-                KWindowInfo info = KWindowSystem::windowInfo(m_dragId, NET::WMDesktop);
+                KWindowInfo info = KWindowSystem::windowInfo(m_dragId, NET::WMDesktop | NET::WMState);
 
                 if (!info.onAllDesktops()) {
                     KWindowSystem::setOnDesktop(m_dragId, m_dragHighlightedDesktop+1);
                 }
 
-                // only move the window if it is kept within the same desktop
-                // moving when dropping between desktop is too annoying due to
-                // the small drop area.
-                if (m_dragHighlightedDesktop == m_dragStartDesktop || info.onAllDesktops()) {
+                // only move the window if it is not full screen and if it is kept within the same desktop
+                // moving when dropping between desktop is too annoying due to the small drop area.
+                if (!(info.state() & NET::FullScreen) &&
+                    (m_dragHighlightedDesktop == m_dragStartDesktop || info.onAllDesktops())) {
                     // use _NET_MOVERESIZE_WINDOW rather than plain move, so that the WM knows this is a pager request
                     NETRootInfo i( QX11Info::display(), 0 );
                     int flags = ( 0x20 << 12 ) | ( 0x03 << 8 ) | 1; // from tool, x/y, northwest gravity
@@ -1303,6 +1323,7 @@ void Pager::themeRefresh()
 {
     delete m_colorScheme;
     m_colorScheme = 0;
+    update();
 }
 
 void Pager::updateToolTip()
