@@ -28,6 +28,9 @@
 #include <QGraphicsScene>
 #include <QGraphicsGridLayout>
 
+// KDE
+#include <KDebug>
+
 #include <math.h>
 
 #include "windowtaskitem.h"
@@ -174,7 +177,7 @@ int TaskItemLayout::size()
             continue;
         }
 
-        if (item->abstractItem()->isGroupItem()) {
+        if (item->abstractItem()->itemType() == TaskManager::GroupItemType) {
             TaskGroupItem *group = static_cast<TaskGroupItem*>(item);
             if (!group->collapsed()) {
                 TaskItemLayout *layout = dynamic_cast<TaskItemLayout*>(group->tasksLayout());
@@ -237,7 +240,17 @@ int TaskItemLayout::preferredColumns()
         if (m_layoutOrientation == Qt::Vertical) {
             m_rowSize = qMax(1, int(m_groupItem->geometry().height() / itemSize.height()));
         } else {
-            m_rowSize = qMax(1, int(m_groupItem->geometry().width() / itemSize.width()));
+            //Launchers doesn't need the same space as task- and groupitems on horizontal Layouts so the size needs to be adjusted
+            qreal horizontalSpace = m_groupItem->geometry().width();
+            int numberOflaunchers = 0;
+            foreach (AbstractTaskItem *item, m_itemPositions) {
+                if (item->abstractItem()->itemType() == TaskManager::LauncherItemType) {
+                    horizontalSpace -= item->preferredHeight(); //The icon is a square so we can use the height as width
+                    numberOflaunchers++;
+                }
+            }
+            m_rowSize = qMax(1, int(horizontalSpace / itemSize.width()));
+            m_rowSize += numberOflaunchers;
         }
     }
     //kDebug() << "preferred columns: " << qMax(1, m_rowSize);
@@ -312,15 +325,7 @@ void TaskItemLayout::layoutItems()
     foreach (AbstractTaskItem *item, m_itemPositions) {
         int row;
         int col;
-        if (m_forceRows) {
-            if (m_layoutOrientation == Qt::Vertical) {
-                row = numberOfItems / rows;
-                col = numberOfItems % rows;
-            } else {
-                row = numberOfItems % rows;
-                col = numberOfItems / rows;
-            }
-        } else if (m_layoutOrientation == Qt::Vertical) {
+        if (m_layoutOrientation == Qt::Vertical) {
             row = numberOfItems % columns;
             col = numberOfItems / columns;
         } else {
@@ -339,14 +344,20 @@ void TaskItemLayout::layoutItems()
                 setRowMaximumHeight(row, maximumCellSize.height());
                 setColumnMaximumWidth(col, QWIDGETSIZE_MAX);
             } else {
-                setColumnMaximumWidth(col, maximumCellSize.width());
+                if (item->abstractItem()->itemType() == TaskManager::LauncherItemType) {
+                    setColumnFixedWidth(col, maximumCellSize.height()); //The Icon size is a sqare, so it needs the same width as height
+                } else {
+                    setColumnMaximumWidth(col, maximumCellSize.width());
+                }
                 setRowMaximumHeight(row, QWIDGETSIZE_MAX);
             }
             setRowPreferredHeight(row, maximumCellSize.height());
             setColumnPreferredWidth(col, maximumCellSize.width());
         }
 
-        if (item->abstractItem() && item->abstractItem()->isGroupItem()) {
+        if (item->abstractItem() &&
+            item->abstractItem()->itemType() == TaskManager::GroupItemType) {
+
             TaskGroupItem *group = static_cast<TaskGroupItem*>(item);
             if (group->collapsed()) {
                 group->unsplitGroup();
@@ -495,11 +506,7 @@ int TaskItemLayout::insertionIndexAt(const QPointF &pos)
 
     //kDebug() << row << col;
 
-    if (!m_forceRows) {
-        insertIndex = row * numberOfColumns() + col;
-    } else {
-        insertIndex = col * numberOfRows() + row;
-    }
+    insertIndex = row * numberOfColumns() + col;
 
     if (insertIndex > count()) {
         insertIndex--;
