@@ -23,24 +23,15 @@
 
 #include <QAction>
 #include <QApplication>
-#include <QBitmap>
-#include <QComboBox>
-#include <QDesktopWidget>
 #include <QGraphicsLinearLayout>
-#include <QGridLayout>
 #include <QGraphicsLayout>
 #include <QGraphicsSceneDragDropEvent>
-#include <QLabel>
-#include <QMenu>
 #include <QTimer>
 #include <QPainter>
-#include <QSignalMapper>
 
 #include <KDebug>
 #include <KIcon>
-#include <KDialog>
-#include <KIntNumInput>
-#include <KMessageBox>
+#include <KIconLoader>
 
 #include <Plasma/Corona>
 #include <Plasma/FrameSvg>
@@ -116,8 +107,6 @@ Panel::Panel(QObject *parent, const QVariantList &args)
     setMinimumSize(m_currentSize);
     setMaximumSize(m_currentSize);
 
-    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
-
     connect(this, SIGNAL(appletRemoved(Plasma::Applet*)),
             this, SLOT(appletWasRemoved(Plasma::Applet*)));
     setContainmentType(Containment::PanelContainment);
@@ -178,6 +167,7 @@ QList<QAction*> Panel::contextualActions()
 void Panel::backgroundChanged()
 {
     constraintsEvent(Plasma::LocationConstraint);
+    update();
 }
 
 void Panel::adjustLastSpace()
@@ -542,11 +532,6 @@ void Panel::saveState(KConfigGroup &config) const
     config.writeEntry("maximumSize", maximumSize());
 }
 
-void Panel::themeUpdated()
-{
-    updateBorders(geometry().toRect());
-}
-
 void Panel::paintInterface(QPainter *painter,
                            const QStyleOptionGraphicsItem *option,
                            const QRect& contentsRect)
