@@ -46,13 +46,12 @@
 #include <Plasma/FrameSvg>
 #include <Plasma/Dialog>
 
+#include "desktopcorona.h"
 #include "plasmaapp.h"
 #include "positioningruler.h"
 #include "toolbutton.h"
 #include "widgetsexplorer/widgetexplorer.h"
 
-#include <kephal/screens.h>
-
 class PanelController::ButtonGroup: public QFrame
 {
 public:
@@ -298,7 +297,12 @@ void PanelController::moveEvent(QMoveEvent *event)
 {
     //FIXME: this is a glorious hack: it causes the window to be positioned correctly after kwin positions it incorrectly.
     //TODO: the proper way to do is to give a window type specific to the panel controller, that gets positioned correctly
-    emit offsetChanged(m_ruler->offset());
+    if (((location() == Plasma::BottomEdge || location() == Plasma::TopEdge) &&
+         event->oldPos().x() != event->pos().x()) ||
+        ((location() == Plasma::LeftEdge || location() == Plasma::RightEdge) &&
+         event->oldPos().y() != event->pos().y())) {
+        emit offsetChanged(m_ruler->offset());
+    }
     ControllerWindow::moveEvent(event);
 }
 
@@ -363,7 +367,7 @@ void PanelController::setLocation(const Plasma::Location &loc)
         break;
     }
 
-    QRect screenGeom = Kephal::ScreenUtils::screenGeometry(containment()->screen());
+    const QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(containment()->screen());
 
     switch (loc) {
     case Plasma::LeftEdge:
@@ -387,9 +391,13 @@ void PanelController::setLocation(const Plasma::Location &loc)
         m_rightAlignTool->setText(i18n("Right"));
     }
 
-    m_ruler->setMinimumSize(m_ruler->sizeHint());
-    m_ruler->setMaximumSize(m_ruler->sizeHint());
+
     syncRuler();
+    QSize rulerSize = m_ruler->sizeHint();
+    m_ruler->hide();
+    m_ruler->setFixedSize(rulerSize);
+    m_ruler->show();
+    updateGeometry();
 
     resize(sizeHint());
 }
@@ -486,6 +494,11 @@ bool PanelController::eventFilter(QObject *watched, QEvent *event)
 {
     ControllerWindow::eventFilter(watched, event);
 
+    if (event->type() == QEvent::MouseButtonPress) {
+        QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
+        m_lastPos = mouseEvent->globalPos();
+    }
+
     if (watched == m_optionsDialog && event->type() == QEvent::WindowDeactivate && (!isControllerViewVisible())) {
         if (!m_settingsTool->underMouse()) {
             m_optionsDialog->hide();
@@ -502,6 +515,7 @@ bool PanelController::eventFilter(QObject *watched, QEvent *event)
         } else if (event->type() == QEvent::MouseButtonRelease) {
             m_dragging = NoElement;
             m_moveTool->releaseMouse();
+            emit locationChanged(location());
         } else if (event->type() == QEvent::MouseMove) {
             QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
             mouseMoveFilter(mouseEvent);
@@ -533,18 +547,27 @@ void PanelController::mouseMoveFilter(QMouseEvent *event)
         return;
     }
 
-    QRect screenGeom = Kephal::ScreenUtils::screenGeometry(containment()->screen());
+    DesktopCorona *corona = PlasmaApp::self()->corona();
+    const QRect screenGeom = corona->screenGeometry(containment()->screen());
 
     if (m_dragging == MoveButtonElement) {
 
         if (!screenGeom.contains(event->globalPos())) {
             //move panel to new screen if dragged there
-            int targetScreen = Kephal::ScreenUtils::screenId(event->globalPos());
+            int targetScreen = corona->screenId(event->globalPos());
+
             //kDebug() << "Moving panel from screen" << containment()->screen() << "to screen" << targetScreen;
             containment()->setScreen(targetScreen);
             return;
         }
 
+        if (location() == Plasma::BottomEdge || location() == Plasma::TopEdge) {
+            emit partialMove(QPoint(0, m_lastPos.y() - event->globalY()));
+        } else if (location() == Plasma::LeftEdge || location() == Plasma::RightEdge) {
+            emit partialMove(QPoint(m_lastPos.x() - event->globalX(), 0));
+        }
+        m_lastPos = event->globalPos();
+
         //create a dead zone so you can go across the middle without having it hop to one side
         float dzFactor = 0.35;
         QPoint offset = QPoint(screenGeom.width()*dzFactor,screenGeom.height()*dzFactor);
@@ -797,14 +820,14 @@ void PanelController::settingsPopup()
                 }
         }
 
-        QRect screenRect = Kephal::ScreenUtils::screenGeometry(containment()->screen());
+        const QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(containment()->screen());
 
-        if (pos.rx() + s.width() > screenRect.right()) {
-            pos.rx() -= ((pos.rx() + s.width()) - screenRect.right());
+        if (pos.rx() + s.width() > screenGeom.right()) {
+            pos.rx() -= ((pos.rx() + s.width()) - screenGeom.right());
         }
 
-        if (pos.ry() + s.height() > screenRect.bottom()) {
-            pos.ry() -= ((pos.ry() + s.height()) - screenRect.bottom());
+        if (pos.ry() + s.height() > screenGeom.bottom()) {
+            pos.ry() -= ((pos.ry() + s.height()) - screenGeom.bottom());
         }
 
         pos.rx() = qMax(0, pos.rx());
@@ -815,7 +838,7 @@ void PanelController::settingsPopup()
 
 void PanelController::syncRuler()
 {
-    QRect screenGeom = Kephal::ScreenUtils::screenGeometry(containment()->screen());
+    const QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(containment()->screen());
 
     switch (location()) {
         case Plasma::LeftEdge:
