@@ -22,6 +22,7 @@
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QGraphicsLinearLayout>
+#include <QPropertyAnimation>
 #include <QTimer>
 #ifdef Q_WS_X11
 #include <X11/Xatom.h>
@@ -40,12 +41,13 @@
 #include <Plasma/Theme>
 #include <Plasma/WindowEffects>
 
+#include <kephal/screens.h>
+
+#include "desktopcorona.h"
 #include "panelappletoverlay.h"
 #include "panelcontroller.h"
 #include "plasmaapp.h"
 
-#include <kephal/screens.h>
-
 class GlowBar : public QWidget
 {
 public:
@@ -212,7 +214,6 @@ public:
 #endif
 
         m_shadow = new Plasma::FrameSvg(this);
-        connect(m_shadow, SIGNAL(repaintNeeded()), m_panel, SLOT(checkShadow()), Qt::QueuedConnection);
     }
 
     void setSvg(const QString &path)
@@ -227,7 +228,6 @@ public:
         }
 
         m_shadow->setElementPrefix("shadow");
-
         adjustMargins(geometry());
     }
 
@@ -238,31 +238,40 @@ public:
 
     void adjustMargins(const QRect &geo)
     {
-        QRect screenRect = Kephal::ScreenUtils::screenGeometry(m_panel->screen());
+        QRect screenRect = PlasmaApp::self()->corona()->screenGeometry(m_panel->screen());
 
         Plasma::FrameSvg::EnabledBorders enabledBorders = Plasma::FrameSvg::AllBorders;
+        qreal left, top, right, bottom;
 
-        if (geo.left() <= screenRect.left()) {
+        m_shadow->getMargins(left, top, right, bottom);
+
+        if (geo.left() + left/2 <= screenRect.left()) {
             enabledBorders ^= Plasma::FrameSvg::LeftBorder;
         }
-        if (geo.top() <= screenRect.top()) {
+        if (geo.top() + top/2 <= screenRect.top()) {
             enabledBorders ^= Plasma::FrameSvg::TopBorder;
         }
-        if (geo.bottom() >= screenRect.bottom()) {
+        if (geo.bottom() - bottom/2 >= screenRect.bottom()) {
             enabledBorders ^= Plasma::FrameSvg::BottomBorder;
         }
-        if (geo.right() >= screenRect.right()) {
+        if (geo.right() - right/2 >= screenRect.right()) {
             enabledBorders ^= Plasma::FrameSvg::RightBorder;
         }
 
         m_shadow->setEnabledBorders(enabledBorders);
 
-        qreal left, top, right, bottom;
-
         m_shadow->getMargins(left, top, right, bottom);
         setContentsMargins(left, top, right, bottom);
     }
 
+    void adjustGeometry()
+    {
+        int left, right, top, bottom;
+        adjustMargins(m_panel->geometry());
+        getContentsMargins(&left, &top, &right, &bottom);
+        setGeometry(m_panel->geometry().adjusted(-left, -top, right, bottom));
+    }
+
 protected:
     bool event(QEvent *event)
     {
@@ -300,7 +309,6 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
       m_glowBar(0),
       m_mousePollTimer(0),
       m_strutsTimer(new QTimer(this)),
-      m_delayedUnhideTimer(new QTimer(this)),
       m_spacer(0),
       m_spacerIndex(-1),
       m_shadowWindow(0),
@@ -317,10 +325,6 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
     KWindowSystem::setType(winId(), NET::Dock);
     setWindowRole(QString("panel_%1").arg(id));
 
-    m_delayedUnhideTimer->setSingleShot(true);
-    m_delayedUnhideTimer->setInterval(200);
-    connect(m_delayedUnhideTimer, SIGNAL(timeout()), this, SLOT(delayedUnhide()));
-
     m_strutsTimer->setSingleShot(true);
     connect(m_strutsTimer, SIGNAL(timeout()), this, SLOT(updateStruts()));
 
@@ -346,15 +350,17 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
     KConfigGroup sizes = KConfigGroup(&viewConfig, "Sizes");
     m_lastHorizontal = isHorizontal();
 
-    const bool onScreen = panel->screen() < Kephal::ScreenUtils::numScreens();
-    const QRect screenRect = onScreen ?  Kephal::ScreenUtils::screenGeometry(panel->screen()) : QRect();
+    const bool onScreen = panel->screen() < PlasmaApp::self()->corona()->numScreens();
+    const QRect screenRect = onScreen ?  PlasmaApp::self()->corona()->screenGeometry(panel->screen()) : QRect();
     if (!onScreen) {
         resize(panel->size().toSize());
     }
     m_lastSeenSize = sizes.readEntry("lastsize", m_lastHorizontal ? screenRect.width() : screenRect.height());
 
     m_alignment = alignmentFilter((Qt::Alignment)viewConfig.readEntry("Alignment", (int)Qt::AlignLeft));
-    m_offset = viewConfig.readEntry("Offset", 0);
+    KConfigGroup sizeConfig(&viewConfig, (m_lastHorizontal ? "Horizontal" : "Vertical") +
+                            QString::number(m_lastSeenSize));
+    m_offset = sizeConfig.readEntry("offset", 0);
     setVisibilityMode((VisibilityMode)viewConfig.readEntry("panelVisibility", (int)m_visibilityMode));
 
     connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
@@ -366,6 +372,7 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
             this, SLOT(updatePanelGeometry()));
     connect(screens, SIGNAL(screenAdded(Kephal::Screen *)),
             this, SLOT(updateStruts()));
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()), Qt::QueuedConnection);
 }
 
 PanelView::~PanelView()
@@ -389,7 +396,7 @@ void PanelView::setContainment(Plasma::Containment *containment)
 
     Plasma::Containment *oldContainment = this->containment();
     if (oldContainment) {
-        disconnect(oldContainment, 0, this, 0);
+        disconnect(oldContainment);
     }
 
     connect(containment, SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(setStatus(Plasma::ItemStatus)));
@@ -398,6 +405,7 @@ void PanelView::setContainment(Plasma::Containment *containment)
     connect(containment, SIGNAL(appletAdded(Plasma::Applet *, const QPointF &)), this, SLOT(appletAdded(Plasma::Applet *)));
     connect(containment, SIGNAL(showAddWidgetsInterface(QPointF)), this, SLOT(showWidgetExplorer()));
     connect(containment, SIGNAL(screenChanged(int,int,Plasma::Containment*)), this, SLOT(pinchContainmentToCurrentScreen()));
+    connect(containment, SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)), this, SLOT(immutabilityChanged(Plasma::ImmutabilityType)));
 
     KConfigGroup viewIds(KGlobal::config(), "ViewIds");
 
@@ -413,7 +421,7 @@ void PanelView::setContainment(Plasma::Containment *containment)
     }
 
     // ensure we aren't overlapping other panels
-    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(containment->screen());
+    const QRect screenRect = PlasmaApp::self()->corona()->screenGeometry(containment->screen());
     const QRegion availGeom = PlasmaApp::self()->corona()->availableScreenRegion(containment->screen());
     const int w = containment->size().width();
     const int h = containment->size().height();
@@ -421,35 +429,6 @@ void PanelView::setContainment(Plasma::Containment *containment)
 
     View::setContainment(containment);
 
-    switch (location()) {
-        case Plasma::LeftEdge: {
-            QRect r = availGeom.intersected(QRect(0, m_offset, w, length)).boundingRect();
-            if (m_offset != r.top()) {
-                setOffset(r.top());
-            }
-        }
-        break;
-
-        case Plasma::RightEdge: {
-            QRect r = availGeom.intersected(QRect(screenRect.right() - w, m_offset, w, length)).boundingRect();
-            setOffset(r.top());
-        }
-        break;
-
-        case Plasma::TopEdge: {
-            QRect r = availGeom.intersected(QRect(m_offset, 0, length, h)).boundingRect();
-            setOffset(r.left());
-        }
-        break;
-
-        case Plasma::BottomEdge:
-        default: {
-            QRect r = availGeom.intersected(QRect(m_offset, screenRect.bottom() - h, length, h)).boundingRect();
-            setOffset(r.left());
-        }
-        break;
-    }
-
     // pinchContainment calls updatePanelGeometry for us
     pinchContainment(screenRect);
     m_lastMin = containment->minimumSize();
@@ -458,10 +437,15 @@ void PanelView::setContainment(Plasma::Containment *containment)
     kDebug() << "about to set the containment" << (QObject*)containment;
 
     updateStruts();
-
     checkShadow();
 }
 
+void PanelView::themeChanged()
+{
+    checkShadow();
+    recreateUnhideTrigger();
+}
+
 void PanelView::checkShadow()
 {
     if (KWindowSystem::compositingActive() && containment()->property("shadowPath").isValid()) {
@@ -473,10 +457,8 @@ void PanelView::checkShadow()
         KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
         KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
         m_shadowWindow->setSvg(containment()->property("shadowPath").toString());
-        int left, right, top, bottom;
-        m_shadowWindow->adjustMargins(geometry());
-        m_shadowWindow->getContentsMargins(&left, &top, &right, &bottom);
-        m_shadowWindow->setGeometry(geometry().adjusted(-left, -top, right, bottom));
+
+        m_shadowWindow->adjustGeometry();
         if (m_shadowWindow->isValid() && isVisible()) {
             m_shadowWindow->show();
         }
@@ -486,6 +468,19 @@ void PanelView::checkShadow()
     }
 }
 
+void PanelView::setPanelDragPosition(const QPoint &point)
+{
+    QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(containment()->screen());
+    QRect geom = geometry();
+    geom.translate(-point);
+    if (screenGeom.contains(geom)) {
+        move(pos() - point);
+        if (m_panelController) {
+            m_panelController->move(m_panelController->pos() - point);
+        }
+    }
+}
+
 void PanelView::setLocation(Plasma::Location location)
 {
     Plasma::Containment *c = containment();
@@ -504,7 +499,7 @@ void PanelView::setLocation(Plasma::Location location)
             // we're switching! swap the sizes about
             panelHeight = s.width();
             if (wasFullSize) {
-                QRect screenGeom = Kephal::ScreenUtils::screenGeometry(c->screen());
+                QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(c->screen());
                 panelWidth = screenGeom.width();
             } else {
                 panelWidth = s.height();
@@ -519,7 +514,7 @@ void PanelView::setLocation(Plasma::Location location)
             // we're switching! swap the sizes about
 
             if (wasFullSize) {
-                QRect screenGeom = Kephal::ScreenUtils::screenGeometry(c->screen());
+                QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(c->screen());
                 panelHeight = screenGeom.height();
             } else {
                 panelHeight = s.width();
@@ -546,10 +541,9 @@ void PanelView::setLocation(Plasma::Location location)
 #ifdef Q_WS_WIN
     appBarPosChanged();
 #endif
-    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(c->screen());
+    const QRect screenRect = PlasmaApp::self()->corona()->screenGeometry(c->screen());
     pinchContainment(screenRect);
     KWindowSystem::setOnAllDesktops(winId(), true);
-    //updatePanelGeometry();
     connect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
 }
 
@@ -614,7 +608,7 @@ void PanelView::updatePanelGeometry()
 
     kDebug() << "New panel geometry is" << c->geometry();
 
-    QSize size = c->size().toSize();
+    QSize size = c->size().expandedTo(c->minimumSize()).toSize();
     QRect geom(QPoint(0,0), size);
     int screen = c->screen();
 
@@ -623,7 +617,7 @@ void PanelView::updatePanelGeometry()
         screen = 0;
     }
 
-    QRect screenGeom = Kephal::ScreenUtils::screenGeometry(screen);
+    QRect screenGeom = PlasmaApp::self()->corona()->screenGeometry(screen);
 
     if (m_alignment != Qt::AlignCenter) {
         m_offset = qMax(m_offset, 0);
@@ -750,12 +744,23 @@ void PanelView::updatePanelGeometry()
         m_strutsTimer->stop();
         m_strutsTimer->start(STRUTSTIMERDELAY);
     } else {
-        setGeometry(geom);
-        if (m_shadowWindow) {
-            int left, right, top, bottom;
-            m_shadowWindow->adjustMargins(geometry());
-            m_shadowWindow->getContentsMargins(&left, &top, &right, &bottom);
-            m_shadowWindow->setGeometry(geometry().adjusted(-left, -top, right, bottom));
+        if (m_panelController && QPoint(pos() - geom.topLeft()).manhattanLength() > 100) {
+            resize(geom.size());
+            QPropertyAnimation *panelAnimation = new QPropertyAnimation(this, "pos", this);
+            panelAnimation->setEasingCurve(QEasingCurve::InOutQuad);
+            panelAnimation->setDuration(300);
+            panelAnimation->setStartValue(pos());
+            panelAnimation->setEndValue(geom.topLeft());
+            panelAnimation->start(QAbstractAnimation::DeleteWhenStopped);
+
+            QPropertyAnimation *controllerAnimation = new QPropertyAnimation(m_panelController, "pos", m_panelController);
+            controllerAnimation->setEasingCurve(QEasingCurve::InOutQuad);
+            controllerAnimation->setDuration(300);
+            controllerAnimation->setStartValue(m_panelController->pos());
+            controllerAnimation->setEndValue(m_panelController->positionForPanelGeometry(geom));
+            controllerAnimation->start(QAbstractAnimation::DeleteWhenStopped);
+        } else {
+            setGeometry(geom);
         }
     }
 
@@ -764,7 +769,6 @@ void PanelView::updatePanelGeometry()
 
     //update the panel controller location position and size
     if (m_panelController) {
-
         m_panelController->setLocation(c->location());
 
         foreach (PanelAppletOverlay *o, m_appletOverlays) {
@@ -784,7 +788,7 @@ bool PanelView::isHorizontal() const
 void PanelView::pinchContainmentToCurrentScreen()
 {
     kDebug() << "pinching to current screen";
-    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(containment()->screen());
+    const QRect screenRect = PlasmaApp::self()->corona()->screenGeometry(containment()->screen());
     pinchContainment(screenRect);
 }
 
@@ -794,8 +798,9 @@ void PanelView::pinchContainment(const QRect &screenGeom)
     disconnect(this, SIGNAL(sceneRectAboutToChange()), this, SLOT(pinchContainmentToCurrentScreen()));
     bool horizontal = isHorizontal();
 
-    int sw = screenGeom.width();
-    int sh = screenGeom.height();
+    const int oldOffset = m_offset;
+    const int sw = screenGeom.width();
+    const int sh = screenGeom.height();
 
     Plasma::Containment *c = containment();
     QSizeF min = c->minimumSize();
@@ -863,56 +868,6 @@ void PanelView::pinchContainment(const QRect &screenGeom)
     // the minimum and maximum sizes.
 
     //kDebug() << "checking panel" << c->geometry() << "against" << screenGeom;
-    if (horizontal) {
-        //kDebug() << "becoming horizontal with" << m_offset << min.width() << max.width() << sw;
-        if (m_offset + min.width() > sw) {
-            //kDebug() << "min size is too wide!";
-            if (min.width() > sw) {
-                c->setMinimumSize(sw, min.height());
-            } else {
-                m_offset = sw - int(min.width());
-            }
-        }
-
-        // if the maximum-size is 0, set it to 100%
-        if (max.width() <= 0) {
-            c->setMaximumSize(sw, max.height());
-            max = c->maximumSize();
-        }
-
-        if (m_offset + max.width() > sw) {
-            //kDebug() << "max size is too wide!";
-            if (max.width() > sw) {
-                c->setMaximumSize(sw, max.height());
-            } else {
-                m_offset = sw - int(max.width());
-            }
-        }
-    } else {
-        if (m_offset + min.height() > sh) {
-            //kDebug() << "min size is too tall!";
-            if (min.height() > sh) {
-                c->setMinimumSize(min.width(), sh);
-            } else {
-                m_offset = sh - int(min.height());
-            }
-        }
-
-        // if the maximum-size is 0, set it to 100%
-        if (max.height() <= 0) {
-            c->setMaximumSize(max.width(), sh);
-            max = c->maximumSize();
-        }
-
-        if (m_offset + max.height() > sh) {
-            //kDebug() << "max size is too tall!";
-            if (max.height() > sh) {
-                c->setMaximumSize(max.width(), sh);
-            } else {
-                m_offset = sh - int(max.height());
-            }
-        }
-    }
 
     // resize to max if for some reason the size is empty
     // otherwise its not possible to interact with the panel at all
@@ -920,16 +875,23 @@ void PanelView::pinchContainment(const QRect &screenGeom)
         c->resize(max);
     }
 
-    if (m_lastHorizontal != horizontal ||
-        m_lastSeenSize != (horizontal ? sw : sh)) {
-        m_lastHorizontal = horizontal;
-        m_lastSeenSize = (horizontal ? sw : sh);
+    // write to the config file if the size has changed, or if we haven't recorded the lastsize
+    // previously which ensures we'll always have a value even after first run
+    const bool writeConfig = m_lastSeenSize != (horizontal ? sw : sh) || !sizes.hasKey("lastsize");
+    m_lastHorizontal = horizontal;
+    m_lastSeenSize = (horizontal ? sw : sh);
+    if (writeConfig) {
         sizes.writeEntry("lastsize", m_lastSeenSize);
         configNeedsSaving();
     }
 
     updatePanelGeometry();
 
+    if (m_offset != oldOffset) {
+        sizes.writeEntry("offset", m_lastSeenSize);
+        configNeedsSaving();
+    }
+
     if (m_panelController) {
         m_panelController->setOffset(m_offset);
     }
@@ -942,14 +904,15 @@ void PanelView::pinchContainment(const QRect &screenGeom)
 void PanelView::setOffset(int newOffset)
 {
     m_offset = newOffset;
-    KConfigGroup viewConfig = config();
-    viewConfig.writeEntry("Offset", m_offset);
-
-    containment()->update();
 
     //TODO: do we ever need to worry about pinching here, or
     //      do we just assume that the offset is always < screenSize - containmentSize?
     updatePanelGeometry();
+
+    KConfigGroup viewConfig = config();
+    viewConfig = KConfigGroup(&viewConfig, (m_lastHorizontal ? "Horizontal" : "Vertical") +
+                              QString::number(m_lastSeenSize));
+    viewConfig.writeEntry("offset", m_offset);
     configNeedsSaving();
 }
 
@@ -971,6 +934,14 @@ Qt::Alignment PanelView::alignment() const
     return m_alignment;
 }
 
+void PanelView::immutabilityChanged(Plasma::ImmutabilityType immutability)
+{
+    if (immutability != Plasma::Mutable) {
+        delete m_panelController;
+        m_panelController = 0;
+    }
+}
+
 void PanelView::togglePanelController()
 {
     //kDebug();
@@ -991,6 +962,7 @@ void PanelView::togglePanelController()
 
         connect(m_panelController, SIGNAL(destroyed(QObject*)), this, SLOT(editingComplete()));
         connect(m_panelController, SIGNAL(offsetChanged(int)), this, SLOT(setOffset(int)));
+        connect(m_panelController, SIGNAL(partialMove(const QPoint&)), this, SLOT(setPanelDragPosition(const QPoint&)));
         connect(m_panelController, SIGNAL(alignmentChanged(Qt::Alignment)), this, SLOT(setAlignment(Qt::Alignment)));
         connect(m_panelController, SIGNAL(locationChanged(Plasma::Location)), this, SLOT(setLocation(Plasma::Location)));
         connect(m_panelController, SIGNAL(panelVisibilityModeChanged(PanelView::VisibilityMode)), this, SLOT(setVisibilityMode(PanelView::VisibilityMode)));
@@ -1052,7 +1024,7 @@ void PanelView::editingComplete()
     updateStruts();
 
     if (m_visibilityMode == LetWindowsCover || m_visibilityMode == AutoHide) {
-        hideIfNotInUse();
+        startAutoHide();
     }
 }
 
@@ -1088,13 +1060,19 @@ void PanelView::updateStruts()
     NETExtendedStrut strut;
 
     if (m_visibilityMode == NormalPanel) {
-        QRect thisScreen = Kephal::ScreenUtils::screenGeometry(containment()->screen());
-        QRect wholeScreen = Kephal::ScreenUtils::desktopGeometry();
+        const QRect thisScreen = PlasmaApp::self()->corona()->screenGeometry(containment()->screen());
+        const QRect wholeScreen = Kephal::ScreenUtils::desktopGeometry();
 
         //Extended struts against a screen edge near to another screen are really harmful, so windows maximized under the panel is a lesser pain
         //TODO: force "windows can cover" in those cases?
-        foreach (Kephal::Screen *screen, Kephal::Screens::self()->screens()) {
-            QRect otherScreen = screen->geom();
+        const int numScreens = PlasmaApp::self()->corona()->numScreens();
+        for (int i = 0; i < numScreens; ++i) {
+            if (i == containment()->screen()) {
+                continue;
+            }
+
+            const QRect otherScreen = PlasmaApp::self()->corona()->screenGeometry(i);
+
             switch (location())
             {
             case Plasma::TopEdge:
@@ -1221,19 +1199,6 @@ void PanelView::migrateTo(int screenId)
     setScreen(screenId);
 }
 
-void PanelView::enterEvent(QEvent *event)
-{
-    // allow unhiding to happen again even if we were delay-unhidden
-    m_delayedUnhideTs = QTime();
-/*
-// handy for debugging :)
-    if (containment()) {
-        kDebug() << sceneRect() << containment()->geometry();
-    }
-*/
-    Plasma::View::enterEvent(event);
-}
-
 void PanelView::showWidgetExplorer()
 {
     if (!containment()) {
@@ -1241,7 +1206,11 @@ void PanelView::showWidgetExplorer()
     }
 
     if (!m_panelController) {
-        PlasmaApp::self()->showWidgetExplorer(screen(), containment());
+        m_editing = true;
+        ControllerWindow *controller = PlasmaApp::self()->showWidgetExplorer(screen(), containment());
+        connect(controller, SIGNAL(destroyed(QObject*)), this, SLOT(editingComplete()), Qt::UniqueConnection);
+    } else {
+        m_panelController->showWidgetExplorer();
     }
 }
 
@@ -1252,6 +1221,10 @@ void PanelView::moveEvent(QMoveEvent *event)
     m_strutsTimer->start(STRUTSTIMERDELAY);
     recreateUnhideTrigger();
 
+    if (m_shadowWindow) {
+        m_shadowWindow->adjustGeometry();
+    }
+
     if (containment()) {
         foreach (Plasma::Applet *applet, containment()->applets()) {
             applet->updateConstraints(Plasma::PopupConstraint);
@@ -1270,6 +1243,10 @@ void PanelView::resizeEvent(QResizeEvent *event)
     appBarPosChanged();
 #endif
 
+    if (m_shadowWindow) {
+        m_shadowWindow->adjustGeometry();
+    }
+
     if (containment()) {
         foreach (Plasma::Applet *applet, containment()->applets()) {
             applet->updateConstraints(Plasma::PopupConstraint);
@@ -1277,16 +1254,6 @@ void PanelView::resizeEvent(QResizeEvent *event)
     }
 }
 
-void PanelView::hideIfNotInUse()
-{
-    //kDebug() << m_delayedUnhideTs.elapsed() << geometry().contains(QCursor::pos()) << hasPopup();
-    //TODO: is 5s too long? not long enough?
-    if ((m_delayedUnhideTs.isNull() || m_delayedUnhideTs.elapsed() > 5000) &&
-        !geometry().adjusted(-10, -10, 10, 10).contains(QCursor::pos()) && !hasPopup()) {
-        startAutoHide();
-    }
-}
-
 void PanelView::updateHinter()
 {
 #ifdef Q_WS_X11
@@ -1385,7 +1352,6 @@ bool PanelView::hasPopup()
 void PanelView::unhide(bool destroyTrigger)
 {
     //kill the unhide stuff
-    m_delayedUnhideTimer->stop();
     hideHinter();
     if (destroyTrigger) {
         destroyUnhideTrigger();
@@ -1395,7 +1361,6 @@ void PanelView::unhide(bool destroyTrigger)
     if (!isVisible()) {
         Plasma::WindowEffects::slideWindow(this, location());
         show();
-        KWindowSystem::setOnAllDesktops(winId(), true);
 
         if (m_shadowWindow && m_shadowWindow->isValid()) {
             Plasma::WindowEffects::slideWindow(m_shadowWindow, location());
@@ -1405,6 +1370,8 @@ void PanelView::unhide(bool destroyTrigger)
         }
     }
 
+    KWindowSystem::setOnAllDesktops(winId(), true);
+
     //non-hiding panels stop here
     if (m_visibilityMode == NormalPanel || m_visibilityMode == WindowsGoBelow) {
         return;
@@ -1415,8 +1382,8 @@ void PanelView::unhide(bool destroyTrigger)
         m_mousePollTimer = new QTimer(this);
     }
 
-    connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideIfNotInUse()), Qt::UniqueConnection);
-    m_mousePollTimer->start(200);
+    connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(startAutoHide()), Qt::UniqueConnection);
+    m_mousePollTimer->start(500);
 
     //avoid hide-show loops
     if (m_visibilityMode == LetWindowsCover) {
@@ -1437,24 +1404,13 @@ void PanelView::setStatus(Plasma::ItemStatus newStatus)
 void PanelView::checkUnhide(Plasma::ItemStatus newStatus)
 {
     //kDebug() << "================= got a new status: " << newStatus << Plasma::ActiveStatus;
-
     if (newStatus > Plasma::ActiveStatus) {
-        if (!m_delayedUnhideTimer->isActive()) {
-            m_delayedUnhideTimer->start();
-        }
-
+        unhide();
     } else {
-        m_delayedUnhideTimer->stop();
+        startAutoHide();
     }
 }
 
-void PanelView::delayedUnhide()
-{
-    m_delayedUnhideTs.start();
-    //kDebug() << "starting at:" << m_delayedUnhideTs << m_delayedUnhideTs.elapsed();
-    unhide(true);
-}
-
 void PanelView::unhide()
 {
     unhide(true);
@@ -1467,9 +1423,19 @@ void PanelView::resetTriggerEnteredSuppression()
 
 void PanelView::startAutoHide()
 {
+    //TODO: is 5s too long? not long enough?
+    if (m_editing || (containment() && containment()->status() > Plasma::ActiveStatus) ||
+        geometry().adjusted(-10, -10, 10, 10).contains(QCursor::pos()) ||
+        hasPopup()) {
+        if (!m_mousePollTimer) {
+            leaveEvent(0);
+        }
+        return;
+    }
+
     if (m_mousePollTimer) {
         m_mousePollTimer->stop();
-        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideIfNotInUse()));
+        disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(startAutoHide()));
     }
 
     if (m_visibilityMode == LetWindowsCover) {
@@ -1501,11 +1467,14 @@ void PanelView::leaveEvent(QEvent *event)
             m_mousePollTimer = new QTimer(this);
         }
 
-        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(hideIfNotInUse()), Qt::UniqueConnection);
-        m_mousePollTimer->start(200);
+        connect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(startAutoHide()), Qt::UniqueConnection);
+        m_mousePollTimer->start(500);
     }
 
-    Plasma::View::leaveEvent(event);
+    if (event) {
+        // startAutoHide calls this with a null event pointer, so we have to check it
+        Plasma::View::leaveEvent(event);
+    }
 }
 
 void PanelView::drawBackground(QPainter *painter, const QRectF &rect)
