@@ -133,12 +133,6 @@ public Q_SLOTS:
      */
     void unhide();
 
-    /**
-     * unhides the panel if it is hidden due to a delayed acdtions, such as a status change
-     * This always destroys the unhide trigger
-     */
-    void delayedUnhide();
-
     /**
      * Decides whether to hide or not based on the status of the containment
      */
@@ -205,6 +199,9 @@ public Q_SLOTS:
 
     void updateStruts();
 
+    bool migratedFrom(int screenId) const;
+    void migrateTo(int screenId);
+
 protected:
     void moveEvent(QMoveEvent *event);
     void resizeEvent(QResizeEvent *event);
@@ -214,7 +211,6 @@ protected:
     bool event(QEvent *event);
     void dragLeaveEvent(QDragLeaveEvent *event);
     void dropEvent(QDropEvent *event);
-    void enterEvent(QEvent *event);
 
 private:
     void createUnhideTrigger();
@@ -236,6 +232,7 @@ private:
 #endif
 
 private Q_SLOTS:
+    void immutabilityChanged(Plasma::ImmutabilityType immutability);
     void togglePanelController();
     void showWidgetExplorer();
     void editingComplete();
@@ -243,10 +240,7 @@ private Q_SLOTS:
     void overlayMoved(PanelAppletOverlay *overlay);
     void panelDeleted();
     void startAutoHide();
-    /**
-     * hides the panel if there's nothing keeping it open
-     */
-    void hideIfNotInUse();
+
     /**
      * update the appearance of the unhide hinter
      * and get rid of it if the mouse has gone elsewhere
@@ -265,15 +259,16 @@ private Q_SLOTS:
      */
     void checkShadow();
 
+    void themeChanged();
+    void setPanelDragPosition(const QPoint &point);
+
 private:
     Plasma::Svg *m_background;
     PanelController *m_panelController;
     QSet<PanelAppletOverlay*> m_appletOverlays;
     GlowBar *m_glowBar;
-    QTime m_delayedUnhideTs;
     QTimer *m_mousePollTimer;
     QTimer *m_strutsTimer;
-    QTimer *m_delayedUnhideTimer;
     QTimeLine *m_timeLine;
     QGraphicsWidget *m_spacer;
     int m_spacerIndex;
