@@ -55,6 +55,7 @@
 #include <KDebug>
 #include <KCmdLineArgs>
 #include <KGlobalAccel>
+#include <KGlobalSettings>
 #include <KNotification>
 #include <KWindowSystem>
 
@@ -170,9 +171,24 @@ PlasmaApp::PlasmaApp()
     // Calculate the size required to hold background pixmaps for all screens.
     // Add 10% so that other (smaller) pixmaps can also be cached.
     int cacheSize = 0;
-    for (int i = 0; i < Kephal::ScreenUtils::numScreens(); i++) {
-        QSize size = Kephal::ScreenUtils::screenSize(i);
+
+    if (KGlobalSettings::isMultiHead()) {
+        int id = 0;
+#ifdef Q_WS_X11
+        Display *dpy = XOpenDisplay(NULL);
+        if (dpy) {
+            id = DefaultScreen(dpy);
+            XCloseDisplay(dpy);
+        }
+#endif
+        QSize size = Kephal::ScreenUtils::screenSize(id);
         cacheSize += 4 * size.width() * size.height() / 1024;
+    } else {
+        const int numScreens = Kephal::ScreenUtils::numScreens();
+        for (int i = 0; i < numScreens; i++) {
+            QSize size = Kephal::ScreenUtils::screenSize(i);
+            cacheSize += 4 * size.width() * size.height() / 1024;
+        }
     }
     cacheSize += cacheSize / 10;
 
@@ -233,7 +249,7 @@ PlasmaApp::PlasmaApp()
 
     KAction *showAction = new KAction(this);
     showAction->setText(i18n("Show Dashboard"));
-    showAction->setObjectName("Show Dashboard"); // NO I18N
+    showAction->setObjectName( QLatin1String("Show Dashboard" )); // NO I18N
     showAction->setGlobalShortcut(KShortcut(Qt::CTRL + Qt::Key_F12));
     connect(showAction, SIGNAL(triggered()), this, SLOT(toggleDashboard()));
 
@@ -275,12 +291,9 @@ void PlasmaApp::setupDesktop()
     m_XdndVersionAtom = (Atom)xdndversion;
 #endif
 
-    m_primaryScreen = Kephal::ScreenUtils::primaryScreenId();
-
     // intialize the default theme and set the font
     Plasma::Theme *theme = Plasma::Theme::defaultTheme();
     theme->setFont(AppSettings::desktopFont());
-    connect(theme, SIGNAL(themeChanged()), this, SLOT(compositingChanged()));
 
     // this line initializes the corona.
     corona();
@@ -461,35 +474,35 @@ QList<PanelView*> PlasmaApp::panelViews() const
     return m_panels;
 }
 
-void PlasmaApp::showWidgetExplorer(int screen, Plasma::Containment *containment)
+ControllerWindow *PlasmaApp::showWidgetExplorer(int screen, Plasma::Containment *containment)
 {
-    showController(screen, containment, true);
+    return showController(screen, containment, true);
 }
 
 //FIXME it'd be easier if we knew which containment triggered this action
-void PlasmaApp::showActivityManager()
+ControllerWindow *PlasmaApp::showActivityManager()
 {
     //try to find the "active" containment
-    int currentScreen = Kephal::ScreenUtils::screenId(QCursor::pos());
+    int currentScreen = m_corona->screenId(QCursor::pos());
     int currentDesktop = -1;
     if (AppSettings::perVirtualDesktopViews()) {
         currentDesktop = KWindowSystem::currentDesktop()-1;
     }
-    Plasma::Containment *containment=m_corona->containmentForScreen(currentScreen, currentDesktop);
+    Plasma::Containment *containment = m_corona->containmentForScreen(currentScreen, currentDesktop);
 
-    showController(currentScreen, containment, false);
+    return showController(currentScreen, containment, false);
 }
 
-void PlasmaApp::showActivityManager(int screen, Plasma::Containment *containment)
+ControllerWindow *PlasmaApp::showActivityManager(int screen, Plasma::Containment *containment)
 {
-    showController(screen, containment, false);
+    return showController(screen, containment, false);
 }
 
-void PlasmaApp::showController(int screen, Plasma::Containment *containment, bool widgetExplorerMode)
+ControllerWindow *PlasmaApp::showController(int screen, Plasma::Containment *containment, bool widgetExplorerMode)
 {
     if (!containment) {
         kDebug() << "no containment";
-        return;
+        return 0;
     }
 
     QWeakPointer<ControllerWindow> controllerPtr = m_widgetExplorers.value(screen);
@@ -502,38 +515,24 @@ void PlasmaApp::showController(int screen, Plasma::Containment *containment, boo
     }
 
     controller->setContainment(containment);
+    if (!containment || containment->screen() != screen) {
+        controller->setScreen(screen);
+    }
+
     controller->setLocation(containment->location());
+
     if (widgetExplorerMode) {
         controller->showWidgetExplorer();
     } else {
         controller->showActivityManager();
     }
-    controller->resize(controller->sizeHint());
-
-    bool moved = false;
-    if (isPanelContainment(containment)) {
-        // try to align it with the appropriate panel view
-        foreach (PanelView * panel, m_panels) {
-            if (panel->containment() == containment) {
-                controller->move(controller->positionForPanelGeometry(panel->geometry()));
-                moved = true;
-                break;
-            }
-        }
-    }
-
-    if (!moved) {
-        // set it to the bottom of the screen as we have no better hints to go by
-        QRect geom = Kephal::Screens::self()->screen(screen)->geom();
-        controller->resize(controller->sizeHint());
-        controller->setGeometry(geom.x(), geom.bottom() - controller->height(), geom.width(), controller->height());
-    }
 
     controller->show();
     Plasma::WindowEffects::slideWindow(controller, Plasma::BottomEdge);
     KWindowSystem::setOnAllDesktops(controller->winId(), true);
-    KWindowSystem::activateWindow(controller->winId());
+    QTimer::singleShot(0, controller, SLOT(activate()));
     KWindowSystem::setState(controller->winId(), NET::SkipTaskbar | NET::SkipPager | NET::Sticky | NET::KeepAbove);
+    return controller;
 }
 
 void PlasmaApp::hideController(int screen)
@@ -544,15 +543,6 @@ void PlasmaApp::hideController(int screen)
     }
 }
 
-void PlasmaApp::compositingChanged()
-{
-#ifdef Q_WS_X11
-    foreach (PanelView *panel, m_panels) {
-        panel->recreateUnhideTrigger();
-    }
-#endif
-}
-
 #ifdef Q_WS_X11
 PanelView *PlasmaApp::findPanelForTrigger(WId trigger) const
 {
@@ -664,7 +654,7 @@ void PlasmaApp::screenRemoved(int id)
     QList<Kephal::Screen *> screens = Kephal::Screens::self()->screens();
     screens.removeAll(primary);
 
-    // Now we process panels: if there is room on another sreen for the panel,
+    // Now we process panels: if there is room on another screen for the panel,
     // we migrate the panel there, otherwise the view is deleted. The primary
     // screen is preferred in all cases.
     QMutableListIterator<PanelView*> pIt(m_panels);
@@ -684,7 +674,7 @@ void PlasmaApp::screenRemoved(int id)
             }
 
             if (moveTo) {
-                panel->containment()->setScreen(moveTo->id());
+                panel->migrateTo(moveTo->id());
             } else {
                 pIt.remove();
                 delete panel;
@@ -704,6 +694,12 @@ void PlasmaApp::screenAdded(Kephal::Screen *screen)
             m_panelViewCreationTimer.start();
         }
     }
+
+    foreach (PanelView *view, m_panels) {
+        if (view->migratedFrom(screen->id())) {
+            view->migrateTo(screen->id());
+        }
+    }
 }
 
 bool PlasmaApp::canRelocatePanel(PanelView * view, Kephal::Screen *screen)
@@ -740,7 +736,7 @@ bool PlasmaApp::canRelocatePanel(PanelView * view, Kephal::Screen *screen)
         if (pv != view &&
             pv->screen() == screen->id() &&
             pv->location() == view->location() &&
-            !pv->geometry().intersects(newGeom)) {
+            pv->geometry().intersects(newGeom)) {
             return false;
             break;
         }
@@ -764,7 +760,7 @@ DesktopView* PlasmaApp::viewForScreen(int screen, int desktop) const
     return 0;
 }
 
-Plasma::Corona* PlasmaApp::corona()
+DesktopCorona* PlasmaApp::corona()
 {
     if (!m_corona) {
         QTime t;
@@ -773,8 +769,6 @@ Plasma::Corona* PlasmaApp::corona()
         connect(c, SIGNAL(containmentAdded(Plasma::Containment*)),
                 this, SLOT(containmentAdded(Plasma::Containment*)));
         connect(c, SIGNAL(configSynced()), this, SLOT(syncConfig()));
-        connect(c, SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)),
-                this, SLOT(updateActions(Plasma::ImmutabilityType)));
         connect(c, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
                 this, SLOT(containmentScreenOwnerChanged(int,int,Plasma::Containment*)));
 
@@ -860,6 +854,10 @@ void PlasmaApp::suspendStartup(bool suspend)
 
 bool PlasmaApp::isPanelContainment(Plasma::Containment *containment)
 {
+    if (!containment) {
+        return false;
+    }
+
     Plasma::Containment::Type t = containment->containmentType();
 
     return t == Plasma::Containment::PanelContainment ||
@@ -884,7 +882,7 @@ void PlasmaApp::createView(Plasma::Containment *containment)
         m_panelsWaiting << containment;
         m_panelViewCreationTimer.start();
     } else if (containment->screen() > -1 &&
-               containment->screen() < Kephal::ScreenUtils::numScreens()) {
+               containment->screen() < m_corona->numScreens()) {
         if (AppSettings::perVirtualDesktopViews()) {
             if (containment->desktop() < 0 ||
                 containment->desktop() > KWindowSystem::numberOfDesktops() - 1) {
@@ -919,11 +917,6 @@ void PlasmaApp::createWaitingPanels()
     const QList<QWeakPointer<Plasma::Containment> > containments = m_panelsWaiting;
     m_panelsWaiting.clear();
 
-    Kephal::Screen *primary = Kephal::Screens::self()->primaryScreen();
-    QList<Kephal::Screen *> screens = Kephal::Screens::self()->screens();
-    screens.removeAll(primary);
-
-    QList<Plasma::Containment *> relocationCandidates;
     foreach (QWeakPointer<Plasma::Containment> containmentPtr, containments) {
         Plasma::Containment *containment = containmentPtr.data();
         if (!containment) {
@@ -941,17 +934,33 @@ void PlasmaApp::createWaitingPanels()
         }
 
         // try to relocate the panel if it is on a now-non-existent screen
-        if (containment->screen() >= Kephal::ScreenUtils::numScreens()) {
-            relocationCandidates << containment;
+        if (containment->screen() >= m_corona->numScreens()) {
+            m_panelRelocationCandidates << containment;
             continue;
         }
 
         createPanelView(containment);
     }
 
+    if (!m_panelRelocationCandidates.isEmpty()) {
+        QTimer::singleShot(0, this, SLOT(relocatePanels()));
+    }
+}
+
+void PlasmaApp::relocatePanels()
+{
     // we go through relocatables last so that all other panels can be set up first,
     // preventing panel creation ordering to trip up the canRelocatePanel algorithm
-    foreach (Plasma::Containment *containment, relocationCandidates) {
+    Kephal::Screen *primary = Kephal::Screens::self()->primaryScreen();
+    QList<Kephal::Screen *> screens = Kephal::Screens::self()->screens();
+    screens.removeAll(primary);
+
+    foreach (QWeakPointer<Plasma::Containment> c, m_panelRelocationCandidates) {
+        Plasma::Containment *containment = c.data();
+        if (!containment) {
+            continue;
+        }
+
         Kephal::Screen *moveTo = 0;
         PanelView *panelView = createPanelView(containment);
         if (canRelocatePanel(panelView, primary)) {
@@ -966,12 +975,14 @@ void PlasmaApp::createWaitingPanels()
         }
 
         if (moveTo) {
-            containment->setScreen(moveTo->id(), -1);
+            panelView->migrateTo(moveTo->id());
         } else {
             m_panels.removeAll(panelView);
             delete panelView;
         }
     }
+
+    m_panelRelocationCandidates.clear();
 }
 
 PanelView *PlasmaApp::createPanelView(Plasma::Containment *containment)
@@ -1005,7 +1016,7 @@ void PlasmaApp::createWaitingDesktops()
             }
 
             const int screen = containment->screen();
-            if (screen >= Kephal::ScreenUtils::numScreens()) {
+            if (screen >= m_corona->numScreens() || screen < 0) {
                 kDebug() << "not creating a view on screen" << screen << "as it does not exist";
                 continue;
             }
@@ -1019,7 +1030,7 @@ void PlasmaApp::createWaitingDesktops()
             }
 
             kDebug() << "creating a new view for" << containment->screen() << containment->desktop()
-                     << "and we have" << Kephal::ScreenUtils::numScreens() << "screens";
+                     << "and we have" << m_corona->numScreens() << "screens";
 
             // we have a new screen. neat.
             view = new DesktopView(containment, id, 0);
@@ -1060,6 +1071,16 @@ void PlasmaApp::containmentAdded(Plasma::Containment *containment)
         if (a) {
             delete a; //activities handle removal now
         }
+        if (!(m_loadingActivity.isEmpty() || m_corona->offscreenWidgets().contains(containment))) {
+            Plasma::Context *context = containment->context();
+            if (context->currentActivityId().isEmpty()) {
+                //kDebug() << "@#$%@#$%@#$%@#$%#@$#@%@$#^%$&^$^$%#%$";
+                //kDebug() << "script->containment->activity";
+                Activity *activity = m_corona->activity(m_loadingActivity);
+                Q_ASSERT(activity);
+                activity->replaceContainment(containment);
+            }
+        }
 
         if (containment->containmentType() == Plasma::Containment::DesktopContainment) {
             foreach (QAction *action, m_corona->actions()) {
@@ -1076,12 +1097,13 @@ void PlasmaApp::containmentAdded(Plasma::Containment *containment)
 void PlasmaApp::containmentScreenOwnerChanged(int wasScreen, int isScreen, Plasma::Containment *containment)
 {
     Q_UNUSED(wasScreen)
-    kDebug() << "@@@was" << wasScreen << "is" << isScreen << (QObject*)containment;
+    kDebug() << "@@@was" << wasScreen << "is" << isScreen << (QObject*)containment << m_desktops.count();
 
     if (isScreen < 0) {
         kDebug() << "@@@screen<0";
         return;
     }
+
     if (isPanelContainment(containment)) {
         kDebug() << "@@@isPanel";
         return;
@@ -1163,6 +1185,9 @@ void PlasmaApp::cloneCurrentActivity()
 //TODO accomodate activities
 void PlasmaApp::setPerVirtualDesktopViews(bool perDesktopViews)
 {
+    if (perDesktopViews == perVirtualDesktopViews()) {
+        return;
+    }
     AppSettings::setPerVirtualDesktopViews(perDesktopViews);
     AppSettings::self()->writeConfig();
 
@@ -1231,6 +1256,10 @@ void PlasmaApp::setFixedDashboard(bool fixedDashboard)
             c = m_corona->addContainment("desktopDashboard");
         }
 
+        //everything failed? probably a badly packaged plasma
+        if (!c) {
+            return;
+        }
         m_corona->addOffscreenWidget(c);
     }
 
@@ -1269,10 +1298,6 @@ void PlasmaApp::panelRemoved(QObject *panel)
     m_panels.removeAll((PanelView *)panel);
 }
 
-void PlasmaApp::updateActions(Plasma::ImmutabilityType immutability)
-{
-}
-
 void PlasmaApp::remotePlasmoidAdded(Plasma::PackageMetadata metadata)
 {
     //kDebug();
@@ -1313,15 +1338,29 @@ void PlasmaApp::plasmoidAccessFinished(Plasma::AccessAppletJob *job)
 void PlasmaApp::createActivity(const QString &plugin)
 {
     KActivityController controller;
-    QString id = controller.addActivity(i18n("unnamed"));
+    QString id = controller.addActivity(i18n("New Activity"));
 
     Activity *a = m_corona->activity(id);
-    if (!a) {
-        kDebug() << "!*!*!*!*!*!*!*!*!*!**!*!*!*!!*!*!*!*!*!*";
-    }
+    Q_ASSERT(a);
     a->setDefaultPlugin(plugin);
 
     controller.setCurrentActivity(id);
 }
 
+void PlasmaApp::createActivityFromScript(const QString &script, const QString &name, const QString &icon)
+{
+    KActivityController controller;
+    m_loadingActivity = controller.addActivity(name);
+    Activity *a = m_corona->activity(m_loadingActivity);
+    Q_ASSERT(a);
+    a->setIcon(icon);
+
+    //kDebug() << "$$$$$$$$$$$$$$$$ begin script for" << m_loadingActivity;
+    m_corona->evaluateScripts(QStringList() << script);
+    //kDebug() << "$$$$$$$$$$$$$$$$ end script for" << m_loadingActivity;
+
+    controller.setCurrentActivity(m_loadingActivity);
+    m_loadingActivity.clear();
+}
+
 #include "plasmaapp.moc"
