@@ -50,7 +50,6 @@
 
 #include <kworkspace/kworkspace.h>
 
-#include <solid/control/powermanager.h>
 #include <solid/powermanagement.h>
 
 #include <Plasma/Animator>
@@ -68,6 +67,7 @@
 #include <Plasma/Theme>
 #include <Plasma/ToolTipContent>
 #include <Plasma/ToolTipManager>
+#include <qdbuspendingreply.h>
 
 
 typedef QHash<QString, QVariant> VariantDict;
@@ -88,7 +88,7 @@ Battery::Battery(QObject *parent, const QVariantList &args)
       m_minutes(0),
       m_hours(0),
       m_theme(0),
-      m_availableProfiles(QStringList()),
+      m_availableProfiles(StringStringMap()),
       m_numOfBattery(0),
       m_acAdapterPlugged(false),
       m_remainingMSecs(0),
@@ -124,6 +124,7 @@ Battery::Battery(QObject *parent, const QVariantList &args)
     m_acAnimation->setStartValue((qreal)(0.0));
     m_acAnimation->setEndValue((qreal)(1.0));
     m_acAnimation->setEasingCurve(QEasingCurve::OutBack);
+    m_acAnimation->setDirection(QAbstractAnimation::Backward);
     connect(m_acAnimation, SIGNAL(finished()), this, SLOT(updateBattery()));
 }
 
@@ -150,7 +151,6 @@ void Battery::init()
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(readColors()));
     connect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()), SLOT(readColors()));
     connect(KGlobalSettings::self(), SIGNAL(appearanceChanged()), SLOT(setupFonts()));
-    connect(KGlobalSettings::self(), SIGNAL(appearanceChanged()), SLOT(setupFonts()));
 
     const QStringList& battery_sources = dataEngine("powermanagement")->query("Battery")["Sources"].toStringList();
 
@@ -160,6 +160,9 @@ void Battery::init()
         dataUpdated(battery_source, dataEngine("powermanagement")->query(battery_source));
     }
     m_numOfBattery = battery_sources.size();
+    if (m_numOfBattery == 0) {
+        m_acAlpha = 1;
+    }
 
     dataUpdated("AC Adapter", dataEngine("powermanagement")->query("AC Adapter"));
 
@@ -196,14 +199,14 @@ void Battery::toolTipAboutToShow()
     int batteryCount = 0;
     foreach (const VariantDict &batteryData, m_batteriesData) {
         if (m_numOfBattery == 1) {
-            subText.append(i18n("Battery:"));
+            subText.append(i18n("<b>Battery:</b>"));
         } else {
             //kDebug() << "More batteries ...";
             if (!subText.isEmpty()) {
                 subText.append("<br/>");
             }
 
-            subText.append(i18nc("Placeholder is the battery ID", "Battery %1:", batteryCount));
+            subText.append(i18nc("tooltip: placeholder is the battery ID", "<b>Battery %1:</b>", batteryCount));
         }
 
         subText.append(' ').append(stringForState(batteryData));
@@ -214,8 +217,8 @@ void Battery::toolTipAboutToShow()
         subText.append("<br/>");
     }
 
-    subText.append(i18n("AC Adapter:")).append(' ');
-    subText.append(m_acAdapterPlugged ? i18n("Plugged in") : i18n("Not plugged in"));
+    subText.append(i18nc("tooltip", "<b>AC Adapter:</b>")).append(' ');
+    subText.append(m_acAdapterPlugged ? i18nc("tooltip", "Plugged in") : i18nc("tooltip", "Not plugged in"));
 
     Plasma::ToolTipContent c(mainText, subText, KIcon("battery"));
     Plasma::ToolTipManager::self()->setContent(this, c);
@@ -248,7 +251,7 @@ void Battery::constraintsEvent(Plasma::Constraints constraints)
         }
     }
 
-    if (constraints & (Plasma::FormFactorConstraint | Plasma::SizeConstraint)) {
+    if (constraints & Plasma::FormFactorConstraint || constraints & Plasma::SizeConstraint) {
         int minWidth = KIconLoader::SizeSmall;
         int minHeight = KIconLoader::SizeSmall;
         bool showToolTips = false;
@@ -280,6 +283,9 @@ void Battery::constraintsEvent(Plasma::Constraints constraints)
         } else {
             setMinimumSize(minWidth, minHeight);
         }
+        if (parentLayoutItem() && parentLayoutItem()->isLayout()) {
+            static_cast<QGraphicsLayout *>(parentLayoutItem())->invalidate();
+        }
 
         if (showToolTips) {
             Plasma::ToolTipManager::self()->registerWidget(this);
@@ -305,32 +311,33 @@ void Battery::constraintsEvent(Plasma::Constraints constraints)
 
 void Battery::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
 {
-    if (source.startsWith(QLatin1String("Battery"))) {
+    if (source == "Battery") {
+        m_remainingMSecs = data["Remaining msec"].toULongLong();
+        kDebug() << "Remaining msecs on battery:" << m_remainingMSecs;
+    }
+    else if (source.startsWith(QLatin1String("Battery"))) {
         m_batteriesData[source] = data;
         //kDebug() << "new battery source" << source;
     } else if (source == "AC Adapter") {
         m_acAdapterPlugged = data["Plugged in"].toBool();
         showAcAdapter(m_acAdapterPlugged);
     } else if (source == "PowerDevil") {
-        m_availableProfiles = data["Available profiles"].toStringList();
+        m_availableProfiles = data["Available profiles"].value< StringStringMap >();
         m_currentProfile = data["Current profile"].toString();
-        //kDebug() << "PowerDevil profiles:" << m_availableProfiles << "[" << m_currentProfile << "]";
+        kDebug() << "PowerDevil profiles:" << m_availableProfiles << "[" << m_currentProfile << "]";
     } else {
         kDebug() << "Applet::Dunno what to do with " << source;
     }
 
-    if (source == "Battery0") {
-        m_remainingMSecs = data["Remaining msec"].toInt();
-        //kDebug() << "Remaining msecs on battery:" << m_remainingMSecs;
-    }
-
-    if (m_numOfBattery == 0) {
-        setStatus(Plasma::PassiveStatus);
-    } else if (m_batteriesData.count() > 0 && data.contains("Percent") && data["Percent"].toInt() < 10) {
-        setStatus(Plasma::NeedsAttentionStatus);
-    } else {
-        setStatus(Plasma::ActiveStatus);
+    Plasma::ItemStatus status = Plasma::PassiveStatus;
+    foreach (const Plasma::DataEngine::Data &data, m_batteriesData) {
+        if (status == Plasma::NeedsAttentionStatus || (data.value("Percent", 0).toInt() < 10)) {
+            status = Plasma::NeedsAttentionStatus;
+        } else {
+            status = Plasma::ActiveStatus;
+        }
     }
+    setStatus(status);
 
     updateStatus();
     update();
@@ -393,26 +400,46 @@ void Battery::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 void Battery::suspend()
 {
     hidePopup();
-    QDBusConnection dbus(QDBusConnection::sessionBus());
-    QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
-    iface.asyncCall("suspend", Solid::Control::PowerManager::ToRam);
+    QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.Solid.PowerManagement",
+                                                      "/org/kde/Solid/PowerManagement",
+                                                      "org.kde.Solid.PowerManagement",
+                                                      "suspendToRam");
+    QDBusPendingReply< QString > reply = QDBusConnection::sessionBus().asyncCall(msg);
 }
 
 void Battery::hibernate()
 {
     hidePopup();
-    QDBusConnection dbus(QDBusConnection::sessionBus());
-    QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
-    iface.asyncCall("suspend", Solid::Control::PowerManager::ToDisk);
+    QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.Solid.PowerManagement",
+                                                      "/org/kde/Solid/PowerManagement",
+                                                      "org.kde.Solid.PowerManagement",
+                                                      "suspendToDisk");
+    QDBusPendingReply< QString > reply = QDBusConnection::sessionBus().asyncCall(msg);
 }
 
 void Battery::brightnessChanged(const int brightness)
 {
     if (!m_ignoreBrightnessChange) {
-        Solid::Control::PowerManager::setBrightness(brightness);
+        QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.Solid.PowerManagement",
+                                                          "/org/kde/Solid/PowerManagement",
+                                                          "org.kde.Solid.PowerManagement",
+                                                          "setBrightness");
+        msg.setArguments(QList<QVariant>() << QVariant::fromValue(brightness));
+        QDBusPendingReply< QString > reply = QDBusConnection::sessionBus().asyncCall(msg);
     }
 }
 
+void Battery::updateSlider()
+{
+    QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.Solid.PowerManagement",
+                                                      "/org/kde/Solid/PowerManagement",
+                                                      "org.kde.Solid.PowerManagement",
+                                                      "brightness");
+    QDBusPendingReply< int > reply = QDBusConnection::sessionBus().asyncCall(msg);
+    reply.waitForFinished();
+    updateSlider(reply.value());
+}
+
 void Battery::updateSlider(const float brightness)
 {
     if (m_brightnessSlider->value() != (int)brightness) {
@@ -464,9 +491,18 @@ void Battery::initExtenderItem(Plasma::ExtenderItem *item)
         int row = 0;
 
         m_controls = new QGraphicsWidget(item);
-        m_controls->setMinimumWidth(360);
         m_controlsLayout = new QGraphicsGridLayout(m_controls);
         m_controlsLayout->setColumnStretchFactor(1, 3);
+        // Do not call QGraphicsWidget::setMinimumWidth() here: for some reason
+        // it prevents the widget from getting wide enough if needed (for
+        // example because of wide translations, see bug #258044)
+        //
+        // Setting the minimum width of a column does not cause such problems,
+        // so use that solution as a workaround.
+        //
+        // This is reproducible with Qt 4.7
+        //m_controls->setMinimumWidth(360);
+        m_controlsLayout->setColumnMinimumWidth(1, 200);
 
         Plasma::Label *spacer0 = new Plasma::Label(m_controls);
         spacer0->setMinimumHeight(8);
@@ -494,7 +530,7 @@ void Battery::initExtenderItem(Plasma::ExtenderItem *item)
         m_controlsLayout->addItem(m_remainingInfoLabel, row, 1);
         row++;
 
-        Battery *m_extenderApplet = static_cast<Battery*>(Plasma::Applet::load("battery"));
+        Battery *m_extenderApplet = new Battery(0, QVariantList());
         m_extenderApplet->setParent(m_controls);
         m_extenderApplet->setAcceptsHoverEvents(false);
         m_extenderApplet->setParentItem(m_controls);
@@ -510,39 +546,29 @@ void Battery::initExtenderItem(Plasma::ExtenderItem *item)
         m_profileLabel->setText(i18n("Power Profile:"));
         m_controlsLayout->addItem(m_profileLabel, row, 0);
 
+
         m_profileCombo = new Plasma::ComboBox(m_controls);
         // Workaround for bug 219873
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 7, 0))
         m_profileCombo->nativeWidget()->setMaxVisibleItems(4);
-#else
-        // Value passed needs to be one less than intended on Qt < 4.7.0
-        m_profileCombo->nativeWidget()->setMaxVisibleItems(3);
-#endif
         // This is necessary until QTBUG-2368 is fixed
         m_profileCombo->setZValue(110);
         connect(m_profileCombo, SIGNAL(activated(QString)),
                 this, SLOT(setProfile(QString)));
-
         m_controlsLayout->addItem(m_profileCombo, row, 1, 1, 2);
         row++;
 
         m_brightnessLabel = createBuddyLabel(m_controls);
-#if KDE_IS_VERSION(4, 5, 60)
         m_brightnessLabel->setText(i18n("Screen Brightness:"));
-#else
-        m_brightnessLabel->setText(i18n("Screen Brightness") + ':');
-#endif
-
         m_controlsLayout->addItem(m_brightnessLabel, row, 0);
 
         m_brightnessSlider = new Plasma::Slider(m_controls);
         m_brightnessSlider->setRange(0, 100);
-        updateSlider(Solid::Control::PowerManager::brightness());
+
+        updateSlider();
         m_brightnessSlider->nativeWidget()->setTickInterval(10);
         m_brightnessSlider->setOrientation(Qt::Horizontal);
         connect(m_brightnessSlider, SIGNAL(valueChanged(int)),
                 this, SLOT(brightnessChanged(int)));
-
         m_controlsLayout->addItem(m_brightnessSlider, row, 1, 1, 2);
         row++;
 
@@ -574,10 +600,16 @@ void Battery::initExtenderItem(Plasma::ExtenderItem *item)
         }
         // Configure button
         Plasma::IconWidget *configButton = createButton(m_controls);
-        configButton->setToolTip(i18nc("tooltip on the config button in the popup", "Configure Power Management..."));
+        QAction *action = new QAction(this);
+        action->setToolTip(i18nc("tooltip on the config button in the popup", "Configure Power Management..."));
+        action->setIcon(KIcon("configure"));
+        action->setText(i18n("Power save settings"));
+        connect(action, SIGNAL(triggered(bool)), this, SLOT(openConfig()));
+        connect(configButton, SIGNAL(activated()), this, SLOT(openConfig()));
+        addAction("configure_powersave", action);
+        configButton->setIcon(action->icon());
+        configButton->setToolTip(action->toolTip());
         configButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum);
-        configButton->setIcon("configure");
-        connect(configButton, SIGNAL(clicked()), this, SLOT(openConfig()));
         configButton->setEnabled(hasAuthorization("LaunchApp"));
 
         buttonLayout->addItem(configButton);
@@ -593,8 +625,8 @@ void Battery::initExtenderItem(Plasma::ExtenderItem *item)
         setupFonts();
 
         m_brightnessOSD = new BrightnessOSDWidget();
-        QDBusConnection::sessionBus().connect("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil",
-                                              "brightnessChanged", this, SLOT(showBrightnessOSD(int,bool)));
+        QDBusConnection::sessionBus().connect("org.kde.Solid.PowerManagement", "/org/kde/Solid/PowerManagement", "org.kde.Solid.PowerManagement",
+                                              "brightnessChanged", this, SLOT(showBrightnessOSD(int)));
     }
 }
 
@@ -633,19 +665,16 @@ QString Battery::stringForState(const QHash<QString, QVariant> &batteryData, boo
 {
     if (batteryData["Plugged in"].toBool()) {
         const QString state = batteryData["State"].toString();
-        if (state == "NoCharge") {
-            if (chargeChanging) {
-                *chargeChanging = true;
-            }
-	    return i18n("%1% (charged)", batteryData["Percent"].toString());
-        } else if (state == "Discharging") {
-            if (chargeChanging) {
-                *chargeChanging = true;
-            }
-            return i18n("%1% (discharging)", batteryData["Percent"].toString());
-        }
+        if (chargeChanging)
+            *chargeChanging = true;
 
-        return i18n("%1% (charging)", batteryData["Percent"].toString());
+        if (state == "NoCharge") {
+            return i18n("%1% (charged)", batteryData["Percent"].toString());
+        } else if (state == "Discharging") {
+            return i18n("%1% (discharging)", batteryData["Percent"].toString());
+        } else  {// charging
+            return i18n("%1% (charging)", batteryData["Percent"].toString());
+        }
     }
 
     return i18nc("Battery is not plugged in", "Not present");
@@ -668,8 +697,8 @@ void Battery::updateStatus()
             } else {
                 //kDebug() << "More batteries ...";
                 if (!batteriesInfo.isEmpty()) {
-                    batteriesLabel.append("<br />");
-                    batteriesInfo.append("<br />");
+                    batteriesLabel.append('\n');
+                    batteriesInfo.append('\n');
                 }
                 batteriesLabel.append(i18nc("Placeholder is the battery ID", "Battery %1:", batteryCount));
             }
@@ -678,41 +707,43 @@ void Battery::updateStatus()
             ++batteryCount;
         }
 
-        m_acLabelLabel->setText(i18n("AC Adapter:")); // ouch ...
+        // Avoid breaking string freeze (fixed correctly in trunk)
+        m_acLabelLabel->setText(i18n("AC Adapter: ").trimmed());
         if (m_acAdapterPlugged) {
-            m_acInfoLabel->setText(i18n("Plugged in"));
+            m_acInfoLabel->setText(i18n("<b>Plugged in</b>"));
         } else {
-            m_acInfoLabel->setText(i18n("Not plugged in"));
+            m_acInfoLabel->setText(i18n("<b>Not plugged in</b>"));
         }
 
-        if (showRemainingTime && m_remainingMSecs > 0) {
+        if (batteryCount && showRemainingTime && m_remainingMSecs > 0) {
             m_remainingTimeLabel->show();
             m_remainingInfoLabel->show();
             // we don't have too much accuracy so only give hours and minutes
             int remainingMinutes = m_remainingMSecs - (m_remainingMSecs % 60000);
-            m_remainingInfoLabel->setText(KGlobal::locale()->prettyFormatDuration(remainingMinutes));
+            QString remText = QString("%1%2%3").arg("<b>", KGlobal::locale()->prettyFormatDuration(remainingMinutes), "</b>");
+            m_remainingInfoLabel->setText(remText);
         } else {
             m_remainingTimeLabel->hide();
             m_remainingInfoLabel->hide();
         }
     } else {
-        batteriesLabel = i18n("<b>Battery:</b> ");
-        batteriesInfo = i18nc("Battery is not plugged in", "Not present");
+        batteriesLabel = i18n("Battery:");
+        batteriesInfo = i18nc("Battery is not plugged in", "<b>Not present</b>");
     }
 
     if (!batteriesInfo.isEmpty()) {
         m_batteryInfoLabel->setText(batteriesInfo);
         m_batteryLabelLabel->setText(batteriesLabel);
+        kDebug() << batteriesInfo;
+        kDebug() << batteriesLabel;
     }
 
     if (!m_availableProfiles.empty() && m_profileCombo) {
         m_profileCombo->clear();
-        m_profileCombo->addItem(m_currentProfile);
-        foreach (const QString &p, m_availableProfiles) {
-            if (m_currentProfile != p) {
-                m_profileCombo->addItem(p);
-            }
+        for (StringStringMap::const_iterator i = m_availableProfiles.constBegin(); i != m_availableProfiles.constEnd(); ++i) {
+            m_profileCombo->addItem(i.value());
         }
+        m_profileCombo->setCurrentIndex(m_profileCombo->nativeWidget()->findText(m_availableProfiles[m_currentProfile]));
     }
 
     if (m_profileLabel && m_profileCombo) {
@@ -726,7 +757,7 @@ void Battery::updateStatus()
     }
 
     if (m_brightnessSlider) {
-        updateSlider(Solid::Control::PowerManager::brightness());
+        updateSlider();
     }
 }
 
@@ -734,7 +765,8 @@ void Battery::openConfig()
 {
     //kDebug() << "opening powermanagement configuration dialog";
     QStringList args;
-    args << "powerdevilconfig";
+    args << "powerdevilglobalconfig";
+    args << "powerdevilprofilesconfig";
     KToolInvocation::kdeinitExec("kcmshell4", args);
 }
 
@@ -742,9 +774,8 @@ void Battery::setProfile(const QString &profile)
 {
     if (m_currentProfile != profile) {
         kDebug() << "Changing power profile to " << profile;
-        QDBusConnection dbus( QDBusConnection::sessionBus() );
-        QDBusInterface iface( "org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus );
-        iface.call( "setProfile", profile );
+        QDBusInterface iface( "org.kde.Solid.PowerManagement", "/org/kde/Solid/PowerManagement", "org.kde.Solid.PowerManagement" );
+        iface.call( "loadProfile", m_availableProfiles.key(profile) );
     }
 }
 
@@ -759,8 +790,16 @@ void Battery::showLabel(bool show)
 
 void Battery::showAcAdapter(bool show)
 {
-    m_acAnimation->setDirection(show ? QAbstractAnimation::Forward
-                                     : QAbstractAnimation::Backward);
+    if (m_numOfBattery == 0) {
+        return;
+    }
+
+    QAbstractAnimation::Direction d = show ? QAbstractAnimation::Forward : QAbstractAnimation::Backward;
+    if (d == m_acAnimation->direction()) {
+        return;
+    }
+
+    m_acAnimation->setDirection(d);
     if (m_acAnimation->state() != QAbstractAnimation::Running) {
         m_acAnimation->start();
     }
@@ -849,7 +888,7 @@ void Battery::paintBattery(QPainter *p, const QRect &contentsRect, const int bat
 {
     int minSize = qMin(contentsRect.height(), contentsRect.width());
     QRect contentsSquare = QRect(contentsRect.x() + (contentsRect.width() - minSize) / 2, contentsRect.y() + (contentsRect.height() - minSize) / 2, minSize, minSize);
-  
+
     if (m_theme->hasElement("Battery")) {
         m_theme->paint(p, contentsSquare, "Battery");
     }
@@ -881,7 +920,9 @@ void Battery::paintBattery(QPainter *p, const QRect &contentsRect, const int bat
         }
     }
 
-    m_theme->paint(p, scaleRectF(m_acAlpha, contentsSquare), "AcAdapter");
+    if (!qFuzzyCompare(1, m_acAlpha + 1)) {
+        m_theme->paint(p, scaleRectF(m_acAlpha, contentsSquare), "AcAdapter");
+    }
 
     if (plugState && m_theme->hasElement("Overlay")) {
         m_theme->paint(p, contentsSquare, "Overlay");
@@ -896,10 +937,6 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
     p->setRenderHint(QPainter::Antialiasing);
 
     if (m_numOfBattery == 0) {
-        QRectF ac_contentsRect(contentsRect.topLeft(), QSizeF(qMax(qreal(0.0), contentsRect.width() * m_acAlpha), qMax(qreal(0.0), contentsRect.height() * m_acAlpha)));
-        if (m_acAdapterPlugged) {
-            m_theme->paint(p, ac_contentsRect, "AcAdapter");
-        }
         paintBattery(p, contentsRect, 0, false);
         return;
     }
@@ -929,7 +966,7 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
             // paint battery with appropriate charge level
             paintBattery(p, corect, batteryData.value()["Percent"].toInt(), batteryData.value()["Plugged in"].toBool());
 
-            if (m_showBatteryLabel || !isConstrained()) {
+            if (!isConstrained()) {
                 // Show the charge percentage with a box on top of the battery
                 QString batteryLabel;
                 if (batteryData.value()["Plugged in"].toBool()) {
@@ -941,7 +978,6 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
                     }
                     QString state = batteryData.value()["State"].toString();
 
-                    m_remainingMSecs = batteryData.value()["Remaining msec"].toInt();
                     if (m_remainingMSecs > 0 && (m_showRemainingTime && (state=="Charging" || state=="Discharging"))) {
                         QTime t = QTime(m_hours, m_minutes);
                         KLocale tmpLocale(*KGlobal::locale());
@@ -1008,6 +1044,7 @@ void Battery::connectSources()
 
     dataEngine("powermanagement")->connectSource("AC Adapter", this);
     dataEngine("powermanagement")->connectSource("PowerDevil", this);
+    dataEngine("powermanagement")->connectSource("Battery", this);
 
     connect(dataEngine("powermanagement"), SIGNAL(sourceAdded(QString)),
             this,                          SLOT(sourceAdded(QString)));
@@ -1044,7 +1081,7 @@ void Battery::setLabelAlpha(qreal alpha)
     update();
 }
 
-qreal Battery::labelAlpha()
+qreal Battery::labelAlpha() const
 {
     return m_labelAlpha;
 }
@@ -1055,29 +1092,35 @@ void Battery::setAcAlpha(qreal alpha)
     update();
 }
 
-qreal Battery::acAlpha()
+QList<QAction*> Battery::contextualActions()
+{
+    QList<QAction *> rv;
+    QAction *act = action("configure_powersave");
+    rv << act;
+    return rv;
+}
+
+qreal Battery::acAlpha() const
 {
     return m_acAlpha;
 }
 
-void Battery::showBrightnessOSD(int brightness, bool byKeyPress)
+void Battery::showBrightnessOSD(int brightness)
 {
-    if (byKeyPress) {
-        // code adapted from KMix
-        m_brightnessOSD->setCurrentBrightness(brightness);
-        m_brightnessOSD->show();
-        m_brightnessOSD->activateOSD(); //Enable the hide timer
+    // code adapted from KMix
+    m_brightnessOSD->setCurrentBrightness(brightness);
+    m_brightnessOSD->show();
+    m_brightnessOSD->activateOSD(); //Enable the hide timer
 
-        //Center the OSD
-        QRect rect = KApplication::kApplication()->desktop()->screenGeometry(QCursor::pos());
-        QSize size = m_brightnessOSD->sizeHint();
-        int posX = rect.x() + (rect.width() - size.width()) / 2;
-        int posY = rect.y() + 4 * rect.height() / 5;
-        m_brightnessOSD->setGeometry(posX, posY, size.width(), size.height());
+    //Center the OSD
+    QRect rect = KApplication::kApplication()->desktop()->screenGeometry(QCursor::pos());
+    QSize size = m_brightnessOSD->sizeHint();
+    int posX = rect.x() + (rect.width() - size.width()) / 2;
+    int posY = rect.y() + 4 * rect.height() / 5;
+    m_brightnessOSD->setGeometry(posX, posY, size.width(), size.height());
 
-        if (m_extenderVisible && m_brightnessSlider) {
-            updateSlider(brightness);
-        }
+    if (m_extenderVisible && m_brightnessSlider) {
+        updateSlider(brightness);
     }
 }
 
