@@ -26,6 +26,7 @@
 
 //KDE
 #include <KConfigDialog>
+#include <KDebug>
 #include <KGlobal>
 #include <KStandardDirs>
 #include <KCModuleProxy>
@@ -46,16 +47,114 @@
 #include "deviceitem.h"
 #include <Plasma/Containment>
 using namespace Plasma;
-using namespace Notifier;
 
 static const char DEFAULT_ICON_NAME[] = "device-notifier";
-static const int NOTIFICATION_TIMEOUT = 10000;
 
-K_EXPORT_PLASMA_APPLET(devicenotifier, DeviceNotifier)
+K_EXPORT_PLASMA_APPLET(devicenotifier, Notifier::DeviceNotifier)
+
+namespace Notifier
+{
+
+HotplugDataConsumer::HotplugDataConsumer(NotifierDialog *parent)
+    : QObject(parent),
+      m_dialog(parent)
+{
+    Q_ASSERT(parent);
+}
+
+void HotplugDataConsumer::dataUpdated(const QString &udi, Plasma::DataEngine::Data data)
+{
+    if (data.isEmpty()) {
+        return;
+    }
+
+    //data from hotplug engine
+    //kDebug() << "adding" << data["udi"];
+    int numActions = 0;
+    QString lastActionLabel;
+    QStringList currentActions = m_dialog->deviceActions(udi);
+    QStringList newActions = data["predicateFiles"].toStringList();
+
+    foreach (const QString &desktop, newActions) {
+        QString filePath = KStandardDirs::locate("data", "solid/actions/" + desktop);
+        QList<KServiceAction> services = KDesktopFileActions::userDefinedServices(filePath, true);
+        numActions += services.size();
+
+        if (!currentActions.contains(desktop)) {
+            m_dialog->insertAction(udi, desktop);
+        }
+
+        if (services.size() > 0) {
+            lastActionLabel = QString(services[0].text());
+        }
+    }
+
+    foreach (const QString &action, currentActions) {
+        if (!newActions.contains(action)) {
+            m_dialog->removeAction(udi, action);
+        }
+    }
+
+    m_dialog->setDeviceData(udi, data["text"], Qt::DisplayRole);
+    m_dialog->setDeviceData(udi, data["isEncryptedContainer"], NotifierDialog::IsEncryptedContainer);
+
+    if (numActions > 1) {
+        QString s = i18np("1 action for this device",
+                          "%1 actions for this device",
+                          numActions);
+        m_dialog->setDeviceData(udi, s, NotifierDialog::DescriptionRole);
+    } else {
+        m_dialog->setDeviceData(udi, lastActionLabel, NotifierDialog::DescriptionRole);
+    }
+}
+
+DeviceDataConsumer::DeviceDataConsumer(NotifierDialog *parent)
+    : QObject(parent),
+      m_dialog(parent)
+{
+    Q_ASSERT(parent);
+}
+
+void DeviceDataConsumer::dataUpdated(const QString &udi, Plasma::DataEngine::Data data)
+{
+    m_dialog->setDeviceData(udi, data["Icon"], NotifierDialog::IconNameRole);
+    m_dialog->setDeviceData(udi, KIcon(data["Icon"].toString(), NULL, data["Emblems"].toStringList()), Qt::DecorationRole);
+
+    const bool isOpticalMedia = data["Device Types"].toStringList().contains("OpticalDisc");
+
+    m_dialog->setDeviceData(udi, isOpticalMedia, NotifierDialog::IsOpticalMedia);
+
+    if (data["Device Types"].toStringList().contains("Storage Access")) {
+        //kDebug() << "DeviceNotifier::solidDeviceEngine updated" << udi;
+        if (data["Accessible"].toBool()) {
+            m_dialog->setMounted(true, udi);
+        } else {
+            m_dialog->setMounted(false, udi);
+        }
+
+        if (data["Ignored"].toBool()) {
+            m_dialog->setDeviceData(udi, data["File Path"], Qt::DisplayRole);
+
+            const QString desktop("test-predicate-openinwindow.desktop");
+            QString filePath = KStandardDirs::locate("data", "solid/actions/" + desktop);
+            QList<KServiceAction> services = KDesktopFileActions::userDefinedServices(filePath, true);
+            if (services.size() > 0) { //in case there is no action at all
+                m_dialog->insertAction(udi, desktop);
+                m_dialog->setDeviceData(udi, services[0].text(), NotifierDialog::DescriptionRole);
+            }
+
+            m_dialog->setDeviceLeftAction(udi, DeviceItem::Nothing);
+        }
+    } else if (data["Device Types"].toStringList().contains("Storage Volume")) {
+        if (isOpticalMedia) {
+            m_dialog->setMounted(true, udi);
+        }
+    }
+}
 
 DeviceNotifier::DeviceNotifier(QObject *parent, const QVariantList &args)
     : Plasma::PopupApplet(parent, args),
-      m_solidEngine(0),
+      m_hotplugEngine(0),
       m_solidDeviceEngine(0),
       m_deviceNotificationsEngine(0),
       m_dialog(0),
@@ -63,6 +162,8 @@ DeviceNotifier::DeviceNotifier(QObject *parent, const QVariantList &args)
       m_itemsValidity(0),
       m_globalVisibility(false),
       m_checkHiddenDevices(true),
+      m_triggeringPopupInternally(false),
+      m_poppedUpInternally(false),
       m_autoMountingWidget(0),
       m_deviceActionsWidget(0)
 {
@@ -72,6 +173,8 @@ DeviceNotifier::DeviceNotifier(QObject *parent, const QVariantList &args)
 
     // let's initialize the widget
     resize(graphicsWidget()->minimumSize());
+    m_hotplugDataConsumer = new HotplugDataConsumer(m_dialog);
+    m_deviceDataConsumer = new DeviceDataConsumer(m_dialog);
 }
 
 DeviceNotifier::~DeviceNotifier()
@@ -83,20 +186,21 @@ void DeviceNotifier::init()
 {
     configChanged();
 
-    m_solidEngine = dataEngine("hotplug");
+    m_hotplugEngine = dataEngine("hotplug");
     m_solidDeviceEngine = dataEngine("soliddevice");
     m_deviceNotificationsEngine = dataEngine("devicenotifications");
 
-    connect(m_dialog, SIGNAL(deviceSelected()), this, SLOT(showPopup()));
+    //don't close the dialog when the user is using it
+    connect(m_dialog, SIGNAL(activated()), this, SLOT(showPopup()));
 
     Plasma::ToolTipManager::self()->registerWidget(this);
 
     setPopupIcon(DEFAULT_ICON_NAME);
 
     //connect to engine when a device is plugged in
-    connect(m_solidEngine, SIGNAL(sourceAdded(const QString&)),
+    connect(m_hotplugEngine, SIGNAL(sourceAdded(const QString&)),
             this, SLOT(onSourceAdded(const QString&)));
-    connect(m_solidEngine, SIGNAL(sourceRemoved(const QString&)),
+    connect(m_hotplugEngine, SIGNAL(sourceRemoved(const QString&)),
             this, SLOT(onSourceRemoved(const QString&)));
     connect(m_deviceNotificationsEngine, SIGNAL(sourceAdded(const QString&)),
             this, SLOT(newNotification(const QString&)));
@@ -113,8 +217,9 @@ void DeviceNotifier::init()
 void DeviceNotifier::newNotification(const QString &source)
 {
     DataEngine::Data data = m_deviceNotificationsEngine->query(source);
-    //TODO Check if we are actually displaying the device in question
-    showErrorMessage(data["error"].toString(), data["errorDetails"].toString(), data["udi"].toString());
+    if (m_lastPlugged.contains(data["udi"].toString()) && !m_hiddenDevices.contains(data["udi"].toString())) {
+        showNotification(data["error"].toString(), data["errorDetails"].toString(), data["udi"].toString());
+    }
 }
 
 void DeviceNotifier::configChanged()
@@ -153,7 +258,7 @@ void DeviceNotifier::fillPreviousDevices()
         }
     }
 
-    foreach (const QString &udi, m_solidEngine->sources()) {
+    foreach (const QString &udi, m_hotplugEngine->sources()) {
         onSourceAdded(udi);
     }
 
@@ -185,93 +290,25 @@ void DeviceNotifier::popupEvent(bool show)
     } else {
         m_dialog->collapseDevices();
     }
-    changeNotifierIcon();
-}
 
-void DeviceNotifier::dataUpdated(const QString &udi, Plasma::DataEngine::Data data)
-{
-    if (data.isEmpty()) {
-        return;
+    if (m_triggeringPopupInternally && show) {
+        m_poppedUpInternally = true;
+    } else if (!show) {
+        m_poppedUpInternally = false;
     }
 
-    //data from hotplug engine
-    //kDebug() << data["udi"] << data["predicateFiles"].toStringList() << data["Device Types"].toStringList();
+    if (!m_triggeringPopupInternally) {
+        changeNotifierIcon();
+    }
+    m_triggeringPopupInternally = false;
+}
 
-    //FIXME: here we rely on the fact that the hotplug engine gives a "text" field (and the soliddevice one does not)
-    // to distinguish between data from the two engines. This is really not nice
-    if (data["text"].isValid()) {
-        //kDebug() << "adding" << data["udi"];
-        int nb_actions = 0;
-        QString lastActionLabel;
-        QStringList currentActions = m_dialog->deviceActions(udi);
-        QStringList newActions = data["predicateFiles"].toStringList();
-        foreach (const QString &desktop, newActions) {
-            QString filePath = KStandardDirs::locate("data", "solid/actions/" + desktop);
-            QList<KServiceAction> services = KDesktopFileActions::userDefinedServices(filePath, true);
-            nb_actions += services.size();
-
-            if (!currentActions.contains(desktop)) {
-                m_dialog->insertAction(udi, desktop);
-            }
-            if (services.size() > 0) {
-                lastActionLabel = QString(services[0].text());
-            }
-        }
-        foreach (const QString &action, currentActions) {
-            if (!newActions.contains(action)) {
-                m_dialog->removeAction(udi, action);
-            }
-        }
-
-        m_dialog->setDeviceData(udi, data["text"], Qt::DisplayRole);
-        m_dialog->setDeviceData(udi, data["isEncryptedContainer"], NotifierDialog::IsEncryptedContainer);
-
-        if (nb_actions > 1) {
-            QString s = i18np("1 action for this device",
-                    "%1 actions for this device",
-                    nb_actions);
-            m_dialog->setDeviceData(udi, s, NotifierDialog::DescriptionRole);
-        } else {
-            m_dialog->setDeviceData(udi, lastActionLabel, NotifierDialog::DescriptionRole);
-        }
-
-    //data from soliddevice engine
-    } else {
-
-        //icon name
-        m_dialog->setDeviceData(udi, data["Icon"], NotifierDialog::IconNameRole);
-        m_dialog->setDeviceData(udi, KIcon(data["Icon"].toString(), NULL, data["Emblems"].toStringList()), Qt::DecorationRole);
-
-        bool isOpticalMedia = data["Device Types"].toStringList().contains("OpticalDisc");
-
-        m_dialog->setDeviceData(udi, isOpticalMedia, NotifierDialog::IsOpticalMedia);
-
-        if (data["Device Types"].toStringList().contains("Storage Access")) {
-            //kDebug() << "DeviceNotifier::solidDeviceEngine updated" << udi;
-            if (data["Accessible"].toBool()) {
-                m_dialog->setMounted(true, udi);
-            } else {
-                m_dialog->setMounted(false, udi);
-            }
-
-            if (data["Ignored"].toBool()) {
-                m_dialog->setDeviceData(udi, data["File Path"], Qt::DisplayRole);
-
-                const QString desktop("test-predicate-openinwindow.desktop");
-                QString filePath = KStandardDirs::locate("data", "solid/actions/" + desktop);
-                QList<KServiceAction> services = KDesktopFileActions::userDefinedServices(filePath, true);
-                if (services.size() > 0) { //in case there is no action at all
-                    m_dialog->insertAction(udi, desktop);
-                    m_dialog->setDeviceData(udi, services[0].text(), NotifierDialog::DescriptionRole);
-                }
-
-                m_dialog->setDeviceLeftAction(udi, DeviceItem::Nothing);
-            }
-        } else if (data["Device Types"].toStringList().contains("Storage Volume")) {
-            if (isOpticalMedia) {
-                m_dialog->setMounted(true, udi);
-            }
-        }
+void DeviceNotifier::keepPopupOpen()
+{
+    if (!m_poppedUpInternally) {
+        m_poppedUpInternally = false;
+        kDebug() << "ping";
+        showPopup();
     }
 }
 
@@ -281,8 +318,10 @@ void DeviceNotifier::notifyDevice(const QString &udi)
 
     if (!m_fillingPreviousDevices) {
         emit activate();
-        showPopup(NOTIFICATION_TIMEOUT);
-        changeNotifierIcon("preferences-desktop-notification", NOTIFICATION_TIMEOUT);
+        changeNotifierIcon("preferences-desktop-notification", LONG_NOTIFICATION_TIMEOUT);
+        m_triggeringPopupInternally = true;
+        kDebug() << "pong";
+        showPopup(LONG_NOTIFICATION_TIMEOUT);
         update();
         setStatus(Plasma::NeedsAttentionStatus);
     } else {
@@ -294,11 +333,12 @@ void DeviceNotifier::toolTipAboutToShow()
 {
     Plasma::ToolTipContent toolTip;
     if (m_lastPlugged.isEmpty()) {
-        toolTip.setSubText(i18n("No devices available."));
+        toolTip.setMainText(i18n("No devices available."));
         toolTip.setImage(KIcon("device-notifier"));
     } else {
         Solid::Device device(m_lastPlugged.last());
-        toolTip.setSubText(i18n("Most recent device: %1", device.description()));
+        toolTip.setMainText(i18n("Most recent device"));
+        toolTip.setSubText(device.description());
         toolTip.setImage(KIcon(device.icon()));
     }
 
@@ -317,7 +357,7 @@ void DeviceNotifier::removeLastDeviceNotification(const QString &udi)
 
 void DeviceNotifier::onSourceAdded(const QString &udi)
 {
-    DataEngine::Data data = m_solidEngine->query(udi);
+    DataEngine::Data data = m_hotplugEngine->query(udi);
     Solid::Device device = Solid::Device(udi);
     deviceAdded(device, data["added"].toBool());
 }
@@ -338,6 +378,12 @@ void DeviceNotifier::deviceAdded(const Solid::Device &device, bool hotplugged)
         if (drive && (!drive->isHotpluggable() && !drive->isRemovable())) {
             return;
         }
+        //HACK for 4.6: Treat remote shares as non-removable devices
+        //              for 4.7 we do need more configuration options
+        //              and better detection of remote shares
+        if (device.parentUdi().startsWith("/org/kde/fstab")) {
+            return;
+        }
     }
 
     kDebug() << "DeviceNotifier:: source added" << udi;
@@ -361,7 +407,12 @@ void DeviceNotifier::deviceAdded(const Solid::Device &device, bool hotplugged)
         //
         // TODO: Handling of fixed devices needs to be rethought for 4.6
 
-        if (!m_lastPlugged.contains(udi)) {
+        if (m_lastPlugged.contains(udi)) {
+            // Reconnect to both engines since now we can possibly
+            // correctly connect to the updates.
+            m_hotplugEngine->disconnectSource(udi, m_hotplugDataConsumer);
+            m_solidDeviceEngine->disconnectSource(udi, m_deviceNotificationsEngine);
+        } else {
             m_dialog->insertDevice(udi);
 
             if (hotplugged) {
@@ -369,17 +420,11 @@ void DeviceNotifier::deviceAdded(const Solid::Device &device, bool hotplugged)
             }
 
             m_dialog->setDeviceData(udi, visibility, NotifierDialog::VisibilityRole);
-
             m_lastPlugged << udi;
-        } else {
-            // Reconnect to both engines since now we can possibly
-            // correctly connect to the updates.
-            m_solidEngine->disconnectSource(udi, this);
-            m_solidDeviceEngine->disconnectSource(udi, this);
         }
 
-        m_solidEngine->connectSource(udi, this);
-        m_solidDeviceEngine->connectSource(udi, this);
+        m_hotplugEngine->connectSource(udi, m_hotplugDataConsumer);
+        m_solidDeviceEngine->connectSource(udi, m_deviceDataConsumer);
     }
 
     if (!visibility && !m_hiddenDevices.contains(udi)) {
@@ -393,8 +438,8 @@ void DeviceNotifier::deviceAdded(const Solid::Device &device, bool hotplugged)
 
 void DeviceNotifier::onSourceRemoved(const QString &udi)
 {
-    m_solidEngine->disconnectSource(udi, this);
-    m_solidDeviceEngine->disconnectSource(udi, this);
+    m_hotplugEngine->disconnectSource(udi, m_hotplugDataConsumer);
+    m_solidDeviceEngine->disconnectSource(udi, m_deviceDataConsumer);
 
     m_dialog->removeDevice(udi);
     removeLastDeviceNotification(udi);
@@ -496,11 +541,16 @@ void DeviceNotifier::setGlobalVisibility(bool visibility)
     resetDevices();
 }
 
-void DeviceNotifier::showErrorMessage(const QString &message, const QString &details, const QString &udi)
+void DeviceNotifier::showNotification(const QString &message, const QString &details, const QString &udi)
 {
+    if (!isPopupShowing()) {
+        m_triggeringPopupInternally = true;
+        kDebug() << "pang";
+        showPopup(LONG_NOTIFICATION_TIMEOUT);
+    }
+
     m_dialog->showStatusBarMessage(message, details, udi);
-    showPopup(NOTIFICATION_TIMEOUT);
-    changeNotifierIcon("dialog-error", NOTIFICATION_TIMEOUT);
+
     update();
 }
 
@@ -509,6 +559,11 @@ bool DeviceNotifier::areThereHiddenDevices()
     return (m_hiddenDevices.count() > 0);
 }
 
+bool DeviceNotifier::poppedUpInternally()
+{
+    return m_poppedUpInternally;
+}
+
 void DeviceNotifier::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 {
     m_dialog->setMenuActionsAt(event->scenePos());
@@ -521,4 +576,6 @@ QList<QAction *> DeviceNotifier::contextualActions()
     return m_dialog->contextualActions();
 }
 
+}
+
 #include "devicenotifier.moc"
