@@ -52,7 +52,8 @@
 #include "devicenotifier.h"
 #include "deviceitem.h"
 
-using namespace Notifier;
+namespace Notifier
+{
 
 NotifierDialog::NotifierDialog(DeviceNotifier * notifier, QObject *parent)
     : QObject(parent),
@@ -139,7 +140,14 @@ void NotifierDialog::insertDevice(const QString &udi)
     }
 
     ++m_deviceCount;
-    DeviceItem *devItem = new DeviceItem(udi);
+    Solid::Device device(udi);
+    Solid::Device parentDevice(device.parentUdi());
+    bool unpluggable = true;
+    if (parentDevice.is<Solid::StorageDrive>()) {
+        Solid::StorageDrive *drive = parentDevice.as<Solid::StorageDrive>();
+        unpluggable = (drive->isHotpluggable() || drive->isRemovable());
+    }
+    DeviceItem *devItem = new DeviceItem(udi, unpluggable);
     connect(devItem, SIGNAL(leftActionActivated(DeviceItem *)), this, SLOT(leftActionActivated(DeviceItem *)));
     connect(devItem, SIGNAL(actionActivated(DeviceItem *, const QString &, const QString &)),
             this, SLOT(actionActivated(DeviceItem *, const QString &, const QString &)));
@@ -152,7 +160,6 @@ void NotifierDialog::insertDevice(const QString &udi)
     devItem->setData(VisibilityRole, true);
 
     //search or create the category for inserted device
-    Solid::Device device(udi);
     QString categoryOfInsertedDevice = getCategoryNameOfDevice(device);
     int index = searchOrCreateDeviceCategory(categoryOfInsertedDevice);
 
@@ -205,10 +212,6 @@ void NotifierDialog::clearItemBackgroundTarget()
 
 void NotifierDialog::itemHoverEnter(DeviceItem *item)
 {
-    // make sure the popup is not only shown, but doesn't automatically retract on us when we're
-    // mousing around in it
-    m_notifier->showPopup(0);
-
     item->setHovered(true);
     if (item->isCollapsed()) {
         m_clearItemBackgroundTargetTimer.stop();
@@ -248,6 +251,16 @@ bool NotifierDialog::eventFilter(QObject* obj, QEvent *event)
             case QEvent::GraphicsSceneHoverEnter:
                 itemHoverEnter(item);
                 break;
+            case QEvent::GraphicsSceneHoverMove:
+                if (m_notifier->poppedUpInternally()) {
+                    m_notifier->showPopup(DeviceNotifier::LONG_NOTIFICATION_TIMEOUT);
+                    kDebug() << "mouse move - keep it up";
+                }
+                break;
+            case QEvent::GraphicsSceneMousePress:
+                m_notifier->keepPopupOpen();
+                kDebug() << "keeping open **************************************************";
+                break;
             default:
                 break;
         }
@@ -322,6 +335,7 @@ void NotifierDialog::selectNextItem()
     if (item) {
         itemHoverLeave(item);
     }
+
     if (nextDevice) {
         itemHoverEnter(nextDevice);
         m_devicesScrollWidget->ensureItemVisible(nextDevice);
@@ -367,6 +381,7 @@ void NotifierDialog::selectPreviousItem()
     if (item) {
         itemHoverLeave(item);
     }
+
     if (previousDevice) {
         itemHoverEnter(previousDevice);
         m_devicesScrollWidget->ensureItemVisible(previousDevice);
@@ -374,7 +389,6 @@ void NotifierDialog::selectPreviousItem()
             previousDevice->selectPreviousAction(0, true);
         }
     }
-
 }
 
 
@@ -415,13 +429,12 @@ void NotifierDialog::setMounted(bool mounted, const QString &udi)
         }
     } else {
         bool safelyRemovable = true;
-        QString parentUdi = Solid::Device(udi).parent().udi();
-        foreach (DeviceItem* sibling, itemsForParentUdi(parentUdi)) {
-            if ((sibling->udi() != udi) && (sibling->isMounted())) {
-                safelyRemovable = false;
+        Solid::Device parent = Solid::Device(udi).parent();
+        if (parent.is<Solid::StorageDrive>()) {
+                safelyRemovable = !parent.as<Solid::StorageDrive>()->isInUse();
             }
-        }
-        foreach (DeviceItem* sibling, itemsForParentUdi(parentUdi)) {
+
+        foreach (DeviceItem* sibling, itemsForParentUdi(parent.udi())) {
                 sibling->setSafelyRemovable(safelyRemovable);
         }
     }
@@ -623,7 +636,14 @@ void NotifierDialog::showStatusBarMessage(const QString & message, const QString
     m_statusText->setText(message);
     m_mainLayout->addItem(m_statusWidget);
     m_statusWidget->show();
+
     m_statusDetailsText->setText(details);
+    if (details.isEmpty()) {
+        m_statusExpandButton->hide();
+    } else {
+        m_statusExpandButton->show();
+    }
+
     showStatusBarDetails(false);
     m_errorUdi = udi;
 }
@@ -649,6 +669,9 @@ void NotifierDialog::showStatusBarDetails(bool show)
         m_statusExpandButton->setIcon(QIcon(svg->pixmap("restore")));
     }
     delete svg;
+
+    kDebug() << "here";
+//    emit activated();
 }
 
 void NotifierDialog::storageTeardownDone(Solid::ErrorType error, QVariant errorData, const QString & udi)
@@ -659,11 +682,13 @@ void NotifierDialog::storageTeardownDone(Solid::ErrorType error, QVariant errorD
     }
 
     if (!error || !errorData.isValid()) {
-        m_notifier->changeNotifierIcon("dialog-ok", 5000);
-        m_notifier->update();
+        m_notifier->changeNotifierIcon("dialog-ok", DeviceNotifier::LONG_NOTIFICATION_TIMEOUT);
         expireStatusBar(udi);
+    } else {
+        m_notifier->changeNotifierIcon("dialog-error", DeviceNotifier::LONG_NOTIFICATION_TIMEOUT);
     }
 
+    m_notifier->update();
     devItem->setState(DeviceItem::Idle);
 }
 
@@ -672,11 +697,14 @@ void NotifierDialog::storageEjectDone(Solid::ErrorType error, QVariant errorData
     Q_UNUSED(udi);
 
     if (!error || !errorData.isValid()) {
-        m_notifier->changeNotifierIcon("dialog-ok", 2000);
-        m_notifier->update();
+        m_notifier->changeNotifierIcon("dialog-ok", DeviceNotifier::SHORT_NOTIFICATION_TIMEOUT);
         expireStatusBar(udi);
+    } else {
+        m_notifier->changeNotifierIcon("dialog-error", DeviceNotifier::LONG_NOTIFICATION_TIMEOUT);
     }
 
+    m_notifier->update();
+
     QList<DeviceItem*> deviceList = itemsForParentUdi(udi);
     if (deviceList.isEmpty()) {
         kDebug() << "This should just not happen";
@@ -695,11 +723,15 @@ void NotifierDialog::storageSetupDone(Solid::ErrorType error, QVariant errorData
         return;
     }
 
-     if (!error || !errorData.isValid()) {
-        m_notifier->changeNotifierIcon("dialog-ok", 2000);
-        m_notifier->update();
+    if (!error || !errorData.isValid()) {
+        m_notifier->changeNotifierIcon("dialog-ok", DeviceNotifier::SHORT_NOTIFICATION_TIMEOUT);
         expireStatusBar(udi);
+    } else {
+        m_notifier->changeNotifierIcon("dialog-error", DeviceNotifier::LONG_NOTIFICATION_TIMEOUT);
     }
+
+    m_notifier->update();
+
     devItem->setState(DeviceItem::Idle);
 }
 
@@ -785,7 +817,7 @@ void NotifierDialog::leftActionActivated(DeviceItem *item)
             if (drive) {
                 drive->eject();
             }
-        } else if (device.is<Solid::StorageVolume>()) {
+        } else if (device.is<Solid::StorageAccess>()) {
             Solid::StorageAccess *access = device.as<Solid::StorageAccess>();
             if (access && access->isAccessible()) {
                 access->teardown();
@@ -823,7 +855,8 @@ void NotifierDialog::deviceActivated(DeviceItem *item)
 
     m_itemBackground->setTargetItem(0);
 
-    emit deviceSelected();
+    kDebug() << "there";
+    emit activated();
 }
 
 void NotifierDialog::deviceCollapsed(DeviceItem *item)
@@ -919,7 +952,7 @@ void NotifierDialog::updateFreeSpace(DeviceItem *item)
     }
 }
 
-void NotifierDialog::setMenuActionsAt(QPointF scenePos)
+void NotifierDialog::setMenuActionsAt(const QPointF& scenePos)
 {
     m_showAll->setVisible(m_notifier->areThereHiddenDevices());
 
@@ -1003,4 +1036,6 @@ void NotifierDialog::updateMainLabelText()
     }
 }
 
+} // namespace Notifier
+
 #include "notifierdialog.moc"
