@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2007-2008 by Riccardo Iaconelli <riccardo@kde.org>      *
- *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
+ *   Copyright (C) 2007-2010 by Sebastian Kügler <sebas@kde.org>           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -48,6 +48,9 @@ Clock::Clock(QObject *parent, const QVariantList &args)
       m_plainClockFont(KGlobalSettings::generalFont()),
       m_useCustomColor(false),
       m_plainClockColor(),
+      m_useCustomShadowColor(false),
+      m_plainClockShadowColor(),
+      m_drawShadow(true),
       m_showDate(false),
       m_showYear(false),
       m_showDay(false),
@@ -70,7 +73,7 @@ Clock::~Clock()
 void Clock::init()
 {
     ClockApplet::init();
-    
+
     configChanged();
 
     dataEngine("time")->connectSource(currentTimezone(), this, updateInterval(), intervalAlignment());
@@ -174,12 +177,12 @@ void Clock::clockConfigChanged()
     }
 
     m_plainClockFont = cg.readEntry("plainClockFont", m_plainClockFont);
-    m_useCustomColor = cg.readEntry("useCustomColor", false);
-    if (m_useCustomColor) {
-        m_plainClockColor = cg.readEntry("plainClockColor", m_plainClockColor);
-    } else {
-        updateColors();
-    }
+    m_useCustomColor = cg.readEntry("useCustomColor", m_useCustomColor);
+    m_plainClockColor = cg.readEntry("plainClockColor", m_plainClockColor);
+    m_useCustomShadowColor = cg.readEntry("useCustomShadowColor", m_useCustomShadowColor);
+    m_plainClockShadowColor = cg.readEntry("plainClockShadowColor", m_plainClockShadowColor);
+    m_drawShadow = cg.readEntry("plainClockDrawShadow", m_drawShadow);
+    updateColors();
 
     const QFontMetricsF metrics(KGlobalSettings::smallestReadableFont());
     const QString timeString = KGlobal::locale()->formatTime(QTime(23, 59), m_showSeconds);
@@ -198,17 +201,15 @@ void Clock::dataUpdated(const QString &source, const Plasma::DataEngine::Data &d
     m_date = data["Date"].toDate();
 
     // avoid unnecessary repaints
-    if ((m_showSeconds && m_time.second() != m_lastTimeSeen.second()) ||
-        m_time.minute() != m_lastTimeSeen.minute()) {
-        m_lastTimeSeen = m_time;
+    if ((m_showSeconds && m_time.second() != lastTimeSeen().second()) ||
+        m_time.minute() != lastTimeSeen().minute()) {
 
         if (Plasma::ToolTipManager::self()->isVisible(this)) {
             updateTipContent();
         }
-        updateClockApplet();
 
+        updateClockApplet(data);
         update();
-        speakTime(m_time);
     }
 }
 
@@ -229,6 +230,20 @@ void Clock::createClockConfigurationInterface(KConfigDialog *parent)
     ui.plainClockFont->setCurrentFont(m_plainClockFont);
     ui.useCustomColor->setChecked(m_useCustomColor);
     ui.plainClockColor->setColor(m_plainClockColor);
+    ui.drawShadow->setChecked(m_drawShadow);
+    ui.useCustomShadowColor->setChecked(m_useCustomShadowColor);
+    ui.plainClockShadowColor->setColor(m_plainClockShadowColor);
+
+    connect(ui.drawShadow, SIGNAL(toggled(bool)),
+            this, SLOT(configDrawShadowToggled(bool)));
+    configDrawShadowToggled(m_drawShadow);
+}
+
+void Clock::configDrawShadowToggled(bool value)
+{
+    ui.useCustomShadowColor->setEnabled(value);
+    ui.customShadowColorLabel->setEnabled(value);
+    ui.plainClockShadowColor->setEnabled(value && ui.useCustomShadowColor->isChecked());
 }
 
 void Clock::clockConfigAccepted()
@@ -246,7 +261,7 @@ void Clock::clockConfigAccepted()
         cg.writeEntry("showSeconds", m_showSeconds);
 
         if (m_showSeconds) {
-            //We don't need to cache the applet if it update every seconds
+            //We don't need to cache the applet if it update every second
             setCacheMode(QGraphicsItem::NoCache);
         } else {
             setCacheMode(QGraphicsItem::DeviceCoordinateCache);
@@ -265,18 +280,28 @@ void Clock::clockConfigAccepted()
     cg.writeEntry("showSeconds", m_showSeconds);
 
     m_useCustomColor = ui.useCustomColor->isChecked();
+    cg.writeEntry("useCustomColor", m_useCustomColor);
     if (m_useCustomColor) {
         m_plainClockColor = ui.plainClockColor->color();
+        cg.writeEntry("plainClockColor", m_plainClockColor);
     } else {
-        updateColors();
+        m_plainClockColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
     }
 
+    m_useCustomShadowColor = ui.useCustomShadowColor->isChecked();
+    cg.writeEntry("useCustomShadowColor", m_useCustomShadowColor);
+    if (m_useCustomShadowColor) {
+        m_plainClockShadowColor = ui.plainClockShadowColor->color();
+        cg.writeEntry("plainClockShadowColor", m_plainClockShadowColor);
+    } else {
+        m_plainClockShadowColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor);
+    }
+    m_drawShadow = ui.drawShadow->isChecked();
+    cg.writeEntry("plainClockDrawShadow", m_drawShadow);
+
     m_plainClockFont.setBold(ui.plainClockFontBold->checkState() == Qt::Checked);
     m_plainClockFont.setItalic(ui.plainClockFontItalic->checkState() == Qt::Checked);
-
     cg.writeEntry("plainClockFont", m_plainClockFont);
-    cg.writeEntry("useCustomColor", m_useCustomColor);
-    cg.writeEntry("plainClockColor", m_plainClockColor);
 
     constraintsEvent(Plasma::SizeConstraint);
     update();
@@ -286,7 +311,7 @@ void Clock::clockConfigAccepted()
 
 void Clock::changeEngineTimezone(const QString &oldTimezone, const QString &newTimezone)
 {
-    m_lastTimeSeen = QTime();
+    resetLastTimeSeen();
     dataEngine("time")->disconnectSource(oldTimezone, this);
     dataEngine("time")->connectSource(newTimezone, this, updateInterval(), intervalAlignment());
 }
@@ -482,7 +507,7 @@ void Clock::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
     // Choose a relatively big font size to start with
     m_plainClockFont.setPointSizeF(qMax(m_timeRect.height(), KGlobalSettings::smallestReadableFont().pointSize()));
     preparePainter(p, m_timeRect, m_plainClockFont, fakeTimeString, true);
-    
+
     if (!m_dateString.isEmpty()) {
         if (m_dateTimezoneBesides) {
             QFontMetrics fm(m_plainClockFont);
@@ -497,6 +522,12 @@ void Clock::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
         QFont f = p->font();
         p->setFont(smallFont);
 
+        QPen datePen = p->pen();
+        QColor dateColor = m_plainClockColor;
+        dateColor.setAlphaF(0.7);
+        datePen.setColor(dateColor);
+        p->setPen(datePen);
+
         if (formFactor() == Plasma::Horizontal && (contentsRect.height() < smallFont.pointSize()*6)) {
             p->drawText(dateRect, Qt::TextSingleLine | Qt::AlignHCenter, m_dateString);
         } else {
@@ -505,11 +536,46 @@ void Clock::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
 
         p->setFont(f);
     }
-    
+
     QFontMetrics fm(p->font());
-    
-    p->drawText(QPoint(m_timeRect.center().x() - fm.width(fakeTimeString) / 2,
-                       m_timeRect.center().y() + fm.height() / 3), timeString);
+
+    QPointF timeTextOrigin(QPointF(qMax(0, (m_timeRect.center().x() - fm.width(fakeTimeString) / 2)),
+                        (m_timeRect.center().y() + fm.height() / 3)));
+    p->translate(-0.5, -0.5);
+
+    if (m_drawShadow) {
+        QPen tmpPen = p->pen();
+
+        // Paint a backdrop behind the time's text
+        qreal shadowOffset = 1.0;
+        QPen shadowPen;
+        QColor shadowColor = m_plainClockShadowColor;
+        shadowColor.setAlphaF(.4);
+        shadowPen.setColor(shadowColor);
+        p->setPen(shadowPen);
+        QPointF shadowTimeTextOrigin = QPointF(timeTextOrigin.x() + shadowOffset,
+                                               timeTextOrigin.y() + shadowOffset);
+        p->drawText(shadowTimeTextOrigin, timeString);
+
+        p->setPen(tmpPen);
+
+        // Paint the time itself with a linear translucency gradient
+        QLinearGradient gradient = QLinearGradient(QPointF(0, 0), QPointF(0, fm.height()));
+
+        QColor startColor = m_plainClockColor;
+        startColor.setAlphaF(.95);
+        QColor stopColor = m_plainClockColor;
+        stopColor.setAlphaF(.7);
+
+        gradient.setColorAt(0.0, startColor);
+        gradient.setColorAt(0.5, stopColor);
+        gradient.setColorAt(1.0, startColor);
+        QBrush gradientBrush(gradient);
+
+        QPen gradientPen(gradientBrush, tmpPen.width());
+        p->setPen(gradientPen);
+    }
+    p->drawText(timeTextOrigin, timeString);
 }
 
 QRect Clock::preparePainter(QPainter *p, const QRect &rect, const QFont &font, const QString &text, bool singleline)
@@ -554,6 +620,11 @@ void Clock::updateColors()
 {
     if (!m_useCustomColor) {
         m_plainClockColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
+    }
+    if (!m_useCustomShadowColor) {
+        m_plainClockShadowColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor);
+    }
+    if (!m_useCustomColor || !m_useCustomShadowColor) {
         update();
     }
 }
