@@ -106,6 +106,8 @@ public:
     Plasma::Location location;
     Plasma::ItemBackground *itemBackground;
     QTimer *hiddenRelayoutTimer;
+    QList<QWeakPointer<Task> > delayedRepositionTasks;
+    QTimer *repositionTimer;
 
     QSet<QString> hiddenTypes;
     QSet<QString> alwaysShownTypes;
@@ -127,6 +129,10 @@ TaskArea::TaskArea(SystemTray::Applet *parent)
     d->topLayout->setContentsMargins(0, 0, 0, 0);
     d->topLayout->setSpacing(5);
 
+    d->repositionTimer = new QTimer(this);
+    d->repositionTimer->setSingleShot(true);
+    connect(d->repositionTimer, SIGNAL(timeout()), this, SLOT(delayedReposition()));
+
     d->hiddenTasksWidget = new QGraphicsWidget(this);
     d->hiddenTasksLayout = new QGraphicsGridLayout(d->hiddenTasksWidget);
     d->hiddenTasksLayout->setHorizontalSpacing(0);
@@ -189,6 +195,21 @@ void TaskArea::syncTasks(const QList<SystemTray::Task*> &tasks)
     emit sizeHintChanged(Qt::PreferredSize);
 }
 
+void TaskArea::delayedReposition()
+{
+    foreach (const QWeakPointer<Task> &task, d->delayedRepositionTasks) {
+        //kDebug() << "checking" << task->name() << task->typeId() << d->alwaysShownTypes;
+        if (task) {
+            addWidgetForTask(task.data());
+        }
+    }
+    d->delayedRepositionTasks.clear();
+
+    checkUnhideTool();
+    d->topLayout->invalidate();
+    emit sizeHintChanged(Qt::PreferredSize);
+}
+
 void TaskArea::addTask(Task *task)
 {
     bool changedPositioning = addWidgetForTask(task);
@@ -219,6 +240,13 @@ bool TaskArea::addWidgetForTask(SystemTray::Task *task)
         return false;
     }
 
+    //Delay the reposition if the applet is under the mouse cursor
+    if (d->host->isUnderMouse()) {
+        d->delayedRepositionTasks.append(task);
+        d->repositionTimer->start(2000);
+        return false;
+    }
+
     checkVisibility(task);
     QGraphicsWidget *widget = task->widget(d->host);
 
@@ -232,29 +260,21 @@ bool TaskArea::addWidgetForTask(SystemTray::Task *task)
         return false;
     }
 
-    if (widget) {
-        //kDebug() << "widget already exists, trying to reposition it";
-        d->firstTasksLayout->removeItem(widget);
-        d->normalTasksLayout->removeItem(widget);
-        d->lastTasksLayout->removeItem(widget);
-        if (d->firstTasksLayout->count() == 0) {
-            d->topLayout->removeItem(d->firstTasksLayout);
-        }
+    //kDebug() << "widget already exists, trying to reposition it";
+    d->firstTasksLayout->removeItem(widget);
+    d->normalTasksLayout->removeItem(widget);
+    d->lastTasksLayout->removeItem(widget);
+    if (d->firstTasksLayout->count() == 0) {
+        d->topLayout->removeItem(d->firstTasksLayout);
     }
 
     //If the applet doesn't want to show FDO tasks, remove (not just hide) any of them
     //if the dbus icon has a category that the applet doesn't want to show remove it
     if (!d->host->shownCategories().contains(task->category()) && !qobject_cast<Plasma::Applet *>(widget)) {
-        if (widget) {
-            widget->deleteLater();
-        }
+        widget->deleteLater();
         return true;
     }
 
-    if (!widget) {
-        widget = task->widget(d->host);
-    }
-
     if (!d->taskForWidget.contains(widget)) {
         d->taskForWidget[widget] = task;
     }
@@ -283,70 +303,66 @@ bool TaskArea::addWidgetForTask(SystemTray::Task *task)
         connect(task, SIGNAL(changed(SystemTray::Task*)), hiddenWidget, SLOT(taskChanged(SystemTray::Task*)));
         d->hiddenTasks.insert(task, hiddenWidget);
 
-        if (widget) {
-            const int row = d->hiddenTasksLayout->rowCount();
-            widget->setParentItem(d->hiddenTasksWidget);
-            //kDebug() << "putting" << task->name() << "into" << row;
-            d->hiddenTasksLayout->setRowFixedHeight(row, 24);
-            d->hiddenTasksLayout->addItem(widget, row, 0);
-            d->hiddenTasksLayout->addItem(d->hiddenTasks.value(task), row, 1);
-        }
+        const int row = d->hiddenTasksLayout->rowCount();
+        widget->setParentItem(d->hiddenTasksWidget);
+        //kDebug() << "putting" << task->name() << "into" << row;
+        d->hiddenTasksLayout->setRowFixedHeight(row, 24);
+        d->hiddenTasksLayout->addItem(widget, row, 0);
+        d->hiddenTasksLayout->addItem(d->hiddenTasks.value(task), row, 1);
     }
 
     d->hasTasksThatCanHide = !d->hiddenTasks.isEmpty();
 
-    if (widget) {
-        if (task->hidden() == Task::NotHidden) {
-            for (int i = 0; i < d->hiddenTasksLayout->count(); ++i) {
-                if (d->hiddenTasksLayout->itemAt(i) == widget) {
-                    d->hiddenTasksLayout->removeAt(i);
-                    d->hiddenRelayoutTimer->start(250);
+    if (task->hidden() == Task::NotHidden) {
+        for (int i = 0; i < d->hiddenTasksLayout->count(); ++i) {
+            if (d->hiddenTasksLayout->itemAt(i) == widget) {
+                d->hiddenTasksLayout->removeAt(i);
+                d->hiddenRelayoutTimer->start(250);
+                break;
+            }
+        }
+
+        widget->setParentItem(this);
+        //not really pretty, but for consistency attempts to put the notifications applet always in the same position
+        if (task->typeId() == "notifications") {
+            if (d->firstTasksLayout->count() == 0) {
+                d->topLayout->insertItem(0, d->firstTasksLayout);
+            }
+
+            d->firstTasksLayout->insertItem(0, widget);
+        } else if (task->order() == SystemTray::Task::First) {
+            if (d->firstTasksLayout->count() == 0) {
+                d->topLayout->insertItem(0, d->firstTasksLayout);
+            }
+
+            d->firstTasksLayout->addItem(widget);
+        } else if (task->order() == SystemTray::Task::Normal) {
+            int insertIndex = -1;
+            for (int i = 0; i < d->normalTasksLayout->count(); ++i) {
+                QGraphicsWidget *widget = static_cast<QGraphicsWidget *>(d->normalTasksLayout->itemAt(i));
+                Task *otherTask = d->taskForWidget.value(widget);
+
+                if (task->category() == Task::UnknownCategory) {
+                    insertIndex = i;
+                    break;
+                } else if (otherTask && task->category() <= otherTask->category()) {
+                    insertIndex = i;
                     break;
                 }
             }
 
-            widget->setParentItem(this);
-            //not really pretty, but for consistency attempts to put the notifications applet always in the same position
-            if (task->typeId() == "notifications") {
-                if (d->firstTasksLayout->count() == 0) {
-                    d->topLayout->insertItem(0, d->firstTasksLayout);
-                }
-
-                d->firstTasksLayout->insertItem(0, widget);
-            } else if (task->order() == SystemTray::Task::First) {
-                if (d->firstTasksLayout->count() == 0) {
-                    d->topLayout->insertItem(0, d->firstTasksLayout);
-                }
-
-                d->firstTasksLayout->addItem(widget);
-            } else if (task->order() == SystemTray::Task::Normal) {
-                int insertIndex = -1;
-                for (int i = 0; i < d->normalTasksLayout->count(); ++i) {
-                    QGraphicsWidget *widget = static_cast<QGraphicsWidget *>(d->normalTasksLayout->itemAt(i));
-                    Task *otherTask = d->taskForWidget.value(widget);
-
-                    if (task->category() == Task::UnknownCategory) {
-                        insertIndex = i;
-                        break;
-                    } else if (otherTask && task->category() <= otherTask->category()) {
-                        insertIndex = i;
-                        break;
-                    }
-                }
-
-                if (insertIndex == -1) {
-                    insertIndex = d->normalTasksLayout->count();
-                }
-
-                d->normalTasksLayout->insertItem(insertIndex, widget);
-            } else {
-                d->lastTasksLayout->insertItem(0, widget);
+            if (insertIndex == -1) {
+                insertIndex = d->normalTasksLayout->count();
             }
-        }
 
-        widget->show();
+            d->normalTasksLayout->insertItem(insertIndex, widget);
+        } else {
+            d->lastTasksLayout->insertItem(0, widget);
+        }
     }
 
+    widget->show();
+
     //the applet could have to be repainted due to easement change
     QTimer::singleShot(0, this, SLOT(delayedAppletUpdate()));
     return true;
@@ -375,6 +391,9 @@ void TaskArea::removeTask(Task *task)
             }
         }
         d->hiddenTasks.remove(task);
+        d->hiddenTasksLayout->invalidate();
+        d->hiddenTasksLayout->updateGeometry();
+        d->hiddenTasksWidget->resize(d->hiddenTasksWidget->effectiveSizeHint(Qt::MinimumSize));
     }
     d->hasTasksThatCanHide = !d->hiddenTasks.isEmpty();
 
@@ -392,6 +411,7 @@ void TaskArea::removeTask(Task *task)
         d->taskForWidget.remove(widget);
         d->taskCategories.remove(task);
     }
+
     relayout();
 }
 
@@ -494,6 +514,7 @@ void TaskArea::initUnhideTool()
 
 void TaskArea::setUnhideToolIconSizes()
 {
+    d->unhider->setPreferredIconSize(QSize(16,16));
     if (d->topLayout->orientation() == Qt::Horizontal) {
         d->unhider->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
     } else {
@@ -518,31 +539,31 @@ void TaskArea::updateUnhideToolIcon()
     switch(d->location) {
     case Plasma::LeftEdge:
         if (d->showingHidden) {
-            d->unhider->setSvg("widgets/systemtray", "expander-left");
+            d->unhider->setSvg("widgets/arrows", "left-arrow");
         } else {
-            d->unhider->setSvg("widgets/systemtray", "expander-right");
+            d->unhider->setSvg("widgets/arrows", "right-arrow");
         }
         break;
     case Plasma::RightEdge:
         if (d->showingHidden) {
-            d->unhider->setSvg("widgets/systemtray", "expander-right");
+            d->unhider->setSvg("widgets/arrows", "right-arrow");
         } else {
-            d->unhider->setSvg("widgets/systemtray", "expander-left");
+            d->unhider->setSvg("widgets/arrows", "left-arrow");
         }
         break;
     case Plasma::TopEdge:
         if (d->showingHidden) {
-            d->unhider->setSvg("widgets/systemtray", "expander-up");
+            d->unhider->setSvg("widgets/arrows", "up-arrow");
         } else {
-            d->unhider->setSvg("widgets/systemtray", "expander-down");
+            d->unhider->setSvg("widgets/arrows", "down-arrow");
         }
         break;
     case Plasma::BottomEdge:
     default:
         if (d->showingHidden) {
-            d->unhider->setSvg("widgets/systemtray", "expander-down");
+            d->unhider->setSvg("widgets/arrows", "down-arrow");
         } else {
-            d->unhider->setSvg("widgets/systemtray", "expander-up");
+            d->unhider->setSvg("widgets/arrows", "up-arrow");
         }
     }
 }
