@@ -19,6 +19,7 @@
 #include <QPropertyAnimation>
 #include <QTimer>
 
+#include <KDebug>
 #include <KDirSelectDialog>
 #include <KDirWatch>
 #include <KFileDialog>
@@ -38,6 +39,7 @@ K_EXPORT_PLASMA_WALLPAPER(image, Image)
 
 Image::Image(QObject *parent, const QVariantList &args)
     : Plasma::Wallpaper(parent, args),
+      m_fileWatch(new KDirWatch(this)),
       m_configWidget(0),
       m_wallpaperPackage(0),
       m_currentSlide(-1),
@@ -52,6 +54,8 @@ Image::Image(QObject *parent, const QVariantList &args)
     connect(this, SIGNAL(renderCompleted(QImage)), this, SLOT(updateBackground(QImage)));
     connect(this, SIGNAL(urlDropped(KUrl)), this, SLOT(setWallpaper(KUrl)));
     connect(&m_timer, SIGNAL(timeout()), this, SLOT(nextSlide()));
+    connect(m_fileWatch, SIGNAL(dirty(QString)), this, SLOT(imageFileAltered(QString)));
+    connect(m_fileWatch, SIGNAL(created(QString)), this, SLOT(imageFileAltered(QString)));
 }
 
 Image::~Image()
@@ -105,6 +109,7 @@ void Image::init(const KConfigGroup &config)
         actions.push_back(m_nextWallpaperAction);
         actions.push_back(m_openImageAction);
         setContextualActions(actions);
+        updateWallpaperActions();
     }
 
     m_animation = new QPropertyAnimation(this, "fadeValue");
@@ -190,15 +195,27 @@ QWidget* Image::createConfigurationInterface(QWidget* parent)
         m_uiSlideshow.setupUi(m_configWidget);
         m_uiSlideshow.m_newStuff->setIcon(KIcon("get-hot-new-stuff"));
         m_uiSlideshow.m_dirlist->clear();
+        m_uiSlideshow.m_systemCheckBox->setChecked(false);
+        m_uiSlideshow.m_downloadedCheckBox->setChecked(false);
+
+        QString systemPath = KStandardDirs::installPath("wallpaper");
+        QString localPath = KGlobal::dirs()->saveLocation("wallpaper");
+
         foreach (const QString &dir, m_dirs) {
-            m_uiSlideshow.m_dirlist->addItem(dir);
+            if (dir == KStandardDirs::installPath("wallpaper")) {
+                m_uiSlideshow.m_systemCheckBox->setChecked(true);
+            } else if (dir == localPath) {
+                m_uiSlideshow.m_downloadedCheckBox->setChecked(true);
+            } else {
+                m_uiSlideshow.m_dirlist->addItem(dir);
+            }
         }
         m_uiSlideshow.m_dirlist->setCurrentRow(0);
         updateDirs();
         m_uiSlideshow.m_addDir->setIcon(KIcon("list-add"));
-        connect(m_uiSlideshow.m_addDir, SIGNAL(clicked()), this, SLOT(slotAddDir()));
+        connect(m_uiSlideshow.m_addDir, SIGNAL(clicked()), this, SLOT(addDir()));
         m_uiSlideshow.m_removeDir->setIcon(KIcon("list-remove"));
-        connect(m_uiSlideshow.m_removeDir, SIGNAL(clicked()), this, SLOT(slotRemoveDir()));
+        connect(m_uiSlideshow.m_removeDir, SIGNAL(clicked()), this, SLOT(removeDir()));
 
         QTime time(0, 0, 0);
         time = time.addSecs(m_delay);
@@ -226,6 +243,10 @@ QWidget* Image::createConfigurationInterface(QWidget* parent)
         connect(m_uiSlideshow.m_color, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
         connect(m_uiSlideshow.m_newStuff, SIGNAL(clicked()), this, SLOT(getNewWallpaper()));
 
+        connect(m_uiSlideshow.m_systemCheckBox, SIGNAL(toggled(bool)),
+                this, SLOT(systemCheckBoxToggled(bool)));
+        connect(m_uiSlideshow.m_downloadedCheckBox, SIGNAL(toggled(bool)),
+                this, SLOT(downloadedCheckBoxToggled(bool)));
         connect(m_uiSlideshow.m_color, SIGNAL(changed(const QColor&)), this, SLOT(modified()));
         connect(m_uiSlideshow.m_resizeMethod, SIGNAL(currentIndexChanged(int)), this, SLOT(modified()));
         connect(m_uiSlideshow.m_addDir, SIGNAL(clicked()), this, SLOT(modified()));
@@ -238,20 +259,35 @@ QWidget* Image::createConfigurationInterface(QWidget* parent)
     return m_configWidget;
 }
 
+void Image::systemCheckBoxToggled(bool checked)
+{
+    if (checked) {
+        m_dirs << KStandardDirs::installPath("wallpaper");
+    } else {
+        m_dirs.removeAll(KStandardDirs::installPath("wallpaper"));
+    }
+    modified();
+}
+
+void Image::downloadedCheckBoxToggled(bool checked)
+{
+    if (checked) {
+        m_dirs << KGlobal::dirs()->saveLocation("wallpaper");
+    } else {
+        m_dirs.removeAll(KGlobal::dirs()->saveLocation("wallpaper"));
+    }
+    modified();
+}
+
 void Image::setConfigurationInterfaceModel()
 {
-    QTime t;
-    t.start();
     m_uiImage.m_view->setModel(m_model);
-    kDebug() << t.restart();
     connect(m_uiImage.m_view->selectionModel(), SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(pictureChanged(const QModelIndex &)));
 
     QModelIndex index = m_model->indexOf(m_wallpaper);
-    kDebug() << t.restart();
     if (index.isValid()) {
         m_uiImage.m_view->setCurrentIndex(index);
     }
-    kDebug() << m_wallpaper << index << t.restart();
 }
 
 void Image::modified()
@@ -316,21 +352,28 @@ void Image::timeChanged(const QTime& time)
     }
 }
 
-void Image::slotAddDir()
+void Image::addDir()
 {
     KUrl empty;
-    KDirSelectDialog dialog(empty, true, m_configWidget);
-    if (dialog.exec()) {
-        QString urlDir = dialog.url().path();
+    KDirSelectDialog *dialog = new KDirSelectDialog(empty, true, m_configWidget);
+    connect(dialog, SIGNAL(finished()), this, SLOT(addDirFromSelectionDialog()));
+    dialog->show();
+}
+
+void Image::addDirFromSelectionDialog()
+{
+    KDirSelectDialog *dialog = qobject_cast<KDirSelectDialog *>(sender());
+    if (dialog) {
+        QString urlDir = dialog->url().path();
         if (!urlDir.isEmpty() && m_uiSlideshow.m_dirlist->findItems(urlDir, Qt::MatchExactly).isEmpty()) {
-            m_uiSlideshow.m_dirlist->addItem(dialog.url().path());
+            m_uiSlideshow.m_dirlist->addItem(urlDir);
             updateDirs();
             startSlideshow();
         }
     }
 }
 
-void Image::slotRemoveDir()
+void Image::removeDir()
 {
     int row = m_uiSlideshow.m_dirlist->currentRow();
     if (row != -1) {
@@ -343,6 +386,14 @@ void Image::slotRemoveDir()
 void Image::updateDirs()
 {
     m_dirs.clear();
+
+    if (m_uiSlideshow.m_systemCheckBox->isChecked()) {
+        m_dirs << KStandardDirs::installPath("wallpaper");
+    }
+    if (m_uiSlideshow.m_downloadedCheckBox->isChecked()) {
+        m_dirs << KGlobal::dirs()->saveLocation("wallpaper");
+    }
+
     const int dirCount = m_uiSlideshow.m_dirlist->count();
     for (int i = 0; i < dirCount; ++i) {
         m_dirs.append(m_uiSlideshow.m_dirlist->item(i)->text());
@@ -358,12 +409,26 @@ void Image::setSingleImage()
     }
 
     QString img;
-    Plasma::Package b(m_wallpaper, packageStructure(this));
-    img = b.filePath("preferred");
-    kDebug() << img << m_wallpaper;
 
-    if (img.isEmpty()) {
-        img = m_wallpaper;
+    if (QDir::isAbsolutePath(m_wallpaper)) {
+        Plasma::Package b(m_wallpaper, packageStructure(this));
+        img = b.filePath("preferred");
+        kDebug() << img << m_wallpaper;
+
+        if (img.isEmpty()) {
+            img = m_wallpaper;
+        }
+    } else {
+        //if it's not an absolute path, check if it's just a wallpaper name
+        const QString path = KStandardDirs::locate("wallpaper", m_wallpaper + "/metadata.desktop");
+
+        if (!path.isEmpty()) {
+            QDir dir(path);
+            dir.cdUp();
+
+            Plasma::Package b(dir.path(), packageStructure(this));
+            img = b.filePath("preferred");
+        }
     }
 
     if (!m_size.isEmpty()) {
@@ -399,9 +464,8 @@ void Image::setWallpaper(const QString &path)
     if (m_wallpaper.isEmpty()) {
         m_slideshowBackgrounds.append(path);
         m_currentSlide = m_slideshowBackgrounds.size() - 2;
-        bool random = m_randomize;
         nextSlide();
-        m_randomize = random;
+        updateWallpaperActions();
     } else {
         m_wallpaper = path;
         setSingleImage();
@@ -419,6 +483,7 @@ void Image::startSlideshow()
     m_slideshowBackgrounds.clear();
     m_slideshowBackgrounds = BackgroundListModel::findAllBackgrounds(this, 0, m_dirs);
 
+    updateWallpaperActions();
     // start slideshow
     if (m_slideshowBackgrounds.isEmpty()) {
         m_pixmap = QPixmap();
@@ -430,6 +495,17 @@ void Image::startSlideshow()
     }
 }
 
+void Image::updateWallpaperActions()
+{
+    if (m_nextWallpaperAction) {
+        m_nextWallpaperAction->setEnabled(!m_slideshowBackgrounds.isEmpty());
+    }
+
+    if (m_openImageAction) {
+        m_openImageAction->setEnabled(!m_slideshowBackgrounds.isEmpty());
+    }
+}
+
 void Image::getNewWallpaper()
 {
     if (!m_newStuffDialog) {
@@ -463,7 +539,7 @@ void Image::pictureChanged(const QModelIndex &index)
         return;
     }
 
-    if (b->structure()->contentsPrefix().isEmpty()) {
+    if (b->structure()->contentsPrefixPaths().isEmpty()) {
         // it's not a full package, but a single paper
         m_wallpaper = b->filePath("preferred");
     } else {
@@ -493,7 +569,12 @@ void Image::positioningChanged(int index)
 void Image::showFileDialog()
 {
     if (!m_dialog) {
-        m_dialog = new KFileDialog(KUrl(), "*.png *.jpeg *.jpg *.xcf *.svg *.svgz *.bmp", m_configWidget);
+        KUrl baseUrl;
+        if(m_wallpaper.indexOf(QDir::homePath()) > -1){
+            baseUrl = KUrl(m_wallpaper);
+        }
+
+        m_dialog = new KFileDialog(baseUrl, "*.png *.jpeg *.jpg *.xcf *.svg *.svgz *.bmp", m_configWidget);
         m_dialog->setOperationMode(KFileDialog::Opening);
         m_dialog->setInlinePreviewShown(true);
         m_dialog->setCaption(i18n("Select Wallpaper Image File"));
@@ -601,6 +682,10 @@ void Image::nextSlide()
 
 void Image::openSlide()
 {
+    if (!m_wallpaperPackage) {
+        return;
+    }
+
     // open in image viewer
     KUrl filepath(m_wallpaperPackage->filePath("preferred"));
     kDebug() << "opening file " << filepath.path();
@@ -609,6 +694,10 @@ void Image::openSlide()
 
 void Image::renderWallpaper(const QString& image)
 {
+    if (!m_img.isEmpty()) {
+        m_fileWatch->removeFile(m_img);
+    }
+
     if (!image.isEmpty()) {
         m_img = image;
     }
@@ -617,9 +706,19 @@ void Image::renderWallpaper(const QString& image)
         return;
     }
 
+    m_fileWatch->addFile(m_img);
     render(m_img, m_size, m_resizeMethod, m_color);
 }
 
+void Image::imageFileAltered(const QString &path)
+{
+    if (path == m_img) {
+        renderWallpaper(path);
+    } else {
+        // somehow this got added to the dirwatch, but we don't care about it anymore
+        m_fileWatch->removeFile(path);
+    }
+}
 void Image::updateBackground(const QImage &img)
 {
     m_oldPixmap = m_pixmap;
@@ -670,13 +769,15 @@ void Image::setFadeValue(qreal value)
     emit update(boundingRect());
 }
 
+//FIXME: we have to save the configuration also when the dialog cancel button is clicked.
 void Image::removeWallpaper(QString name)
 {
     int wallpaperIndex = m_usersWallpapers.indexOf(name);
     if (wallpaperIndex >= 0){
         m_usersWallpapers.removeAt(wallpaperIndex);
         m_model->reload(m_usersWallpapers);
-        emit settingsChanged(true);
+        //TODO: save the configuration in the right way
+	emit settingsChanged(true);
     }
 }
 
