@@ -23,6 +23,7 @@
 
 class QPropertyAnimation;
 
+class KDirWatch;
 class KFileDialog;
 class KJob;
 
@@ -54,8 +55,8 @@ class Image : public Plasma::Wallpaper
         void removeWallpaper(QString name);
         void timeChanged(const QTime& time);
         void positioningChanged(int index);
-        void slotAddDir();
-        void slotRemoveDir();
+        void addDir();
+        void removeDir();
         void getNewWallpaper();
         void colorChanged(const QColor& color);
         void pictureChanged(const QModelIndex &);
@@ -78,6 +79,10 @@ class Image : public Plasma::Wallpaper
         void newStuffFinished();
         void setConfigurationInterfaceModel();
         void updateDirs();
+        void addDirFromSelectionDialog();
+        void systemCheckBoxToggled(bool);
+        void downloadedCheckBoxToggled(bool);
+        void imageFileAltered(const QString &path);
 
     protected:
         void init(const KConfigGroup &config);
@@ -85,6 +90,7 @@ class Image : public Plasma::Wallpaper
         void suspendStartup(bool suspend); // for ksmserver
         void calculateGeometry();
         void setSingleImage();
+        void updateWallpaperActions();
 
     private:
         static bool s_startupResumed;
@@ -96,6 +102,7 @@ class Image : public Plasma::Wallpaper
         QString m_wallpaper;
         QColor m_color;
         QStringList m_usersWallpapers;
+        KDirWatch *m_fileWatch;
 
         QWidget* m_configWidget;
         Ui::ImageConfig m_uiImage;
